
from FITS import *
from FITStable import *
from ReadSex import *
from masks import *
from VTKHelperFunctions import *
from basics import *

WFSCALE = 0.05

def rebin( d, dxa, dya, data=1, cubic=0, verb=1, constant=0, interp=1):
    if verb: print "Transforming."
    if data:
       y,x = indices(dxa.shape)
       g = VTKGradMag(y+dya,1)*VTKGradMag(x+dxa,1)
    t = VTKImageTransform(d,dxa,dya,numret=1,interp=interp,cubic=cubic,constant=constant)
    if data: t=t*g
    return t

def reversebin( d, dxa, dya, origsize=None, cubic=0, data=1, word="coordinates",verb=1,constant=0.0,interp=1):
    if verb:
       if origsize: print "Reverse Transforming to size:",origsize
       else: print "Reverse Transforming %s." % (word)
    y,x = indices(dxa.shape)
    if data:
       g = VTKGradMag(y+dya,1)*VTKGradMag(x+dxa,1)
       d = divz(d,g)
    t = VTKImageTransform(d,dxa.astype(Float32),dya.astype(Float32),numret=1,reverse=1,origsize=origsize,interp=interp,cubic=cubic,constant=constant)
    return t

def randback(F):
   bg = F["BACKGRND"]
#   if bg == "N/A":
#      data = F.data().flat
#      data = compress(not_equal(data,0),data)
#      bg = GaussianBG(data.flat,199,verb=0)
   return bg
         
def ParseHeader(l):
   h = l[1:-2].lower()
   k = map(eval,split(h))
   return k

def ParseListing(H,L):
   D = {}
   for w in H: D[w] = []
   for k in H:
      D[k] = []
   for l in L:
     if l[0] != "#":
       K = l.split()
       k = l.lower().split()
       v = []
       p = []
       a = 0
       for j in range(len(k)):
         if not (k[j].count("'") % 2) and not a:
             if not j: v.append(K[j])
             else:
                v.append( k[j])
                p.append( k[j].count("'"))
         else:
            if not a:
               v.append(k[j])
               a = 1
            else:
               v[-1]=join([v[-1],k[j]])
               a = (a + k[j].count("'")) % 2
       for j in range(len(v)):
         if v[j].count("'") and not v[j].count("'") % 2: v[j] = eval(v[j]).strip()
         D[H[j]].append(v[j])
   return D

def ReadListing(f):
   F = open(f,"r")
   L = F.readlines()
   H = ParseHeader(L[1])
   D = ParseListing(H,L[3:])
   F.close()
   return D

class IDC:
   def __init__(self,idctab,filter1,filter2,chip,verb=1):
      if verb: print "Reading %s for filters %s and %s." % (idctab,filter1,filter2)
      descr,table = readFITStable(idctab,1,verb=0)
      k1 = array([(f1.strip() == filter1) for f1 in table["FILTER1"]])
      k2 = array([(f2.strip() == filter2) for f2 in table["FILTER2"]])
      keep = k1*k2
      for k in table.keys():
         if type(table[k]) == ArrayType: table[k] = compress(keep,table[k])
         else: table[k] = [table[k][j] for j in nonzero(keep)]
      self.filter1 = filter1
      self.filter2 = filter2
      self.idc = table
      self.chip = chip
      self.GetCoeffs()

   def GetCoeffs(self):
      u = equal(self.idc["DETCHIP"],self.chip)
      nu = nonzero( u)[0]
      fu = u*equal([(self.idc["DIRECTION"][j].strip() == "FORWARD") for j in range(len(u))],1)
      iu = u*equal([(self.idc["DIRECTION"][j].strip() == "INVERSE") for j in range(len(u))],1)
      fl = nonzero(fu)[0]
      il = nonzero(iu)[0]
      ij = [[(1.0*j,1.0*i) for j in range(i+1)] for i in range(1,5)]
      self.ij = []
      self.fcxs = []
      self.fcys = []
      self.icxs = []
      self.icys = []
      fcxs = [[self.idc["CX%d%d" % (i,j)][fl] for j in range(i+1)] for i in range(1,5)]
      fcys = [[self.idc["CY%d%d" % (i,j)][fl] for j in range(i+1)] for i in range(1,5)]
      icxs = [[self.idc["CX%d%d" % (i,j)][il] for j in range(i+1)] for i in range(1,5)]
      icys = [[self.idc["CY%d%d" % (i,j)][il] for j in range(i+1)] for i in range(1,5)]
      for ij1 in ij: self.ij += ij1
      self.ij = transpose(self.ij)
      for fcxs1 in fcxs: self.fcxs += fcxs1
      for fcys1 in fcys: self.fcys += fcys1
      for icxs1 in icxs: self.icxs += icxs1
      for icys1 in icys: self.icys += icys1
      self.fcxs = array( self.fcxs)
      self.fcys = array( self.fcys)
      self.icxs = array( self.icxs)
      self.icys = array( self.icys)
      self.xref = self.idc["XREF"][nu]
      self.yref = self.idc["YREF"][nu]
      self.v2ref = self.idc["V2REF"][nu]
      self.v3ref = self.idc["V3REF"][nu]
      self.scale = self.idc["SCALE"][nu]

   def ToSky(self,xi,yi):
      i,j = self.ij
      aij = self.fcxs
      bij = self.fcys
      xi = ((xi + 1) - self.xref)
      yi = ((yi + 1) - self.yref)
      xc, yc = 0.0, 0.0
      for l in range(len(i)):
         pp = power(xi,i[l])*power(yi,j[l]-i[l])
         xc += aij[l]*pp
         yc += bij[l]*pp
      xc = xc + self.v2ref
      yc = -yc + self.v3ref
      return xc,yc

   def FromSky(self,xs,ys):
      i,j = self.ij
      aij = self.icxs
      bij = self.icys
      xi, yi = 0.0, 0.0
      xs = (xs - self.v2ref)
      ys = -(ys - self.v3ref)
      for l in range(len(i)):
         pp = power(xs,i[l])*power(ys,j[l]-i[l])
         xi += aij[l]*pp
         yi += bij[l]*pp
      xi = (xi + self.xref) - 1
      yi = (yi + self.yref) - 1
      return xi,yi

class ACSImage:
   def __init__(self,image,idcs,include=True):
      self.image = image
      self.include = include
      self.spt = self.image.replace("_flt","_spt")
      self.catalogs = [self.image.replace("_flt.fits","_fltcr1.cat"), self.image.replace("_flt.fits","_fltcr4.cat")]
      self.idcs = idcs
      self.frames = [2,5]
      self.idcs = []
      self.FITS = FITS(self.image)
      self.exptime = self.FITS["exptime"]
      for c in self.frames:
        self.FITS.goto(c)
        self.idcs.append(idcs[self.FITS["ccdchip"]])
        self.ra_ap = self.FITS["ra_aper"]
        self.dec_ap = self.FITS["dec_aper"]
        self.pa_ap = self.FITS["pa_aper"]
      self.FITS.goto(1)
      fspt = FITS(self.spt)
      self.ra_v1 = fspt["ra_v1"]
      self.dec_v1 = fspt["dec_v1"]
      self.pa_v1 = fspt["pa_v3"]
      self.aperoffx = fspt["aperoffx"]
      self.aperoffy = fspt["aperoffy"]
      self.v2aperce = fspt["v2aperce"]
      self.v3aperce = fspt["v3aperce"]
      self.apver1v2 = fspt["apver1v2"]
      self.apver1v3 = fspt["apver1v3"]
      self.hosv2cnt = fspt["hosv2cnt"]
      self.hosv3cnt = fspt["hosv3cnt"]
      fspt.close()
#      print self.aperoffx, self.aperoffy
#      print self.v2aperce, self.v3aperce
#      print self.apver1v2, self.apver1v3
#      print self.hosv2cnt, self.hosv3cnt
      self.uoff = 0.0
      self.voff = 0.0
      self.pa_v1r = degrad(self.pa_v1)
      self.ra_v1r = degrad(self.ra_v1)
      self.dec_v1r = degrad(self.dec_v1)
      self.pa_apr = degrad(self.pa_ap)
      self.ra_apr = degrad(self.ra_ap)
      self.dec_apr = degrad(self.dec_ap)
      self.ra_image = self.ra_apr
      self.dec_image = self.dec_apr
      self.pa_image = (-self.pa_v1r + pi)
      self.cpa = cos(self.pa_image)
      self.spa = sin(self.pa_image)
      self.icpa = cos(-self.pa_image)
      self.ispa = sin(-self.pa_image)
      self.ras = formatSexagesimal(self.ra_ap/15,4,3600000)
      self.decs = formatSexagesimal(self.dec_ap,4,3600000)
      self.pas = formatSexagesimal(self.pa_ap,4,3600000)
#      print "%15s %15s %15s %15s" % (self.image,self.ras,self.decs,self.pas)
      self.FITS.close()

   def __repr__(self): return os.path.basename(self.image)

#   def __del__(self):
#      self.FITS.close()

   def WriteSoln(self):
      cra,cdec = self.cx,self.cy
      self.FITS = FITS(self.image)
      for chip in [0,1]:
         c = self.frames[chip]
         self.FITS.goto(c)
         naxis1 = self.FITS["naxis1"]
         naxis2 = self.FITS["naxis2"]
         yco, xco = indices((16,16),Float32)
         xco = (xco/16.0 * naxis1).flat
         yco = (yco/16.0 * naxis2).flat
         skyx,skyy = self.idcs[chip].ToSky(xco,yco)
         ras,decs = self.ToSky(chip,skyx,skyy)
         u,v = normalcoord(ras,decs,cra,cdec)
         xco = xco - naxis1/2.0
         yco = yco - naxis2/2.0
         ab = abasis(3,xco,yco)
         uv = concatenate([u,v])
         sol = svdfit(ab,uv)
         soln = self.FITS.name().replace(".fits","cr%d.soln" % ([1,4][chip]))
         print "solution %s rms=%f" % (soln,rms(dot(sol,transpose(ab)),uv)*3600*180/pi)
         f = open(soln,"w")
         print >> f, len(sol), 0, naxis1/2.0, naxis2/2.0
         print >> f, cra, cdec
         for s in sol: print >> f, s
         f.close()
      self.FITS.close()

   def CatsToSky(self,cra,cdec):
      self.su, self.sv = [], []
      self.cx, self.cy = self.ra_image, self.dec_image
      for chip in [0,1]:
         cf = self.catalogs[chip]
         cd = readsex(cf)[0]
         cx = asarray(cd["X_IMAGE"])-1
         cy = asarray(cd["Y_IMAGE"])-1
         cu = cd["FLAGS"]
         cs = cd["CLASS_STAR"]
#         use = less(cu,4)
#         cx,cy = compress(use,[cx,cy],1)
         idc = self.idcs[chip]
         skyx,skyy = self.idcs[chip].ToSky(cx,cy)
         ras,decs = self.ToSky(chip,skyx,skyy)
         self.su.append(ras)
         self.sv.append(decs)
      self.su = concatenate(self.su)
      self.sv = concatenate(self.sv)

   def GetOffset(self,ref=None,refu=None,refv=None,calc=0,tol=3.0,jtol=3.0,verb=0):
      if ref: refu,refv = ref.su, ref.sv
      good1 = self.GetPixelPositions(0,refu,refv)[2]
      good2 = self.GetPixelPositions(1,refu,refv)[2]
      good = logical_or(good1,good2)
      refu,refv = compress(good,[refu,refv],1)
      if ref: cra,cdec = ref.cx,ref.cy
      else: cra,cdec = self.cx,self.cy
      refu,refv = normalcoord(refu,refv,cra,cdec,conv=3600*180/pi)
      su,sv=self.su,self.sv
      su,sv = normalcoord(su,sv,cra,cdec,conv=3600*180/pi)
      mu,uoff,urms,voff,vrms = GetOffset(su,sv,refu,refv,tol=tol,jtol=jtol,verb=verb)
      if calc:
         self.uoff = self.uoff + uoff
         self.voff = self.voff + voff
         fmt = "%s (%3d) = %7.3f +-%6.3f %7.3f +-%6.3f"
         print fmt % (self,len(mu),self.uoff,urms,self.voff,vrms)
      elif ref and len(mu):
         fmt = "[%s,%s] (%3d) = %7.3f +-%6.3f %7.3f +-%6.3f"
         print fmt % (self,ref,len(mu),uoff,urms,voff,vrms)
      return len(mu),uoff,urms,voff,vrms

   def ReadChip(self,chip):
      self.FITS = FITS(self.image)
      c = self.frames[chip]
      self.FITS.goto(c)
      D = F.data()
      return D
      self.FITS.close()

   def GetCorners(self,cra,cdec):
      self.bgs = [None,None]
      self.ccors = []
      self.xcors = []
      self.ycors = []
      self.x0cors = []
      self.y0cors = []
      self.naxis1s = []
      self.naxis2s = []
      self.FITS = FITS(self.image)
      for chip in range(2):
         c = self.frames[chip]
         self.FITS.goto(c)
         naxis1 = self.FITS["naxis1"]
         naxis2 = self.FITS["naxis2"]
         self.naxis1s.append(naxis1)
         self.naxis2s.append(naxis2)
         self.x0cors, self.y0cors = transpose([(xi,yi) for xi in [0,naxis1-1] for yi in [0,naxis2-1]])
         xyc = [self.idcs[chip].ToSky(xi,yi) for xi in [0,naxis1-1] for yi in [0,naxis2-1]]
         rds = [self.ToSky(chip,xyc[j][0],xyc[j][1]) for j in range(4)]
         self.xcor,self.ycor = transpose([normalcoord(rds[j][0],rds[j][1],cra,cdec,conv=3600*180/pi) for j in range(4)])
         self.xcors.append(self.xcor)
         self.ycors.append(self.ycor)
         self.ccors.append(chip)
      self.FITS.close()

   def GetBackground(self,chip):
      print "Getting background for chip %d of %s." % (chip,self.image)
      self.bgs[chip] = randback(self.FITS)
      print "Found background of %.1f in chip %d of %s." % (self.bgs[chip],chip,self.image)

   def ToSky(self,chip,xc,yc):
       xc = xc - self.hosv2cnt
       yc = yc - self.hosv3cnt
       xs, ys = -self.cpa*xc + self.spa*yc, - self.cpa*yc - self.spa*xc
       xs = xs + self.uoff
       ys = ys + self.voff
       xr, yr = array([xs,ys])/3600.0 * pi/180
       rs,ds = equicoord(xr,yr,self.ra_image,self.dec_image,text=0)
       return rs,ds

   def GetPixelPositions(self,chip,ras,decs):
      xs,ys = normalcoord(ras,decs,self.ra_image,self.dec_image,conv=3600*180/pi)
      xs = xs - self.uoff
      ys = ys - self.voff
      xr,yr = -self.icpa*xs + self.ispa*ys,  -self.icpa*ys - self.ispa*xs
      xr = xr + self.hosv2cnt
      yr = yr + self.hosv3cnt
      idc = self.idcs[chip]
      xi ,yi = idc.FromSky(xr,yr)
      gx = between(xi,0,self.naxis1s[chip],inc=1).astype(Float32)
      gy = between(yi,0,self.naxis2s[chip],inc=1).astype(Float32)
      good = gx*gy
      return xi, yi, good.astype(Float32)

   def GetSquare(self,chip,xlo,xhi,ylo,yhi,cra,cdec,buffer=0):
      self.FITS = FITS(self.image)
      c = self.frames[chip]
      self.FITS.goto(c)
      print  "Reading %s." % (self.image),;sys.stdout.flush()
      if self.bgs[chip] == None: self.GetBackground(chip)
      bg = self.bgs[chip]
      xlo = max([0,xlo-buffer])
      xhi = min([xhi+buffer,self.FITS["naxis1"]])
      ylo = max([0,ylo-buffer])
      yhi = min([yhi+buffer,self.FITS["naxis2"]])
      data = self.FITS.readsec([xlo,ylo],[xhi,yhi])
      self.FITS.goto(c+1)
      sig = self.FITS.readsec([xlo,ylo],[xhi,yhi])
      self.FITS.goto(c+2)
      dq = self.FITS.readsec([xlo,ylo],[xhi,yhi],vnul=0)
      self.FITS.close()
      return (data,bg,sig,dq,xlo,ylo)

def GetOffset(su,sv,refu,refv,verb=1,tol=5.0,jtol=3.0):
    utot,vtot = 0.0, 0.0
    mu,utot,urms,vtot,vrms = [array([])]+[0.0 for j in range(4)]
    if refu.shape[0]:
        for i in range(10):
           if i:
               mru,mrv = take([refu,refv],oo,1)
               msu,msv = take([su,sv],kk,1)
               mu = mru - msu
               mv = mrv - msv
               ds = sqrt(mu**2+mv**2)
               oo,kk = compress(less(ds,tol),[oo,kk],1)
               mru,mrv = take([refu,refv],oo,1)
               msu,msv = take([su,sv],kk,1)
           else:
               oo,kk = [], []
               for k in range(su.shape[0]):
                  du = refu - su[k]
                  dv = refv - sv[k]
                  dr = sqrt(du**2+dv**2)
                  o = argmin(dr)
                  if dr[o] < tol:
                     oo.append(o)
                     kk.append(k)
               mru,mrv = take([refu,refv],oo,1)
               msu,msv = take([su,sv],kk,1)
           mu = mru - msu
           mv = mrv - msv
           uoff,urms = bwt(mu)
           voff,vrms = bwt(mv)
           ma = arctan2(mv,mu)*180/pi
           utot = utot + uoff
           vtot = vtot + voff
           su = su + uoff
           sv = sv + voff
           if verb:
              print "%3d %9.5f %4d" % (i+1,tol,len(mu)),
              print "%9.5f %9.5f" % (uoff,urms),
              print "%9.5f %9.5f" % (voff,vrms)
           if sqrt(uoff**2 + voff**2) < 1e-4:
               if jtol > 0.9: jtol = jtol - 0.5
               elif jtol > 0.4: jtol = jtol - 0.2
           if sqrt(uoff**2 + voff**2) < 1e-4 and i > 10: break
           tol = max([jtol*sqrt(urms**2 + vrms**2),0.5])
    return mu,utot,urms,vtot,vrms

def AdjustPositions(images,cra,cdec,usecats,astro):
    [image.GetCorners(cra,cdec) for image in images]
    if usecats:
       limages = len(images)
       rimages = range(limages)
       [image.CatsToSky(cra,cdec) for image in images]
       if astro:
          simdat = readsim(astro)[0]
          simra = asarray([hrrad(scanSexagesimal(ra1)) for ra1 in simdat[3]])
          simdec = asarray([degrad(scanSexagesimal(dec1)) for dec1 in simdat[4]])
          rematch1d = [images[i].GetOffset(refu=simra,refv=simdec,tol=3.0,jtol=2.0,verb=0,calc=1) for i in rimages]
          [image.CatsToSky(cra,cdec) for image in images]
          for tpass in range(2):
              rematch1d = [images[i].GetOffset(ref=images[j],tol=[0.5,0.5][tpass],jtol=[3.0,3.0][tpass],\
                 verb=0,calc=0) for i in rimages for j in rimages if i > j]
              i1d = [i for i in rimages for j in rimages if i > j]
              j1d = [j for i in rimages for j in rimages if i > j]
              nms1d = [r[0] for r in rematch1d]
              ruoff1d = [r[1] for r in rematch1d]
              suoff1d = [r[2] for r in rematch1d]
              rvoff1d = [r[3] for r in rematch1d]
              svoff1d = [r[4] for r in rematch1d]
              nms = zeros((limages,limages))
              ruoff = zeros((limages,limages),Float64)
              suoff = zeros((limages,limages),Float64)
              rvoff = zeros((limages,limages),Float64)
              svoff = zeros((limages,limages),Float64)
              for ij in range(len(i1d)):
                  j = j1d[ij]
                  i = i1d[ij]
                  nms[j,i] = nms1d[ij]
                  ruoff[j,i] =  ruoff1d[ij]
                  rvoff[j,i] =  rvoff1d[ij]
                  nms[i,j] = nms1d[ij]
                  ruoff[i,j] = -ruoff1d[ij]
                  rvoff[i,j] = -rvoff1d[ij]
                  suoff[i,j] =  suoff1d[ij]
                  svoff[i,j] =  svoff1d[ij]
                  suoff[j,i] =  suoff1d[ij]
                  svoff[j,i] =  svoff1d[ij]
              soff = sqrt(suoff**2+svoff**2)
              if 0:
                  fmt = limages*"%4d" + "\n"
                  [sys.stdout.write(fmt % tuple(nms1)) for nms1 in nms]
                  fmt = limages*"%6.2f" + "\n"
                  [sys.stdout.write(fmt % tuple(ru1)) for ru1 in ruoff]
                  [sys.stdout.write(fmt % tuple(rv1)) for rv1 in rvoff]
              mj,mi = indices(nms.shape)
              shifted = len(images)*[0]
              nma = 5
              nmt = add.reduce(greater(nms,nma),1)
              while sum(nms.flat):
                j = argmax(nmt)
                if not add.reduce(greater(nms,nma),1) and \
                  add.reduce(greater(nms,0),1) > add.reduce(greater(nms,nma),1) :
                    print shifted
                    if nma > 1: nma -= 1
                    else:
                      raise "Crap"
                for i in nonzero(nms[j]):
                  if nms[j,i] > nma:
                    jimage = os.path.basename(images[j].image)
                    iimage = os.path.basename(images[i].image)
                    images[i].uoff = images[i].uoff + ruoff[j,i]
                    images[i].voff = images[i].voff + rvoff[j,i]
                    fmt = "%3d %3d %s %s %4d %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f"
                    print fmt % (j,i,jimage,iimage,nms[j,i],ruoff[j,i],suoff[j,i],rvoff[j,i],svoff[j,i],images[i].uoff,images[i].voff)
                    ruoff[i] = ruoff[i] + ruoff[j,i]
                    rvoff[i] = rvoff[i] + rvoff[j,i]
                    nms[::,i] *= 0
                    nms[i,j] = 0
                    nmt = add.reduce(greater(nms,nma),1)
                    shifted[i] = 1
              print
              print "CatsToSky"
              [image.CatsToSky(cra,cdec) for image in images]
              su = concatenate( [image.su for image in images],0)
              sv = concatenate( [image.sv for image in images],0)
              su,sv= normalcoord(su,sv,cra,cdec,conv=3600*180/pi)
              su,sv = cleancat(su,sv)
              refu,refv = normalcoord(simra,simdec,cra,cdec,conv=3600*180/pi)
              print
              newmu,newuoff,newurms,newvoff,newvrms = GetOffset(su,sv,refu,refv,tol=[1.0,1.0][tpass],jtol=[3.0,3.0][tpass],verb=1)
#              raw_input("<pausing> ")
              print
              for image in images: image.uoff, image.voff = image.uoff+newuoff, image.voff+newvoff
#              for image in images: print "%3d %7.3f %7.3f" % (images.index(image),image.uoff,image.voff)
              print
              [image.CatsToSky(cra,cdec) for image in images]
       else:
          [image.GetOffset(ref=images[0],calc=1) for chip in [0,1] for image in images]

def cleancat(su,sv,tol=0.3):
  print "Input to clean",len(su)
  ul,vl = [],[]
  while len(su):
    du = subtract(su[0],su)
    dv = subtract(sv[0],sv)
    ds = sqrt(du**2+dv**2)
    match = (ds <= tol)
    u1,v1 = compress(match,[su,sv],1)
    if len(u1) > 1:
       ul.append(mean(u1))
       vl.append(mean(v1))
    su,sv = compress(logical_not(match),[su,sv],1)
  ul,vl = asarray([ul,vl])
  print "Output from clean",len(ul)
  return ul,vl

