
from FITS import *
from FITStable import *
from ReadSex import *
from masks import *
from VTKHelperFunctions import *
from basics import *

WFSCALE = 0.05

def rebin( d, dxa, dya, data=1, cubic=0, verb=1, constant=0, interp=1):
    if verb: print "Transforming."
    if data:
       y,x = indices(dxa.shape,Float32)
       g = VTKGradMag((y+dya),1)*VTKGradMag((x+dxa),1)
    t = VTKImageTransform(d.astype(Float32),dxa.astype(Float32),dya.astype(Float32),numret=1,interp=interp,cubic=cubic,constant=constant)
    if data: t=VTKMultiply(t,g)
    return t

def reversebin( d, dxa, dya, origsize=None, cubic=0, data=1, word="coordinates",verb=1,constant=0.0,interp=1):
    if verb:
       if origsize: print "Reverse Transforming to size:",origsize
       else: print "Reverse Transforming %s." % (word)
    y,x = indices(dxa.shape,d.typecode())
    if data:
       g = VTKMultiply(VTKGradMag(VTKAdd(y,dya),1),VTKGradMag(VTKAdd(x,dxa),1))
       d = VTKDivide(d,g)
    t = VTKImageTransform(d,dxa.astype(Float32),dya.astype(Float32),numret=1,reverse=1,origsize=origsize,interp=interp,cubic=cubic,constant=constant)
    return t

def randback(F):
   data = F.data().flat
   data = compress(not_equal(data,0),data)
   bg = GaussianBG(data.flat,299,verb=0)
   return bg

class Image:
   def __init__(self,image,subbg=0,suffix="",photoff=None,bgkey=None):
      self.image = image
      self.subbg = subbg
      self.solnfile = self.image.replace(".fits",".soln")
      self.solndat = readsoln(self.solnfile)
      self.sigim = self.image.replace(suffix+".fits","sig.fits")
      if not os.path.exists(self.sigim):
         self.sigim = self.image.replace(suffix+".fits","_sig.fits")
         if not os.path.exists(self.sigim): self.sigim = None
      self.open()
      self.naxes = self.FITS.naxes()
      self.naxis1, self.naxis2 = self.naxes
      self.exptime = self.FITS["exptime"]
      if self.exptime == "N/A": self.exptime = self.FITS["elaptime"]
      if self.exptime == "N/A": self.exptime = 1.0
      self.bg = 0.0
      if bgkey:
         self.bg = self.FITS[bgkey]
         if self.bg == "N/A": self.bg = 0.0
      if photoff:
         self.photoff = self.FITS[photoff]
         if self.photoff == "N/A": self.photoff = None
         else: self.exptime = 1.0
      else:
         self.photoff = None
      if self.photoff: self.exptime = 1.0
      self.ra_image,self.dec_image = self.ToSky(self.naxes[0]/2.,self.naxes[1]/2.)
      self.ras = formatSexagesimal(self.ra_image*180/pi/15,4,3600000)
      self.decs = formatSexagesimal(self.dec_image*180/pi,4,3600000)
      print "%20s %15s %15s" % (self.image,self.ras,self.decs),
      if self.photoff: print "%7.4f" % (self.photoff)
      else: print
      self.isolndat = self.invsoln(verb=1)
      if self.sigim: print "Got %s" % (self.sigim)
      self.close()

   def open(self):
      self.FITS = FITS(self.image)
      if self.sigim: self.sigFITS = FITS(self.sigim)
      else: self.sigFITS = None

   def close(self):
      self.FITS.close()
      if self.sigFITS: self.sigFITS.close()

   def GetCorners(self,cra=None,cdec=None):
       if not cra and not cdec:
          cra,cdec = self.solndat["sky"]
       self.bg = None
       self.ccors = []
       self.xcors = []
       self.ycors = []
       self.x0cors = []
       self.y0cors = []
       naxis1, naxis2 = self.naxes
       self.naxis1 = naxis1
       self.naxis2 = naxis2
       self.x0cors, self.y0cors = transpose([(xi,yi) for xi in [0,naxis1-1] for yi in [0,naxis2-1]])
       rds = [self.ToSky(xi,yi) for xi in [0,naxis1-1] for yi in [0,naxis2-1]]
       self.xcor,self.ycor = transpose([normalcoord(rds[j][0],rds[j][1],cra,cdec,conv=3600*180/pi) for j in range(4)])
       self.xcors.append(self.xcor)
       self.ycors.append(self.ycor)

   def GetBackground(self):
      self.open()
      self.bg = randback(self.FITS)
      self.close()
      print "Found background of %.1f %s." % (self.bg,self.image)

   def ToSky(self,xc,yc):
       x0,y0 = self.solndat["center"]
       rc,dc = self.solndat["sky"]
       xc = (xc-x0) + 1
       yc = (yc-y0) + 1
       u,v = mbasis(self.solndat["c"],xc,yc)
       rs, ds = equicoord(u,v,rc,dc,text=0)
       return rs,ds

   def invsoln(self,verb=0):
       nterms = len(self.solndat["c"])
       x = asarray( range(0,self.naxis1+self.naxis1/nterms,self.naxis1/nterms) + [self.naxis1])
       y = asarray( range(0,self.naxis2+self.naxis2/nterms,self.naxis2/nterms) + [self.naxis2])
       x = VTKSubtract(x, self.solndat["center"][0])
       y = VTKSubtract(y, self.solndat["center"][1])
       nx = len(x); ny = len(y)
       o = ones((ny,nx),Float32)
       yy = (o*y[::,NewAxis]).flat
       xx = (o*x[NewAxis,::]).flat
       u,v = mbasis(self.solndat["c"],xx,yy)
       nterm = [4,6,12,20,30,42,56]
       nords = [0,1, 2, 4, 4, 4, 4]
       m = nords[nterm.index(nterms)]
       b = abasis(m,u,v)
       c = svdfit(b,concatenate([xx,yy]))
       xf,yf = mbasis(c,u,v)
       xres = VTKSubtract(xf,xx); yres = VTKSubtract(yf,yy)
       if verb:
         print "Using",nterm,"terms (order=",m,")"
         print "Inverse solution RMS=",rms(xres),rms(yres)
       self.solndat["ic"] = c

   def invsoln_new(self,verb=0):
       nterms = len(self.solndat["c"])
       nterm = [4,6,12,20,30,42,56]
       nords = [0,1,3,4,5,6,6]
       mo = nords[nterm.index(nterms)]
       ksp = (2*mo+2)
       x0,y0 = self.solndat["center"]
       rc,dc = self.solndat["sky"]
       imsize = self.naxis2, self.naxis1
       dyc,dxc = imsize
       ux = arange(-dxc/ksp,imsize[1]+0.5*dxc/ksp,dxc/ksp)
       uy = arange(-dyc/ksp,imsize[0]+0.5*dxc/ksp,dyc/ksp)
       x4,y4 = asarray(len(uy)*[ux]), transpose(len(ux)*[uy])
       x4,y4 = x4.flat, ravel(y4)
       tra,tdec = self.ToSky(x4,y4)
       tsqu,teta = normalcoord(tra,tdec,rc,dc)
       dsqu = (max(tsqu)-min(tsqu))/ksp
       deta = (max(teta)-min(teta))/ksp
       kx,ky = 3,3
       xknots = concatenate([ (kx+1)*[min(tsqu)], (kx+1)*[max(tsqu)]])
       yknots = concatenate([ (ky+1)*[min(teta)], (ky+1)*[max(teta)]])
       xb = min(xknots); xe = max(xknots); yb = min(yknots); ye = max(yknots)
       x4 = (x4 - x0) + 1
       y4 = (y4 - y0) + 1
       xtck = bisplrep(tsqu,teta,x4,task=-1,s=None,kx=kx,ky=ky,tx=xknots,ty=yknots,\
          xb=xb, xe=xe, yb=yb, ye=ye, nxest=len(xknots), nyest=len(yknots), full_output=0,quite=1)
       ytck = bisplrep(tsqu,teta,y4,task=-1,s=None,kx=kx,ky=ky,tx=xknots,ty=yknots,\
          xb=xb, xe=xe, yb=yb, ye=ye, nxest=len(xknots), nyest=len(yknots), full_output=0,quite=1)
       self.solndat["xtck"] = xtck
       self.solndat["ytck"] = ytck

   def FromSky_new(self,rs,ds):
       x0,y0 = self.solndat["center"]
       rc,dc = self.solndat["sky"]
       u, v = normalcoord(rs,ds,rc,dc)
#       xfold,yofld = self.FromSky_old(rs,ds)
       eu = extrema(self.solndat["xtck"][0])
       ev = extrema(self.solndat["xtck"][1])
       good = logical_and(between(u,eu[0],eu[1]), between(v,ev[0],ev[1]))
       xf = -999*ones(u.shape,Float32)
       yf = -999*ones(v.shape,Float32)
       if len(good.shape) == 1:
          if sum(good):
             u = clip(u,eu[0],eu[1])
             v = clip(v,ev[0],ev[1])
             xi,yi = gbspl(u,v,self.solndat["xtck"],self.solndat["ytck"],x0,y0)
             xu = where(good,xi,-999)
             yu = where(good,yi,-999)
             xf = xu
             yf = yu
       else:
         for j in range(len(good)):
            if sum(good[j]):
               gu = clip(u[j],eu[0],eu[1])
               gv = clip(v[j],ev[0],ev[1])
               xi,yi = gbspl(gu,gv,self.solndat["xtck"],self.solndat["ytck"],x0,y0)
               xf[j] = where(good[j],xi,-999).astype(xf.typecode())
               yf[j] = where(good[j],yi,-999).astype(yf.typecode())
       return xf, yf

   def FromSky(self,rs,ds):
       x0,y0 = self.solndat["center"]
       rc,dc = self.solndat["sky"]
       u, v = normalcoord(rs,ds,rc,dc)
       xf, yf = mbasis(self.solndat["ic"],u,v)
       xc = xf + x0 - 1
       yc = yf + y0 - 1
       return xc, yc

   def GetPixelPositions(self,ras,decs,rc=None,dc=None):
       if not rc and not dc:
          rc,dc = self.solndat["sky"]
       u, v = normalcoord(ras,decs,rc,dc,conv=3600*180/pi)
       xi, yi = self.FromSky(ras,decs)
       gx = between(xi,0,self.naxes[0],inc=1).astype(Float32)
       gy = between(yi,0,self.naxes[1],inc=1).astype(Float32)
       gr = between(u,min(self.xcor),max(self.xcor),inc=1)
       gd = between(v,min(self.ycor),max(self.ycor),inc=1)
       good = gx*gy*gr*gd
       return xi, yi, good.astype(Float32)

   def GetSquare(self,xlo,xhi,ylo,yhi,cra,cdec,buffer=0,verb=0):
      if verb: print  "Reading %s." % (self.image),;sys.stdout.flush()
      if self.subbg:
        if self.bg == None: self.GetBackground()
      bg = self.bg
      if bg == None: bg = 0.0
      self.open()
      xlo = max([0,xlo-buffer])
      xhi = min([xhi+buffer,self.naxis1])
      ylo = max([0,ylo-buffer])
      yhi = min([yhi+buffer,self.naxis2])
      data = self.FITS.readsec([xlo,ylo],[xhi,yhi])
      data = where(between(data,-1e-24,1e24),data,0.0)
      if self.sigFITS:
         sig = self.sigFITS.readsec([xlo,ylo],[xhi,yhi])
         sig = where(between(sig,-1e-24,1e24),sig,0.0)
      else: sig = not_equal(data,0).astype(Float32)
      if self.photoff:
        data = VTKMultiply(data,power(10,-0.4*self.photoff))
        if self.sigFITS: sig = VTKMultiply(sig, power(10,-0.2*self.photoff))
        bg = bg * power(10,-0.4*self.photoff)
      dq = greater(sig,0).astype(data.typecode())
      self.close()
      return (data,bg,sig,dq,xlo,ylo)

def gbspl(u,v,xtck,ytck,x0,y0):
      xc = bisplev2(u,v,xtck)
      yc = bisplev2(u,v,ytck)
      xi = xc + x0 - 1
      yi = yc + y0 - 1
      return xi,yi



