
from basics import *
from pgplot import *
from sets import *
from NumRec import *
from masks import *
from scipy.special import *

DATABASE = sys.prefix+"/Database"
GAINS = [0.82,0.85,0.87,1.13,0.77,0.93,0.92,0.80]

LDPDISP = { "lambda": 100.0*array([38,39,40,41,42,43,44,45,46,47,48,
            49,50,51,52,53,54,55,56,57,58,59,60, 62,64,66,68,70,72,74,76,78,80,82,84,
            86,88,90,92,94,96,98,100,102,104,106]),
            "position": array([2269,2033,1823,1635,1467,1315,1177,1052,
            938,834,738,650,569,494,424,360,300,243, 191,142,96,53,12,-61,-127,-186,-238,-285,
            -327,-365,-400,-431,-459,-484,-507,-528, -547,-564,-579,-593,-606,-617,-628,-637, -645,-652])}

LDPCOEF = array([ 0.48682052, 0.26810576,-0.19797286,-0.02573573,-0.02867552,-0.03297195,
      -0.00763004,-0.00584553,])

LDPCOEF0 = 1e4/3.8e3
LDPCOEF1 = 1e4/13e3
LDPCOEFC = (LDPCOEF0+LDPCOEF1)/2.
LDPCOEFS = (LDPCOEF0-LDPCOEF1)/2.
LDPCOEFCi = (13e3+3.8e3)/2.
LDPCOEFSi = (13e3-3.8e3)/2.
LDPZ = 75.0

ROTepoch = 2007.37

sstr = lambda s: s.strip()

class SMFDict(DictType):
    SMFKEYS = ["type","id","ra","dec","x","y","w","t","b"]
    fn = [sstr,sstr,sstr,sstr,float,float,float,float,float]
    def ReadFile(self,smf_file):
        self.name = os.path.basename(smf_file).split(".")[0]
        for key in self.SMFKEYS: self[key] = []
        f = open(smf_file,"r")
        for l in f:
           data = l.split()
           if data[0] == "POSITION":
              self["center"] = data[2:4]
           elif (data[0] == "SLIT") or (data[0] == "HOLE" and data[7] == "1"):
              extra = lambda key: (key in "tb" and data[7] == "1" and 1) or 0
              [self[key].append(self.fn[self.SMFKEYS.index(key)](data[self.SMFKEYS.index(key)+extra(key)])) for key in self.SMFKEYS]
        f.close()
        for key in self.SMFKEYS[4:]: self[key] = array(self[key])
        self["spec"] = [None for n in range(len(self["id"]))]

def ReadChip(root,n,sex=0,sexdir="./",M=3,N=9,FW=1.0,direct=False):
   cat = "%sc%d.cat" % (root,n)
   if sex or not os.path.exists(cat):
     image = "%sc%d.fits" % (root,n)
     fsexdir=sexdir
     if fsexdir[-1] != "/": fsexdir = sexdir + "/"
     pid = os.getppid()
     filtname = "simple%d.conv" % (pid)
     kernelM = exp(-0.5*(arange(-M,M+0.1,1,Float32))**2/(FW/2.35)**2)
     kernelN = ones(arange(-N,N+0.1,1,Float32).shape,Float)
     kernel = kernelN[::,NewAxis]*kernelM[NewAxis,::]
     filtout = open(filtname,"w")
     print >> filtout, "CONV NORM"
     print >> filtout, "# %dx%d simple convolution mask" % (len(kernelM),len(kernelN))
     fmt = len(kernelM)*" %.7f"
     for k1 in kernel: print >> filtout, fmt % tuple(k1)
     filtout.close()
     command = " ".join(["sex",image,"-c",fsexdir+"default.sex",
            "-CATALOG_NAME",cat,
            "-DETECT_THRESH %f" % ([2,100][direct]),
            "-DETECT_MINAREA %d" % ([N,N][direct]),
            "-ANALYSIS_THRESH %f" % ([2,20][direct]),
            "-DEBLEND_NTHRESH %d" % ([64,64][direct]),
            "-GAIN 2",
            "-BACK_SIZE %d" % ([5,64][direct]),
            "-BACKFILTER_SIZE %d" % ([5,3][direct]),
            "-PIXEL_SCALE 0.2",
            "-SEEING_FWHM 1.0",
            "-FILTER_NAME",filtname,
            "-STARNNW_NAME",fsexdir+"default.nnw",
            "-PARAMETERS_NAME",fsexdir+"default.param"
            ])
     verbex(command)
     os.remove(filtname)
   cx,cy,cm = transpose([map(float,l.split()[1:4]) for l in open(cat,"r").readlines() if "#" not in l]) - 1
   ca = exp10(-0.4*cm)
   return (cx,cy,ca)

def OneGaussian(p,x):
   a,c,s = p
   g = a*exp(-0.5*((x-c)/s)**2)
   return g

def MultiGaussian(p,c0,s0,x,lam,dlam,y,n,w,fit=1,verb=0):
   l = len(c0)
   a = power(10,p[:l])
   c = p[l:2*l]
   c = where(equal(w,0),c0,c)
   cpars = p[2*l:]
   cont = (cpars[0]+cpars[1]*dlam) * exp(-(lam-cpars[2])*greater(lam,cpars[2])/cpars[3])
   s = power(10,s0)
   m = len(a)
   p1 = lambda j: [a[j],c[j],s[j]]
   if verb:
      print p
   g = sum([OneGaussian(p1(j),x) for j in range(m)]) + cont
   if fit:
      d = g-y
      h = d*n
      return h
   else:
      return g,cont

def AdjustSpacings(A,s=1e-6,extras=None,verb=1):
   x = A[0]
   o = argsort(x)
   A = take(A,o,1)
   if extras: extras = [take(extra,o) for extra in extras]
   x = A[0]
   dx = diff(x)
   while sometrue(equal(dx,0)):
      if verb: print "adjusting spacings...", sum(equal(dx,0))
      p = concatenate([[0.0],equal(dx,0)])
      x = x + s * p * (random(p.shape)-0.5)
      A[0] = x
      o = argsort(x)
      A = take(A,o,1)
      if extras: extras = [take(extra,o) for extra in extras]
      x = A[0]
      dx = diff(x)
   if extras: return A,extras
   else: return A


def ghquad(w,s,tck,X,W):
   s2 = sqrt(2)*s
   x = s2 * X
   f = splev(x+w,tck)
   k2 = pow(x/s2,2)
   h = exp(-k2) * W
   i = sum(h*f)
   i = i/sqrt(pi)
   return i


