
import time
import cPickle as cP
from BC03 import *
from PhotoZ import *
from Expector import *
from basics import *
from FITS import *
from pgplot import *
from VTKHelperFunctions import *
from Multipack import *
from sets import Set
from NumRec import *
from scipy.special import *
from masks import *
from ReadSex import *
from Mestimators import *
from persistent import Persistent
from persistent.mapping import PersistentMapping
from BTrees.OOBTree import OOBTree
from BTrees.IOBTree import IOBTree

MEAN_ZF = 5.0
AgeOfU = tlook(MEAN_ZF,0.7,[0.3,0.7])*1e9

SAVEITEMS = ["xs","xes","specs","noises","specas","noiseas",
        "skys","waves","disps","qes","rqes","qe0s","okeeps",
        "w1","w2","weights","gscales","escale","scales","mscales","match",
        "p_z","peak_z","prob_z","sprob_z","cprob_z","conf_z",
        "p_t","peak_t","prob_t","sprob_t","cprob_t","conf_t",
        "p_Z","peak_Z","prob_Z","sprob_Z","cprob_Z","conf_Z",
        "chi_3d","prob_3d","chi_zt","chi_tZ","dchi2","mchi2","rchi2",
        "gridname",
        ]

def Select(xp,yp):
   key,(x,y),(xi,yi) = getcurs(xp,yp)
   return key,x,y

class QE(Persistent):
  def __init__(self,qefile):
     if qefile.split(".")[-1] == "fits":
       q = FITS(qefile)
       qe_thru,qe_wav = q.data()[:2]
       q.close()
       qe_thru, qe_wav = take([qe_thru, qe_wav], argsort(qe_wav),1)
       qe_thru, qe_wav = compress(qe_wav,[qe_thru, qe_wav], 1)
       qe_thru, qe_wav = take([qe_thru, qe_wav], nonzero(diff(qe_wav)), 1)
       self.qe_wav,self.qe_thru = qe_wav, qe_thru
     else:
       self.qe_wav,self.qe_thru = readtable(qefile)
     self.qe_lthru = getlog10(self.qe_thru)
     self.qtck = splrep(self.qe_wav, self.qe_lthru ,s=0,k=1)
     self.wtck = splrep(self.qe_wav,arange(self.qe_wav.shape[0]),s=0,k=1)
  def interp(self,wave,simple=0):
      if simple: return self.Sinterp(wave)
      else: return self.Binterp(wave)
  def Binterp(self,wave):
      l = splev(wave,self.qtck)
      return exp10(l)
  def Sinterp(self,wave):
      order = argsort(wave)
      wx = zeros(wave.shape,Float32)
      put(wx,order,splev(take(wave,order),self.wtck))
      wi = clip(wx,0,self.qe_wav.shape[0]-2)
      wi0 = wi.astype(Int32)
      wf = 1.0-(wi-wi0)
      q0 = take(self.qe_lthru,wi0)
      q1 = take(self.qe_lthru,wi0+1)
      q = wf*q0 + (1-wf)*q1
      return exp10(q)

class FauxTemplate:
   def __init__(self,lam,flam):
      self.lam = lam
      self.flam = flam

class Grid(Persistent):
   def __init__(self,grid,ageprior=0,metalprior=0,summary=0,MEST="Huber2",lines=[3727.]):
       self.name = grid
       self.MEST = MEST
       print "Reading templates"
       self.ReadGrid(grid,summary=summary)
       print "Reading magnitudes"
       self.ReadMags(grid)
       print "Reading wavelengths and wavelength binning of templates"
       self.ReadWave(grid)
       print "Constructing arrays of template parameters"
       self.GetParams()
       if not summary:
          print "Constructing simple emission lines"
          self.Emission(lines)
          print "Constructing simple priors"
          self.WhatPriors(ageprior=ageprior,metalprior=metalprior)
   def ReadGrid(self,grid,summary=0):
       Gin = FITS(grid)
       self.z0 = Gin["z0"]
       self.z1 = Gin["z1"]
       self.dz = Gin["dz"]
       self.Z0 = Gin["metal0"]
       self.Z1 = Gin["metal1"]
       self.dZ = Gin["dmetal"]
       self.T0 = Gin["logage0"]
       self.T1 = Gin["logage1"]
       self.dT = Gin["dlogage"]
       self.zstart = Gin["zstart"]
       if not summary: self.flams = Gin.data()
       Gin.close()
   def ReadWave(self,grid):
       Gin = FITS(grid.replace(".fits","_wave.fits"))
       self.waves,self.disps = Gin.data()
       Gin.close()
   def ReadMags(self,grid):
       if os.path.exists(grid.replace(".fits","_mags.fits")):
          Gin = FITS(grid.replace(".fits","_mags.fits"))
          self.mags = Gin.data()
          Gin.close()
          self.fmags = None
       else:
          self.mags = None
          self.fmags = None
   def Emission(self,lines=[3727.]):
       lines = asarray(lines)
       tck = splrep(self.waves,self.disps,s=0,k=3)
       sig = lambda w: asarray([1.7*splev(w1,tck) for w1 in w])
       redlines = lambda z: ((1+z)*lines[::,NewAxis]-self.waves[NewAxis,::])/sig((1+z)*lines)[::,NewAxis]
       not_aband = lambda z: logical_or(less((1+z)*lines,7300),greater((1+z)*lines,7900))
       self.emit = {}
       self.mask = {}
       self.extinct = {}
       for z in self.redshifts:
           self.emit[z] = sum(not_aband(z)*getexp(-0.5*redlines(z)**2)/sqrt(2*pi)/sig((1+z)*lines)[::,NewAxis])
           self.mask[z] = sum(not_aband(z)*getexp(-0.5*redlines(z)**2))
           self.extinct[z] = Cardelli(self.waves/(1+z))
       self.emission = asarray([self.emit[z]*less(a,10) for a,z in zip(self.allage,self.allred)]).astype(Float32)
       self.extinction = asarray([self.extinct[z]*less(a,10) for a,z in zip(self.allage,self.allred)]).astype(Float32)
       self.emask = asarray([self.mask[z]*less(a,10) for a,z in zip(self.allage,self.allred)]).astype(Float32)
   def GetParams(self):
       self.redshifts = arange(self.z0,self.z1+self.dz/200.,self.dz,Float)
       self.logages = arange(self.T0,self.T1+self.dT/200.,self.dT,Float)
       self.metals = arange(self.Z0,self.Z1+self.dZ/200.,self.dZ,Float)
       self.allred,self.allage,self.allmet = transpose([(redshift1,logage1,metal1) \
                for metal1 in self.metals \
                for logage1 in self.logages \
                for redshift1 in self.redshifts])
   def WhatPriors(self,ageprior=0,metalprior=0):
       looks = array([(z and tlook(z,0.7,[0.3,0.7])*1e9) or tlook(self.redshifts[0]/4.,0.7,[0.3,0.7]) for z in self.redshifts])
       cms = array([(z and tlook(z,0.7,[0.3,0.7])*1e9*SECPYR*CLIGHT) or tlook(self.redshifts[0]/4.,0.7,[0.3,0.7])*1e9*SECPYR*CLIGHT for z in self.redshifts])
       self.looks = zeros(self.allred.shape,Float64)
       [add(self.looks,l*equal(self.allred,z),self.looks) for z,l in zip(self.redshifts,looks)]
       self.cms = zeros(self.allred.shape,Float64)
       [add(self.cms,c*equal(self.allred,z),self.cms) for z,c in zip(self.redshifts,cms)]

       dcms = (cms[2:] - cms[:-2])/2
       volumes = 4*pi*power(cms[1:-1],2)*dcms
       volume0 = 4*pi*power(cms[0],3)/3.
       volumeN = 4*pi*power(cms[-1],2)*dcms[-1]
       volumes = concatenate([[volume0],volumes,[volumeN]])
       volumes = volumes / median(volumes)

       vol = zeros(self.cms.shape,Float64)
       [add(vol,v*equal(self.cms,c),vol) for c,v in zip(cms,volumes)]
       self.priors = vol
#       self.priors = self.priors * less(self.allred,0.95)

       if ageprior:
          print "Setting age priors"
          allT = exp10(self.allage)
          Zages = AgeOfU - self.looks
          AgeDist = where(greater(allT,Zages),exp(-0.5*(allT-Zages)**2/0.5e9**2),1.0)
          self.priors *= AgeDist

       if metalprior:
          print "Setting metallicity priors"
          MetalDist = exp(-0.5*(self.allmet/0.3)**2)
          self.priors *= MetalDist

       print "Done computing priors"

   def Pull(self,owaves,odisps,oqes,rqes,oqe0s,etimes):
       xtemp = arange(len(self.waves))
       wtemp = self.waves
       wtck = splrep(wtemp,xtemp,s=0,k=1)

#       gxs = [splev(owave,wtck) for owave in owaves]

       gxs = [zeros(owave.shape,Float32) for owave in owaves]
       [put(gx,argsort(owave),splev(take(owave,argsort(owave)),wtck)) for owave,gx in zip(owaves,gxs)]

       gus = [between(gx,0,len(self.waves)-2) for gx in gxs]
       gxs = [clip(gx,0,len(self.waves)-2) for gx in gxs]
       gfs = [gx-gx.astype(Int32) for gx in gxs]
       gcs = [(1-gf) for gf in gfs]
       gis = [clip(gx.astype(Int32),0,len(self.waves)-2) for gx in gxs]

       gw1s = [take(self.waves,gi) for gi in gis]
       gw2s = [take(self.waves,gi+1) for gi in gis]
       gws = [(1.0-gf)*gw1 + gf*gw2 for gf,gw1,gw2 in zip(gfs,gw1s,gw2s)]

       gd1s = [take(self.disps,gi) for gi in gis]
       gd2s = [take(self.disps,gi+1) for gi in gis]
       gds = [(1.0-gf)*gd1 + gf*gd2 for gf,gd1,gd2 in zip(gfs,gd1s,gd2s)]

       nflams = self.flams.shape[0]
       facs = [array(nflams*[(etime*rqe*divz(oqe,oqe0)*odisp).astype(Float32)],Float32) for oqe,rqe,oqe0,etime,odisp in zip(oqes,rqes,oqe0s,etimes,odisps)]

       gf1s = [(take(self.flams,gi,1)*gc).astype(Float32) for gi,gc in zip(gis,gcs)]
       gf2s = [(take(self.flams,gi+1,1)*gf).astype(Float32) for gi,gf in zip(gis,gfs)]
       self.rflams = []
       [self.rflams.append(VTKMultiply(VTKAdd(gf1,gf2),fac)) for gf1,gf2,etime,fac in zip(gf1s,gf2s,etimes,facs)]
       del facs

       ge1s = [(take(self.emission,gi,1)*gc).astype(Float32) for gi,gc in zip(gis,gcs)]
       ge2s = [(take(self.emission,gi+1,1)*gf).astype(Float32) for gi,gf in zip(gis,gfs)]
       self.eflams = []
       [self.eflams.append(VTKAdd(ge1,ge2)) for ge1,ge2,etime in zip(ge1s,ge2s,etimes)]

       ge1s = [(take(self.emask,gi,1)*gc).astype(Float32) for gi,gc in zip(gis,gcs)]
       ge2s = [(take(self.emask,gi+1,1)*gf).astype(Float32) for gi,gf in zip(gis,gfs)]
       self.emasks = []
       [self.emasks.append(VTKAdd(ge1,ge2)) for ge1,ge2,etime in zip(ge1s,ge2s,etimes)]

       ge1s = [(take(self.extinction,gi,1)*gc).astype(Float32) for gi,gc in zip(gis,gcs)]
       ge2s = [(take(self.extinction,gi+1,1)*gf).astype(Float32) for gi,gf in zip(gis,gfs)]
       self.extincts = []
       [self.extincts.append(VTKAdd(ge1,ge2)) for ge1,ge2,etime in zip(ge1s,ge2s,etimes)]

       guses = [gu*greater(gd,0)*greater(gw,0) for gu,gd,gw in zip(gus,gds,gws)]

       self.gxs = gxs
       self.gws = gws
       self.gds = gds
       self.guses = guses

   def Push(self,galaxy):
      oname = galaxy.oname
      oxs = galaxy.xs
      oxes = galaxy.xes
      ospecs = galaxy.specs
      onoises = galaxy.noises
      oskys = galaxy.skys
      owaves = galaxy.waves
      odisps = galaxy.disps
      oqe0s = galaxy.qe0s
      oqes = galaxy.qes
      rqes = galaxy.rqes
      etimes = galaxy.etimes
      usesex = self.mags and galaxy.sex
#      usesex = False

      ospecas = galaxy.specas
      onoiseas = galaxy.noiseas

      okeeps = galaxy.okeeps

      guses = self.guses
      rflams = self.rflams
      eflams = self.eflams
      emasks = self.emasks
      gds = self.gds
      gws = self.gws

      if galaxy.down:
          Niter = 3
      else:
          Niter = 1

      if "," in self.MEST:
          MESTS = self.MEST.split(",")
      else:
          MESTS = [self.MEST]
      Niter = len(MESTS)

      glams = [[] for f in range(len(ospecas))]
      edifs = [[] for f in range(len(ospecas))]
      w2ds = [[] for f in range(len(ospecas))]
      for Jiter in range(Niter):
          ochi2 = 0.0
          weights = []
          mscales = []
          gscales = []
          escale1 = 0.0
          escale2 = 0.0
          prob = 1.0
          werrprob = 0.0

          for f in range(len(ospecas)):
             ospec = ospecs[f]
             onoise = onoises[f]
             owave = owaves[f]
             odisp = odisps[f]
             okeep = okeeps[f]
             rflam = rflams[f]
             eflam = eflams[f]
             emask = emasks[f]
             oqe = oqes[f]
             rqe = rqes[f]
             guse = guses[f]
             etime = etimes[f]

             weight = 1.*between(owave,galaxy.w1,galaxy.w2)
             weight = weight * guse * okeep * greater(onoise,0)
#             weight = weight * oqe**2 / mean(compress(weight,oqe**2))
#             weight = weight * (odisp / mean(compress(weight,odisp)))
#             weight = weight * not_atmos
             weights.append(weight)

             oyb = 1/(etime*oqe*rqe*odisp)
             oy = ospec*oyb
             oye = onoise*oyb

             if galaxy.down and Jiter:
                r2 = divz(ospec-glams[f][match],onoise)
                r2u = r2*(greater(r2,0)/galaxy.down+less(r2,0)/(2*galaxy.down))
                dweight = 1./(1. + power(r2u,2))
                weight2d = weight * dweight
             elif Jiter:
                r2 = divz(ospec-glams[f][match],onoise)
                weight2d = weight * eval(MESTS[Jiter])(r2,w=1)[NewAxis,::]
#                r2 = divz(ospec[NewAxis,::]-glams[f],onoise[NewAxis,::])
#                weight2d = weight[NewAxis,::] * eval(MESTS[Jiter])(r2,w=1)
             else:
                weight2d = weight
             w2ds[f] = weight2d

             wno = weight2d*divz(ospec,onoise**2)
             wn = weight2d*divz(1,onoise**2)

             wne0 = greater(emask,0.2)
             wne1 = less_equal(emask,0.2)

             if 0:
                wl = divz(weight2d,noise)
                wr = divz(ospec*weight2d,noise)
                gscale = array([dot(linear_least_squares( \
                        transpose([rflam[l]])*wl,wr)[0],transpose([rflam[l]])) \
                    for l in range(len(rflam))])
                gscale = gscale * greater(gscale,0)
             else:
                gscale1 = add.reduce((wne1*wno)*rflam,1)
                gscale2 = add.reduce((wne1*wn)*power(rflam,2),1)
                gscale = divz(gscale1,gscale2)
                gscale = gscale * greater(gscale,0)

             gscales.append(gscale)

             mscale = getlog10(gscale * (4*pi*power(self.cms,2)) * (1+self.allred) / LSUN, rep=-500)
             mscales.append(mscale)

             eflam = etime*gscale[::,NewAxis]*eflams[f]

             glams[f] = (rflam*gscale[::,NewAxis]).astype(Float32)
             edif = ((ospec[NewAxis,::] - glams[f])/rqe[NewAxis,::]).astype(Float32)

#             qdump("edif%d.fits" % (f),edif*greater(wne0,0.01))

             escale1 += add.reduce(wne0*wn*edif*eflam,1)
             escale2 += add.reduce(wne0*wn*power(eflam,2),1)

          escale = divz(escale1,escale2)
          escale = escale*greater(escale,0)

#          qdump("escale.fits",escale)

          mest = 0.0

          for f in range(len(ospecas)):
             ospec = ospecs[f]
             onoise = onoises[f]
             owave = owaves[f]
             odisp = odisps[f]
             okeep = okeeps[f]
             oqe = oqes[f]
             rqe = rqes[f]
             guse = guses[f]
             etime = etimes[f]

             oyb = 1/(etime*oqe*rqe*odisp)
             oy = ospec*oyb
             oye = onoise*oyb

             eflam1 = ((etime*gscales[f]*escale)[::,NewAxis]*eflams[f]*rqe[NewAxis,::]).astype(Float32)
             glams[f] = VTKAdd(glams[f],eflam1)

             numer = ospec[NewAxis,::]-glams[f]
             denom = onoise[NewAxis,::]
             aprob1 = divz(numer,denom)

#             ochi1 = add.reduce(w2ds[f]*power(aprob1,2),1)
             ochi1 = add.reduce(weights[f][NewAxis,::]*power(aprob1,2),1)
             ochi2 += ochi1

             print MESTS[Jiter],
             sys.stdout.flush()

             mest1 = eval(MESTS[Jiter])(aprob1)
             mest1 = weights[f][NewAxis,::]*mest1
             mest = mest + add.reduce(mest1,1)

#             prob1 = exp(-mest)
#             prob1 = exp(-weights[f][NewAxis,::]*L2(aprob1))
#             prob = prob * multiply.reduce(prob1,1)

#             match = argmax(prob)
#             ndeg = (len(nonzero(weights[f])) - 1. - 1. - 3.)
#             sc = sqrt(ochi1[match]/ndeg)

          print

          ndeg = (len(nonzero(concatenate(weights))) - len(gscales) - 1. - 3.)

          if usesex and galaxy.sexdata["FLAGS"] <= 3:
              filts = galaxy.photfilt
              mags = array([galaxy.magdata[filt]["mag1"] for filt in filts])
              errs = array([galaxy.magdata[filt]["err1"] for filt in filts])
              errs = sqrt(errs**2 + 0.03**2)
              fabs = array([galaxy.tranfilt[filt].conv_AB for filt in filts])
              umags = 1.0*greater(mags,-90.0)
              fmags = exp10(-0.4*(mags+fabs-25.0))
              ferrs = errs*(2.303/2.5)*fmags * umags
              if not self.fmags:
                 self.umags = greater(self.mags,-90)
                 self.fmags = exp10(-0.4*(self.mags+fabs[NewAxis,::])) * self.umags
              fscales1 = add.reduce(divz(self.fmags*fmags[NewAxis,::],self.umags*ferrs[NewAxis,::]**2),1)
              fscales2 = add.reduce(divz(self.fmags*self.fmags,self.umags*ferrs[NewAxis,::]**2),1)
              fscales = divz(fscales1,fscales2)
              mmags = self.fmags * fscales[::,NewAxis]
              fprobs = divz(fmags[NewAxis,::] - mmags,self.umags*ferrs[NewAxis,::])
              fmest = eval(MESTS[Jiter])(fprobs)
              fmest = umags[NewAxis,::]*fmest
              mest = mest + add.reduce(fmest,1)
#              fprob1 = exp(-umags[NewAxis,::]*fmest)
#              prob = prob * multiply.reduce(fprob1,1)
              fochi1 = add.reduce(power(fprobs,2),1)
              ochi2 += fochi1
              ndeg = ndeg + (sum(umags) - 1)

          mest = mest - min(mest.flat)
          prob = exp(-mest)
          match = argmax(prob)

#          sc = ochi2[match]/ndeg
#          prob = power(prob,1.0/sc)

#      3 SSP parameters, 1 emission parameter

      zeroloc = argmin(ochi2.flat)
      zerochi2 = ochi2.flat[zeroloc]

      prob = prob * self.priors
#      forming = less_equal(self.allage,9.0)*(escale/max(compress(escale,escale)))
#      prob = prob * (forming+greater(self.allage,9.0))
      match = argmax(prob)

      prob_3d = reshape(prob,(len(self.metals),len(self.logages),len(self.redshifts)))
      prob_3dn = add.reduce(add.reduce(add.reduce(prob_3d*self.dz,-1)*self.dT,-1)*self.dZ,-1)
      if prob_3dn:
         prob = divz(prob, prob_3dn)
         prob_3d = divz(prob_3d, prob_3dn)
      else:
         print "Zero probabilities!"
         prob = 1e-12*exp((zerochi2-ochi2)/zerochi2/2.)
         prob_3d = reshape(prob,(len(self.metals),len(self.logages),len(self.redshifts)))
         prob_3dn = add.reduce(add.reduce(add.reduce(prob_3d*self.dz,-1)*self.dT,-1)*self.dZ,-1)
         prob = divz(prob, prob_3dn)
         prob_3d = divz(prob_3d, prob_3dn)

      chi_3d = reshape(ochi2,(len(self.metals),len(self.logages),len(self.redshifts)))

      self.glams = glams
      galaxy.weights = weights
      galaxy.gscales = gscales
      galaxy.escale = escale
      galaxy.scales = divz(gscales , mean(gscales))
      galaxy.mscales = mscales

      return chi_3d,ndeg,prob_3d,match

class Extractions(Persistent):
  def __init__(self,spectra):
      self.spectra = spectra
      self.ReadFrames()
      self.what = {}
      self.times = {}
      self.where = {}
      self.WhatObjects()
  def WhatObjects(self):
      SFITS = [FITS(s) for s in self.spectra]
      objects = []
      for f,s in zip(self.spectra,SFITS):
         self.what[f] = [s["OBJ%04d" % (j+1)].strip() for j in range(s["naxis2"])]
         objects.append( self.what[f])
         self.times[f] = 2*s["exptime"]
      [s.close() for s in SFITS]
      for s,s_o in zip(self.spectra,objects):
         for o in s_o:
            row = s_o.index(o)
            check = sum(greater(self.Data[s][3,row,::],5000))
#            if o == "H052360": print s,row,check
            if check > 70:
               if o in self.where:
                  self.where[o] += [s]
               else:
                  self.where[o] = [s]
  def ReadFrames(self):
      self.Data = {}
      SFITS = [FITS(s) for s in self.spectra]
      for s in SFITS: self.Data[s.name()] = s.data()
      [s.close() for s in SFITS]
  def Cull(self,X):
     W = X[3]
     X = compress(greater(W,0),X,1)
     return X
  def Shift(self,X,verb=0):
   for i in range(1):
     N = X[1]
     S = X[2]
     W = X[3]
     D = X[4]
     x = arange(0,W.shape[0],1,Float32)
     u = between(W,5577-180,5577+180)
     if 0:
       W2 = VTKImageShift(1.*W,-1,0,numret=1,interp=1,cubic=1)
       D2 = VTKImageShift(1.*D,-1,0,numret=1,interp=1,cubic=1)
       X[3] = W2.astype(X[3].typecode())
       X[4] = D2.astype(X[4].typecode())
       W = X[3]
       D = X[4]
     u = between(W,5577-200,5577+200)
     if 0 and len(u) > 100 and sum(u)>6:
       Y = compress(u,[x,W,D,S,N],1)
       x,w,d,s,n = Y
       g = [log10(max(s)-min(s)),argmax(s)+x[0],0.3,min(s),(s[-1]-s[0])/(x[-1]-x[0])]
       g = leastsq(Gaussian,g,args=(x,w,d,s,n))
       tck = splrep(w,x,s=0,k=3)
       x0 = splev(5577.34,tck)
       o = g[0][1] - x0
       if 1 or verb: print "Shift from 5577 is",o
       W2 = VTKImageShift(1.*W,-o,0,numret=1,interp=1,cubic=1)
       D2 = VTKImageShift(1.*D,-o,0,numret=1,interp=1,cubic=1)
       if 0:
          x = arange(0,W.shape[0],1,Float32)
          u = between(W,5200,9000,inc=1)
          Y = compress(u,[x,W,D,S,N],1)
          x,w,d,s,n = Y
          tck = splrep(w,s/d,s=0,k=3)
          s2 = splev(arange(5200,9000,10,Float),tck)*15
          qdump("old.fits",s2,extras=[("crval1",5200),("cdelt1",10),("crpix1",1),("dc-flag",0)])
          x = arange(0,W.shape[0],1,Float32)
          u = between(W2,5200,9000,inc=1)
          Y = compress(u,[x,W2,D2,S,N],1)
          x,w,d,s,n = Y
          tck = splrep(w,s/d,s=0,k=3)
          s2 = splev(arange(5200,9000,10,Float),tck)*15
          qdump("new.fits",s2,extras=[("crval1",5200),("cdelt1",10),("crpix1",1),("dc-flag",0)])
          raw_input("wait")
       X[3] = W2.astype(X[3].typecode())
       X[4] = D2.astype(X[4].typecode())
   return X
  def ReadGalaxy(self,n,verb=1,nodata=0):
      where = self.where[n]
      rows = [self.what[s].index(n) for s in where]
      if verb:
        print n
        print where
        print rows
      times = [self.times[s] for s in where]
      if nodata:
         T = times
         W = where
         return W,T
      else:
         X = [self.Shift(self.Cull(self.Data[w][::,self.what[w].index(n),::]),verb=verb) for w in where]
         T = times
         W = where
         return W,X,T

def Gaussian(p,x,w,d,s,n):
   a,c,ls,b1,b2 = p
   m = exp10(a)*exp(-0.5*power(x-c,2)/power(exp10(ls),2)) + b1 + b2*(x-x[0])
   return divz(s-m,n)

class Galaxy(Persistent):
  def __init__(self,n,SpecFiles,Temps,QE,QE0,SMF,w1=4000,w2=9000,sex=None,flux=0,verb=0,summary=0,noav=0):
      self.SpecFiles = SpecFiles
      self.oname = n
      M = len(self.SpecFiles.where[n])
      self.SMF = SMF
      self.flux = flux
      self.QE = QE
      self.QE0 = QE0
      self.Temps = Temps
      if self.Temps: self.gridname = Temps.name
      self.showcurves = False
      self.verb = verb
      self.specz = (None,-999)

      self.where = self.SpecFiles.where[n]

      if verb:
        print "Reading galaxy",n

      if sex:
         self.sex = True
         nn = int(n[1:])
         nn1 = searchsorted(sex.alias[1],nn)
         nn = sex.alias[0][nn1]
#         print sex.alias[1][nn1], sex.alias[0][nn1]
         self.photfilt = sex.filters
         self.tranfilt = sex.trans
         self.sexdata = {}
         self.magdata = {}
         self.cordata = {}
         self.specdata = {}
         for key in sex.sex:
            self.sexdata[key] = sex.sex[key][nn-1]
         for key in sex.mag:
            self.magdata[key] = {}
            for key2 in sex.mag[key]:
               self.magdata[key][key2] = sex.mag[key][key2][nn-1]
         if "naps" in sex.cor:
            self.cordata["naps"] = sex.cor["naps"]
            self.cordata["apcor"] = sex.cor["apcor"][nn-1]
            self.cordata["aperr"] = sex.cor["aperr"][nn-1]
         self.trans = sex.trans
         if sex.specz:
            si = self.SMF["id"].index(self.oname)
            ra = hrrad(scanSexagesimal(self.SMF["ra"][si]))
            dec = degrad(scanSexagesimal(self.SMF["dec"][si]))
            dra = (ra-sex.specz["ra"])/cos(sex.specz["dec"])
            ddec = (dec-sex.specz["dec"])
            ds = sqrt(dra**2+ddec**2) * 3600*180/pi
            m = argmin(ds)
            if ds[m] < 1.5:
              self.specz = (sex.specz["id"][m], sex.specz["z"][m])
      else:
         self.sex = False

      if summary:
            self.restored = False
      else:

            W,X,etimes = self.SpecFiles.ReadGalaxy(n,verb=verb)
            ospecs,onoises,oskys,owaves,odisps = [[X[m][j] for m in range(M)] for j in range(5)]

      #      onoises = [getsqrt( \
      #              (ospec/2.+osky)/0.85 + 2*pow(6/0.85,2) + \
      #              (ospec/2.+osky)/0.85 + 2*pow(6/0.85,2) ) for ospec,osky in zip(ospecs,oskys)]

            self.etimes = etimes
            self.where = W

            oxs = [arange(ospec.shape[0]) for ospec in ospecs]
            oxes = [0.5*ones(ox.shape) for ox in oxs]

            dowaves = [diff(owave) for owave in owaves]
            dowaves = [concatenate([dowave[:1],dowave]) for dowave in dowaves]
            nn = [sum(less_equal(diff(owave),0)) for owave,ospec,odisp,etime in zip(owaves,ospecs,odisps,etimes)]

            while sum(nn):
               goods = [greater(onoise,0)*greater(odisp,0)*greater(dowave,0) \
                         for onoise,odisp,dowave in zip(onoises,odisps,dowaves)]
               oxs = [compress(good,ox) for good,ox in zip(goods,oxs)]
               oxes = [compress(good,oxe) for good,oxe in zip(goods,oxes)]
               ospecs = [compress(good,ospec) for good,ospec in zip(goods,ospecs)]
               onoises = [compress(good,onoise) for good,onoise in zip(goods,onoises)]
               oskys = [compress(good,osky) for good,osky in zip(goods,oskys)]
               owaves = [compress(good,owave) for good,owave in zip(goods,owaves)]
               odisps = [compress(good,odisp) for good,odisp in zip(goods,odisps)]
               dowaves = [diff(owave) for owave in owaves]
               dowaves = [concatenate([dowave[:1],dowave]) for dowave in dowaves]
               nn = [sum(less_equal(diff(owave),0)) for owave,ospec,odisp,etime in zip(owaves,ospecs,odisps,etimes)]

            QE0w1,QE0w2, = extrema(QE0.qtck[0])
            QEw1,QEw2, = extrema(QE.qtck[0])

            oqe0s = [QE0.interp(owave) for owave in owaves]
            oqes = [QE.interp(owave) for owave in owaves]

            self.xs = oxs
            self.xes = oxes
            self.specs = ospecs
            self.noises = onoises
            self.specas = [divz(ospec,odisp) for ospec,odisp in zip(ospecs,odisps)]
            self.noiseas = [divz(onoise,odisp) for onoise,odisp in zip(onoises,odisps)]

            self.skys = oskys
            self.waves = owaves
            self.disps = odisps
            self.qe0s = oqe0s
            self.qes = oqes
            self.rqes = [ones(qe.shape) for qe in self.qes]
            abands = [1.*less(owave-2*odisp,7550)*greater(owave+2*odisp,7750) for owave,odisp in zip(owaves,odisps)]
            self.okeeps = [logical_not(aband) for aband in abands]

            nrit = 1

            for rit in range(nrit):
               if rit:
                  if rit == nrit-1: nt = 2
                  else: nt = 4
                  self.Average(n=nt)
                  omods = [etime*rqe*oqe*splev(owave,self.atck)*odisp for owave,odisp,oqe,rqe,etime in zip(owaves,odisps,oqes,self.rqes,etimes)]
                  omons = [etime*rqe*oqe*splev(owave,self.ntck)*odisp for owave,odisp,oqe,rqe,etime in zip(owaves,odisps,oqes,self.rqes,etimes)]
                  if rit == nrit-1: fcut = 3.0
                  else: fcut = 4.0
                  self.okeeps = [not_equal(ospec,0)*greater(ospec,-1*onoise)*between(ospec-omod,-fcut*onoise,1.0*fcut*onoise) for owave,ospec,onoise,odisp,omod,omon in zip(owaves,ospecs,onoises,odisps,omods,omons)]
                  self.okeeps = [okeep*logical_not(aband) for okeep,aband in zip(self.okeeps,abands)]
                  self.Average(n=nt)
               else:
#                  otcks = [splrep(sort(owave),take(ospec/odisp/rqe/etime,argsort(owave)),s=0,k=1) for owave,ospec,odisp,rqe,etime in zip(owaves,ospecs,odisps,self.rqes,etimes)]
#                  ntcks = [splrep(sort(owave),take(onoise/odisp/rqe/etime,argsort(owave))**2,s=0,k=1) for owave,onoise,odisp,rqe,etime in zip(owaves,onoises,odisps,self.rqes,etimes)]
#                  omods = [rqe*etime*odisp*median([splev(owave,otck2) for otck2 in otcks if otck2 != otck1]) for owave,odisp,otck1,ntck1,rqe,etime in zip(owaves,odisps,otcks,ntcks,self.rqes,etimes)]
#                  omons = [etime*odisp*mean([splev(owave,ntck2) for ntck2 in ntcks if ntck2 != ntck1])/(len(ntcks)-1) for owave,odisp,otck1,ntck1,rqe,etime in zip(owaves,odisps,otcks,ntcks,self.rqes,etimes)]
#                  omods = [omod*sum(ospec*omod*between(owave,w1,w2)*odisp/onoise**2)/sum(omod**2*between(owave,w1,w2)*odisp/onoise**2) for owave,ospec,onoise,odisp,omod,omon in zip(owaves,ospecs,onoises,odisps,omods,omons)]
#                  omons = [omon*sum(ospec*omod*between(owave,w1,w2)*odisp/onoise**2)/sum(omod**2*between(owave,w1,w2)*odisp/onoise**2) for owave,ospec,onoise,odisp,omod,omon in zip(owaves,ospecs,onoises,odisps,omods,omons)]
                  omods = [VTKMedian(ospec,5,1,1) for owave,ospec,onoise,odisp in zip(owaves,ospecs,onoises,odisps)]
                  omons = [onoise for owave,ospec,onoise,odisp in zip(owaves,ospecs,onoises,odisps)]
                  fcut = 4
                  self.okeeps = [not_equal(ospec,0)*greater(ospec,-1*onoise)*between(ospec-omod,-fcut*onoise,fcut*onoise) for owave,ospec,onoise,odisp,omod,omon in zip(owaves,ospecs,onoises,odisps,omods,omons)]
                  self.okeeps = [okeep*logical_not(aband) for okeep,aband in zip(self.okeeps,abands)]
               if sex and self.flux and self.trans:
                  self.Flux(rit)

            if noav: status = 0
            else: status = self.Average(n=2)
            if status == 0: self.okay = True
            else: self.okay = False


            self.match = -1
            self.units = 'flambda'
            self.pixels = 'lambda'

            self.w10 = w1
            self.w20 = w2
            self.w1 = w1
            self.w2 = w2

            self.top = 0
            self.bot = 0

            self.down = 0
            self.restored = False
            self.avshow = False

  def Flux(self,R):

      ftest = [GetFilter(self.trans[f].name,self.trans[f].detector.name,ab=0) for f in self.photfilt]
      abtest = [f.conv_AB for f in ftest]
      wleff = array([f.wleff for f in ftest])
      lwleff = log10(wleff)
      omags = array([self.magdata[f]["mag3"] for f in self.photfilt])
      emags = array([self.magdata[f]["err3"] for f in self.photfilt])
      fab = [(1-self.trans[f].ab)*self.trans[f].conv_AB for f in self.photfilt]
      fnus = [CLIGHT/(self.trans[f].wwav*1e-8) for f in self.photfilt]
      dtcks = [splrep(CLIGHT/(self.trans[f].detector.dwwav*1e-8)[::-1],self.trans[f].detector.dewav[::-1],s=0,k=1) for f in self.photfilt]
      dets = [splev(fnu[::-1],dtck) for dtck,fnu in zip(dtcks,fnus)]
      ofnu = exp10(-0.4*(omags+48.60))
      oflam = ofnu*array([CLIGHT/power(self.trans[f].wleff*1e-8,2) for f in self.photfilt]) * 1e-8

      vpnu = CLIGHT/(VEGA.lam*1e-8)
      vpenergy = PLANCK*vpnu
      vpflux = (VEGA.flam / vpenergy) / power(vpnu,2) * CLIGHT
      vptck = splrep(vpnu[::-1],vpflux[::-1],s=0,k=1)
      vffluxes = [splev(fnu[::-1],vptck)[::-1] for fnu in fnus]
      vfnumers = [vfflux*self.trans[f].ewav*det for vfflux,f,det in zip(vffluxes,self.photfilt,dets)]
      vfdenoms = [self.trans[f].ewav*det for fflux,f,det in zip(vffluxes,self.photfilt,dets)]
      vfnumers = [splint(fnu[-1],fnu[0],splrep(fnu[::-1],vfnumer[::-1],s=0,k=1)) for fnu,vfnumer in zip(fnus,vfnumers)]
      vfdenoms = [splint(fnu[-1],fnu[0],splrep(fnu[::-1],vfdenom[::-1],s=0,k=1)) for fnu,vfdenom in zip(fnus,vfdenoms)]
      vfmags = -2.5*getlog10(divz(vfnumers,vfdenoms)) - fab
      vfmags2 = array([self.trans[f].integrate(VEGA,logmult=0,straight=1) for f in self.photfilt])

      self.rqes = [ones(qe.shape) for qe in self.qes]
      J = arange(len(self.specs))
      if R: mit = 4
      else: mit = 1
      for rit in range(mit):
         for j1,where,okeep,owave,odisp,ospec,onoise,oqe,rqe,etime in zip(J,self.where,self.okeeps,self.waves,self.disps,self.specs,self.noises,self.qes,self.rqes,self.etimes):
             self.rqes[j1] = ones(okeep.shape,Float32)
             x,xe,y,ye,dx = compress(okeep,[owave,odisp,divz(ospec,odisp*etime*rqe*oqe),divz(onoise,odisp*etime*rqe*oqe),odisp],1)
             if len(x):
                pnu = CLIGHT/(x*1e-8)
                penergy = PLANCK*pnu
                pflux = (y / penergy) / power(pnu,2) * CLIGHT
                ptck = splrep(pnu[::-1],pflux[::-1],s=0,k=1)
                ffluxes = [splev(fnu[::-1],ptck)[::-1] for fnu in fnus]
                fnumers = [fflux*self.trans[f].ewav*det for fflux,f,det in zip(ffluxes,self.photfilt,dets)]
                fdenoms = [self.trans[f].ewav*det for fflux,f,det in zip(ffluxes,self.photfilt,dets)]
                fnumers = [splint(fnu[-1],fnu[0],splrep(fnu[::-1],fnumer[::-1],s=0,k=1)) for fnu,fnumer in zip(fnus,fnumers)]
                fdenoms = [splint(fnu[-1],fnu[0],splrep(fnu[::-1],fdenom[::-1],s=0,k=1)) for fnu,fdenom in zip(fnus,fdenoms)]
                fmags = -2.5*getlog10(divz(fnumers,fdenoms)) - fab - vfmags
                FT = FauxTemplate( x,y)
                fmags2 = array([self.trans[f].integrate(FT,logmult=0,straight=1) for f in self.photfilt]) - vfmags2
                if sum(between(fmags2,10,35)) > sum(between(fmags,10,35)):
                   fmags = fmags2
                umags = between(fmags,10,35)
                if self.verb:
                  print rit+1,where,etime
                  print self.photfilt
                  print fmags
                  print omags
                  print fmags - omags
                  print umags
                if sum(umags) < 3:
                   self.rqes[j1] = ones(okeep.shape,Float32)
                else:
                   if R and rit == mit-1: omax = 0
                   else: omax = 0
                   ford = min([sum(umags)-1,omax])
                   wb = polynom(wleff,ford)
                   ff = exp10(-0.4*(fmags-25))
                   of = exp10(-0.4*(omags-25))
####                fsol = svdfit(transpose(wb)*(umags)[::,NewAxis],(fmags-omags)*(umags))
####                rqe = exp10(-0.4*dot(fsol,polynom(owave,ford)))
                   fsol = svdfit(transpose(wb)*(of*umags)[::,NewAxis],(ff*umags))
                   rqe = dot(fsol,polynom(owave,ford))
                   self.rqes[j1] = rqe * self.rqes[j1]
                   if self.verb:
                     print fsol
                     print mean(compress(greater(owave,5500),rqe)),
                     print extrema(rqe)
                     print

  def Average(self,n=1):
      x = self.waves
      y = [divz(ospeca,etime*oqe*rqe) for ospeca,oqe,rqe,etime in zip(self.specas,self.qes,self.rqes,self.etimes)]
      ey = [divz(onoisea,etime*oqe*rqe) for onoisea,oqe,rqe,etime in zip(self.noiseas,self.qes,self.rqes,self.etimes)]
      k = concatenate(self.okeeps)
      x = concatenate(x)
      y = concatenate(y)
      ey = concatenate(ey)
      A = compress(k*ey,[x,y,ey],1).astype(Float64)
      A = take(A,argsort(A[0]),1)
      x,y,ey = adjspac(A,verb=0)
      uw = argmax([diff(extrema(wave))[0] for wave in self.waves])
      tx = compress(self.okeeps[uw],self.waves[uw])
      tx = tx[::n]
      ntx = diff(searchsorted(x,tx))
      htx = concatenate([ntx,ntx[-1:]])
      tx = compress(greater(htx,0),tx)
#      tx = diff(tx)/2. + tx[:-1]
      if len(tx) > 3:
        self.atck = splrep(x,y,1/ey,s=0,k=1,task=-1,nest=2*len(tx),t=tx,xb=x[0],xe=x[-1],quite=1)
        self.ntck = splrep(x,ey**2,1/ey,s=0,k=1,task=-1,nest=2*len(tx),t=tx,xb=x[0],xe=x[-1],quite=1)
        self.avwave = self.waves[uw]
        self.avdisp = self.disps[uw]
        avspec = splev(self.avwave,self.atck)
        avnoise = sqrt(splev(self.avwave,self.ntck))
        self.avspec = avspec
        self.avnoise = avnoise
        return 0
      else:
        self.avwave = None
        self.avdisp = None
        self.avspec = None
        self.avnoise = None
        return -1

  def Pull(self):
      self.Temps.Pull(self.waves,self.disps,self.qes,self.rqes,self.qe0s,self.etimes)

  def Save(self):
      if self.Temps:
         if not os.path.exists("savefiles"): os.mkdir("savefiles")
         savefile = "savefiles/%s-%s.save" % (self.SMF.name,self.oname)
         print "Writing to",savefile
         D = {}
         for item in SAVEITEMS: D[item] = eval("self.%s" % (item))
         f = open(savefile,"w")
         cP.dump(D,f,-1)
         f.close()

  def Restore(self,clean,keep=SAVEITEMS):
      savefile = "savefiles/%s-%s.save" % (self.SMF.name,self.oname)
      if not clean and os.path.exists(savefile) :
         f = open(savefile,"r")
         try: D=cP.load(f)
         except: D={}
         f.close()
         if len(D.keys()) != len(SAVEITEMS): return False
         if keep == SAVEITEMS:
            print "Reading:",
         for item in keep:
            i = "self.%s = D[\"%s\"]" % (item,item)
            if keep == SAVEITEMS: print item,
            exec(i)
         self.restored = True
         if keep == SAVEITEMS:
            print
            self.bestfit()
         return True
      else:
         return False

  def Compute(self):
      Temps = self.Temps

      self.chi_3d,ndeg,self.prob_3d,match = Temps.Push(self)

      rchi2 = self.chi_3d.flat / ndeg
      zerochi2 = min(self.chi_3d.flat)
      emission = self.escale * mean(self.gscales)

      nsig = 5
      dchi2 = chdtri(2,1-chdtr(1,power(1+arange(nsig),2)))

      self.chi_zt = minimum.reduce(self.chi_3d,0)
      self.chi_tZ = minimum.reduce(self.chi_3d,-1)

      self.prob_zt = add.reduce(self.prob_3d*Temps.dZ,0)
      self.prob_tZ = add.reduce(self.prob_3d*Temps.dz,-1)

      mchi2 = min(self.chi_3d.flat)
      mrchi2 = min(rchi2.flat)

      self.prob_z = add.reduce(add.reduce(self.prob_3d*Temps.dZ,0)*Temps.dT,0)*Temps.dz
      self.prob_t = add.reduce(add.reduce(self.prob_3d*Temps.dZ,0)*Temps.dz,-1)*Temps.dT
      self.prob_Z = add.reduce(add.reduce(self.prob_3d*Temps.dz,-1)*Temps.dT,-1)*Temps.dZ

      self.match = match
#      print self.escale[self.match]

      self.mchi2 = mchi2
      self.dchi2 = dchi2
      self.rchi2 = rchi2

  def Fit(self):
      self.Compute()

      oname = self.oname

      oxs = self.xs
      oxes = self.xes
      ospecs = self.specs
      onoises = self.noises
      oskys = self.skys
      owaves = self.waves
      odisps = self.disps
      oqe0s = self.qe0s
      oqes = self.qes
      rqes = self.rqes

      ospecas = self.specas
      onoiseas = self.noiseas

      okeeps = self.okeeps

      Temps = self.Temps

      ndiv = 1.

      if len(Temps.redshifts) > 5: kz = 3
      elif len(Temps.redshifts) > 3: kz = 3
      else: kZ = 1
      if len(Temps.logages) > 5: kT = 3
      elif len(Temps.logages) > 3: kT = 3
      else: kZ = 1
      if len(Temps.metals) > 5: kZ = 3
      elif len(Temps.metals) > 3: kZ = 3
      else: kZ = 1

      ptck_z = splrep(Temps.redshifts,getlog10(self.prob_z,rep=log10(min(compress(self.prob_z,self.prob_z)))-1.0),s=0,k=kz)
      p_z = arange(Temps.redshifts[0],Temps.redshifts[-1]+Temps.dz/ndiv,Temps.dz/ndiv,Float)
      sprob_z = exp10(splev(p_z,ptck_z))/ndiv
      ptck_z = splrep(p_z,sprob_z,s=0,k=kz)
#      cprob_z = array([splint(0,zz,ptck_z) for zz in p_z])
      cprob_z = add.accumulate([Temps.dz*splev(zz,ptck_z) for zz in p_z])
      cprob_z = divz(cprob_z , cprob_z[-1])
      ctck_z = splrep(p_z,cprob_z,s=0,k=kz)

      mp_p = [0.025,0.975]
      mp_z = p_z[argmax(sprob_z)]
      peak_z = fsolve(lambda z: splev(z,ptck_z,1),[mp_z])
      conf_z = array([fsolve(lambda z: splev(z,ctck_z)-mp,[mp_z]) for mp in mp_p])

      ptck_t = splrep(Temps.logages,getlog10(self.prob_t,rep=log10(min(compress(self.prob_t,self.prob_t)))-1.0),s=0,k=kT)
      p_t = arange(Temps.logages[0],Temps.logages[-1]+Temps.dT/ndiv,Temps.dT/ndiv,Float)
      sprob_t = exp10(splev(p_t,ptck_t))/ndiv
      ptck_t = splrep(p_t,sprob_t,s=0,k=kT)
#      cprob_t = array([splint(0,tt,ptck_t) for tt in p_t])
      cprob_t = add.accumulate([Temps.dT*splev(tt,ptck_t) for tt in p_t])
      cprob_t = divz(cprob_t, cprob_t[-1])
      ctck_t = splrep(p_t,cprob_t,s=0,k=kT)

      mp_p = [0.025,0.975]
      mp_t = p_t[argmax(sprob_t)]
      peak_t = fsolve(lambda t: splev(t,ptck_t,1),[mp_t])
      conf_t = array([fsolve(lambda t: splev(t,ctck_t)-mp,[mp_t]) for mp in mp_p])

      ptck_Z = splrep(Temps.metals,getlog10(self.prob_Z,rep=log10(min(compress(self.prob_Z,self.prob_Z)))-1.0),s=0,k=kZ)
      p_Z = arange(Temps.metals[0],Temps.metals[-1]+Temps.dZ/ndiv,Temps.dZ/ndiv,Float)
      sprob_Z = exp10(splev(p_Z,ptck_Z))/ndiv
      ptck_Z = splrep(p_Z,sprob_Z,s=0,k=kZ)
#      cprob_Z = array([splint(0,ZZ,ptck_Z) for ZZ in p_Z])
      cprob_Z = add.accumulate([Temps.dZ*splev(ZZ,ptck_Z) for ZZ in p_Z])
      cprob_Z = divz(cprob_Z, cprob_Z[-1])
      ctck_Z = splrep(p_Z,cprob_Z,s=0,k=kZ)

      mp_p = [0.025,0.975]
      mp_Z = p_Z[argmax(sprob_Z)]
      peak_Z = fsolve(lambda Z: splev(Z,ptck_Z,1),[mp_Z])
      conf_Z = array([fsolve(lambda Z: splev(Z,ctck_Z)-mp,[mp_Z]) for mp in mp_p])

      self.p_z = p_z
      self.sprob_z = sprob_z
      self.cprob_z = cprob_z
      self.conf_z = conf_z
      self.peak_z = peak_z

      self.p_t = p_t
      self.sprob_t = sprob_t
      self.cprob_t = cprob_t
      self.conf_t = conf_t
      self.peak_t = peak_t

      self.p_Z = p_Z
      self.sprob_Z = sprob_Z
      self.cprob_Z = cprob_Z
      self.conf_Z = conf_Z
      self.peak_Z = peak_Z

      self.bestfit()

  def bestfit(self):
      print "%-7s" % (self.oname),
      print "%7.3f %5.3f %5.3f" % (self.peak_z, self.conf_z[0], self.conf_z[1]),
      print "%7.3f %5.3f %5.3f" % (self.peak_t, self.conf_t[0], self.conf_t[1]),
      print "%7.3f %5.3f %5.3f" % (self.peak_Z, self.conf_Z[0], self.conf_Z[1]),
      print self.escale[self.match],
      print self.escale[self.match] * mean([gscale[self.match] for gscale in self.gscales])
      print 8*" " + "%7.3f   " % (self.rchi2[self.match]),
      for scale in self.scales: print "%7.3f" % (scale[self.match]),
      for mscale in self.mscales: print "%7.3f" % (mscale[self.match]),
      for gscale in self.gscales: print "%7.3f" % (getlog10(gscale[self.match])),
      print

  def display(self,onlyshow):

      oname = self.oname
      oxs = self.xs
      oxes = self.xes
      oskys = self.skys
      owaves = self.waves
      odisps = self.disps
      oqe0s = self.qe0s
      oqes = self.qes
      rqes = self.rqes
      okeeps = self.okeeps
      etimes = self.etimes

      if self.Temps:
         match = self.match
         gscales = self.gscales
         scales = divz(gscales , mean(gscales))
         scales = [scale[match] for scale in scales]
         if sum(scales) == 0:
            scales = [1. for scale in scales]
         oweights = self.weights
      else:
         scales = ones((len(self.specs),))
         oweights = [ones(spec.shape,Float32) for spec in self.specs]

      ospecs = [divz(spec,scale) for spec,scale in zip(self.specs,scales)]
      onoises = [divz(noise,scale) for noise,scale in zip(self.noises,scales)]
      ospecas = [divz(speca,scale) for speca,scale in zip(self.specas,scales)]
      onoiseas = [divz(noisea,scale) for noisea,scale in zip(self.noiseas,scales)]

      self.xmar = extrema(array([[ox[0]-1,ox[-1]+1] for ox in oxs]).flat)
      self.wmar = [3880.,11.0e3]

      smars = []
      samars = []
      sdmars = []
      for etime,okeep,oweight,ospec,onoise,ospeca,onoisea,oqe,rqe,oqe0 in zip(etimes,okeeps,oweights,ospecs,onoises,ospecas,onoiseas,oqes,rqes,oqe0s):
         kospec,konoise,kospeca,konoisea,koqe,krqe,koqe0 = compress(okeep*oweight, [ospec,onoise,ospeca,onoisea,oqe,rqe,oqe0], 1)
         smars += [-2*getpct(konoise,95),1.5*getpct(kospec,98)]
         samars += [-2*getpct(divz(konoisea,koqe*krqe*etime),75),1.5*getpct(divz(kospeca,koqe*krqe*etime),95)]
         sdmars += [-2*getpct(konoisea,75),1.5*getpct(kospeca,95)]

      self.smar = extrema(asarray(smars).flat)
      self.samar = extrema(asarray(samars).flat)
      self.sdmar = extrema(asarray(sdmars).flat)

      if self.top: self.samar[1] = self.top
      if self.bot: self.samar[0] = self.bot

      if self.Temps:
         cpgsvp(0.10,0.23,0.75,0.95)
         self.showchi_zt(era=1,manvp=1,ch=0.65)

         cpgsvp(0.28,0.41,0.75,0.95)
         self.showchi_tZ(era=0,manvp=1,ch=0.65)

         cpgsvp(0.46,0.59,0.75,0.95)
         self.showpx("z",self.p_z,self.sprob_z,era=0,manvp=1,ch=0.65)
         subtitle(oname,ch=0.8)
     
         cpgsvp(0.64,0.77,0.75,0.95)
         self.showpx("logT",self.p_t,self.sprob_t,era=0,manvp=1,ch=0.65)
     
         cpgsvp(0.82,0.95,0.75,0.95)
         self.showpx("[Z]",self.p_Z,self.sprob_Z,era=0,manvp=1,ch=0.65)
  
      if self.Temps:
         cpgsvp(0.10,0.95,0.10,0.65)
         era = 0
      else:
         cpgsvp(0.10,0.95,0.10,0.95)
         era = 1
      self.showfit(onlyshow,era=era,manvp=1)

  def saveps(self,xdev):

      galps = "%s_fit.ps/vcps" % (self.oname)
      print "Saving %s" % (galps)
      galdev = opendev(galps,land=1,size=6)
      self.showfit(0,era=1,manvp=0,ch=1.5,save=1)
      closedev(sdev=galdev)
      cpgslct(xdev)

      galps = "%s_chi.ps/vcps" % (self.oname)
      print "Saving %s" % (galps)
      galdev = opendev(galps,land=1,size=6)
      self.showchi_zt(era=1,manvp=0,ch=1.5)
      closedev(sdev=galdev)
      cpgslct(xdev)

      galps = "%s_pz.ps/vcps" % (self.oname)
      print "Saving %s" % (galps)
      galdev = opendev(galps,land=1,size=6)
      self.showpx("z",self.p_z,self.sprob_z,era=0,manvp=1,ch=1.5)
      closedev(sdev=galdev)
      cpgslct(xdev)

  def showpx(self,plab,p_x,prob_x,era=0,manvp=1,ch=0.8):

      newbox([p_x[0],p_x[-1]],[0.0,1.5*max(prob_x)],ch=ch,manvp=manvp,era=era)
      xlabel(plab,ch=ch)
      ylabel("P(%s)d%s" % (plab,plab),ch=ch)
      drawln(p_x,prob_x,ls=1,hist=1)

  def showchi_zt(self,era=0,manvp=1,ch=0.8):

      chi_zt = self.chi_zt
      prob_zt = self.prob_zt
      dchi2 = self.dchi2

      z0 = self.Temps.z0
      z1 = self.Temps.z1
      dz = self.Temps.dz
      T0 = self.Temps.T0
      T1 = self.Temps.T1
      dT = self.Temps.dT

      newbox([z0,z1],[T0,T1],ch=ch,manvp=manvp,era=era)
      xlabel("z",ch=ch)
      ylabel("log T",ch=ch)
      za_trf = array([z0-dz,dz,0.0,T0-dT,0.0,dT])
#      cpggray(chi_zt, chi_zt.shape[1],chi_zt.shape[0],\
#                   1,chi_zt.shape[1],1,chi_zt.shape[0],\
#                   self.mchi2, self.mchi2+self.dchi2[-1], za_trf)
      cpggray(prob_zt, prob_zt.shape[1],prob_zt.shape[0],\
                   1,prob_zt.shape[1],1,prob_zt.shape[0],\
                   max(prob_zt.flat), 0.0, za_trf)
#      cpgcont(chi_zt, chi_zt.shape[1],chi_zt.shape[0],\
#                   1,chi_zt.shape[1],1,chi_zt.shape[0],\
#                   self.mchi2+self.dchi2, -len(self.dchi2), za_trf)
      newbox([z0,z1],[T0,T1],ch=ch,manvp=manvp,era=0)

  def showchi_tZ(self,era=0,manvp=1,ch=0.8):

      chi_tZ = self.chi_tZ
      prob_tZ = self.prob_tZ
      dchi2 = self.dchi2

      Z0 = self.Temps.Z0
      Z1 = self.Temps.Z1
      dZ = self.Temps.dZ
      T0 = self.Temps.T0
      T1 = self.Temps.T1
      dT = self.Temps.dT

      newbox([T0,T1],[Z0,Z1],ch=ch,manvp=manvp,era=era)
      xlabel("log T",ch=ch)
      ylabel("[Z]",ch=ch)
      za_trf = array([T0-dT,dT,0.0,Z0-dZ,0.0,dZ])
#      cpggray(chi_tZ, chi_tZ.shape[1],chi_tZ.shape[0],\
#                   1,chi_tZ.shape[1],1,chi_tZ.shape[0],\
#                   self.mchi2, self.mchi2+self.dchi2[-1], za_trf)
      cpggray(prob_tZ, prob_tZ.shape[1],prob_tZ.shape[0],\
                   1,prob_tZ.shape[1],1,chi_tZ.shape[0],\
                   max(prob_tZ.flat), 0.0, za_trf)
#      cpgcont(chi_tZ, chi_tZ.shape[1],chi_tZ.shape[0],\
#                   1,chi_tZ.shape[1],1,chi_tZ.shape[0],\
#                   self.mchi2+self.dchi2, -len(self.dchi2), za_trf)
      newbox([T0,T1],[Z0,Z1],ch=ch,manvp=manvp,era=0)


  def showfit(self,onlyshow,era=0,manvp=1,ch=0.9,save=0):

      oname = self.oname
      oxs = self.xs
      oxes = self.xes
      oskys = self.skys
      owaves = self.waves
      odisps = self.disps
      oqe0s = self.qe0s
      oqes = self.qes
      rqes = self.rqes
      okeeps = self.okeeps
      etimes = self.etimes

      if self.Temps:
         match = self.match
         gscales = self.gscales
         scales = divz(gscales, mean(gscales))
         scales = [scale[match] for scale in scales]
         if sum(scales) == 0:
            scales = [1. for scale in scales]
         oweights = self.weights
      else:
         scales = ones((len(self.specs),))
         oweights = [ones(spec.shape,Float32) for spec in self.specs]

      ospecs = [divz(spec,scale) for spec,scale in zip(self.specs,scales)]
      onoises = [divz(noise,scale) for noise,scale in zip(self.noises,scales)]
      ospecas = [divz(speca,scale) for speca,scale in zip(self.specas,scales)]
      onoiseas = [divz(noisea,scale) for noisea,scale in zip(self.noiseas,scales)]

      if self.Temps:
         ugxs = oxs
         ugws = self.Temps.gws
         ugds = self.Temps.gds
         ugms = [divz(glam[match],scale) for glam,scale in zip(self.Temps.glams,scales)]
      else:
         ugxs = []
         ugws = []
         ugds = []
         ugms = []

      if save:
        wmar = [self.w1,self.w2]
      else:
        wmar = self.wmar

      if self.pixels == 'lambda':
         xmar = wmar
         xlab = "Wavelength (\A)"
         uxs = owaves
         uxes = odisps
         gxs = ugws
      elif self.pixels == 'pixels':
         xmar = self.xmar
         xlab = "x (pixels)"
         uxs = oxs
         uxes = [2*oxe for oxe in oxes]
         gxs = ugxs

      if self.units == 'counts':
         ymar = self.smar
         ylab = "Counts pixel\u-1\d"
         uys = ospecs
         uyes = onoises
         gys = ugms
      elif self.units == 'acounts':
         ymar = self.sdmar
         ylab = "Counts \A\u-1\d pixel\u-1\d"
         uys = ospecas
         uyes = onoiseas
         gys = [divz(ugm,ugd) for ugm,ugd in zip(ugms,ugds)]
      elif self.units == 'flambda':
         ymar = self.samar
         ylab = "ergs cm\u-2\d \A\u-1\d s\u-1\d"
         uys = [divz(ospeca,etime*oqe*rqe) for ospeca,oqe,rqe,etime in zip(ospecas,oqes,rqes,etimes)]
         uyes = [divz(onoisea,etime*oqe*rqe) for onoisea,oqe,rqe,etime in zip(onoiseas,oqes,rqes,etimes)]
         uqes = [self.QE.interp(ugw) for ugw in ugws]
         urqes = [splev(ugw,splrep(owave,rqe,s=0,k=3)) for ugw,owave,rqe in zip(ugws,owaves,rqes)]
         gys = [divz(ugm,etime*urqe*uqe*ugd) for ugm,uqe,urqe,ugd,etime in zip(ugms,uqes,urqes,ugds,etimes)]

      newbox(xmar,ymar,ch=ch,manvp=manvp,era=era)
      xlabel(xlab,ch=ch)
      ylabel(ylab,ch=ch)
      if self.Temps:
         if onlyshow:
            subtitle("%s" % (self.where[onlyshow-1]),ch=0.8)
      else:
         if onlyshow:
            subtitle("%s (%s)" % (oname,self.where[onlyshow-1]),ch=0.8)
         else:
            subtitle(oname,ch=0.8)
      if onlyshow:
         okeep,ux,uxe,uy,uye = [x[onlyshow-1] for x in (okeeps,uxs,uxes,uys,uyes)]
         x,xe,y,ye = compress(okeep,[ux,uxe,uy,uye],1)
         drawpterr(x,xe/2.,y,ye,ch=3,ci=1)
      else:
         if self.avshow and self.avspec and self.units == 'flambda' and self.pixels == 'lambda':
#            drawln(self.avwave,self.avspec,ls=1,ci=7,lw=1,hist=1)
            drawpterr(self.avwave,self.avdisp/2.,self.avspec,self.avnoise,ci=7,ch=3)
         else:
           for okeep,ux,uxe,uy,uye in zip(okeeps,uxs,uxes,uys,uyes):
              x,xe,y,ye = compress(okeep,[ux,uxe,uy,uye],1)
              drawpterr(x,xe/2.,y,ye,ch=3,ci=1)
      if save:
         fitci = 1
         fitlw = 3
      else:
         fitci = 4
         fitlw = 1
         if onlyshow:
            okeep,ux,uxe,uy,uye = [x[onlyshow-1] for x in (okeeps,uxs,uxes,uys,uyes)]
            x,xe,y,ye = compress(1-okeep,[ux,uxe,uy,uye],1)
            drawpterr(x,xe/2.,y,ye,ch=3,ci=2)
         else:
           if self.avshow and self.avspec and self.units == 'flambda' and self.pixels == 'lambda':
              pass
           else:
              for okeep,ux,uxe,uy,uye in zip(okeeps,uxs,uxes,uys,uyes):
                 x,xe,y,ye = compress(1-okeep,[ux,uxe,uy,uye],1)
                 drawpterr(x,xe/2.,y,ye,ch=3,ci=2)
         drawln([self.w1,self.w1],[ymar[0],ymar[1]],ls=4,ci=3)
         drawln([self.w2,self.w2],[ymar[0],ymar[1]],ls=4,ci=3)

      if gxs:
         if onlyshow:
           gx,gy = [x[onlyshow-1] for x in (gxs,gys)]
           drawln(gx,gy,ls=1,ci=fitci,lw=fitlw,hist=1)
         else:
           for gx,gy in zip(gxs,gys):
              drawln(gx,gy,ls=1,ci=fitci,lw=fitlw,hist=1)

      if self.showcurves and self.pixels == 'lambda' and self.flux and self.trans:
          fws = []
          fes = []
          mes = -1e24
          for f in self.photfilt:
             fw = self.trans[f].wwav
             fe = self.trans[f].ewav
             if self.trans[f].detector:
                fe = fe * splev(fw,splrep(self.trans[f].detector.dwwav,self.trans[f].detector.dewav,s=0,k=1))
             fws.append(fw)
             fes.append(fe)
             mes = max([mes,max(fe)])
          for fw,fe in zip(fws,fes):
             drawln(compress(fe,fw),ymar[1]*compress(fe,fe)/mes,ls=4,ci=4)




class Summary:
   def __init__(self,onames,Specs,Templates,qe,qe0,SMF,sex=None,w1=4000,w2=9000,z1=0.79,z2=0.87,chicut=999.):
      savefile = lambda n: "savefiles/%s-%s.save" % (SMF.name,n)
      self.sex = sex
      self.SMF = SMF
      onames = [n for n in onames if os.path.exists(savefile(n))]
      print onames
      self.Gs = [Galaxy(n,Specs,Templates,qe,qe0,SMF,w1=w1,w2=w2,sex=self.sex,verb=0,summary=1) for n in onames]
      keys = ["p_z","conf_z","peak_z","conf_t","peak_t","conf_Z","peak_Z","escale","prob_3d",
              "sprob_z","prob_z","prob_t","prob_Z","chi_zt","chi_tZ","match","rchi2"]
      print "Restoring:"
      print keys
      [G.Restore(clean=0,keep=keys) for G in self.Gs]
      print "Restored!"
      self.Gs = [G for G in self.Gs if G.restored]
      print len(self.Gs)

      sgs = array([G.sexdata["CLASS_STAR"] for G in self.Gs])
      mag = array([G.magdata[self.sex.refmag]["mag3"] for G in self.Gs])
      lrchi2 = array([log10(G.rchi2[G.match]) for G in self.Gs])
      ukeep0 = greater(mag,-99)*less(sgs,0.98)
      ukeep0 = ukeep0 * less(lrchi2,4)
      ukeep = ukeep0
      for j in range(3):
        umag,ulrchi2 = compress(ukeep, [mag,lrchi2], 1)
        umgb = polynom(umag-22,2)
        self.chisol = svdfit(transpose(umgb),ulrchi2)
        umchi = dot(self.chisol,umgb)
        self.scatchi2 = bwt(ulrchi2-umchi)[1]
        print j+1,self.scatchi2
        mgb = polynom(mag-22,2)
        mchi = dot(self.chisol,mgb)
        ukeep = ukeep0 * less(lrchi2-mchi,2*self.scatchi2)

      for jG,G in enumerate(self.Gs):
          G.mchicut = mchi[jG]
          G.conf_z[0] = G.peak_z - sqrt((G.peak_z - G.conf_z[0])**2 + 0.015**2)
          G.conf_z[1] = G.peak_z + sqrt((G.peak_z - G.conf_z[1])**2 + 0.015**2)

      keep_mag = lambda G: -50 < G.magdata[self.sex.refmag]["mag3"] < 24.5
      self.Gs = [G for G in self.Gs if keep_mag(G)]
      print len(self.Gs)

#      keep_zerr = lambda G: G.conf_z[0]>=Templates.redshifts[1]/10. and diff(G.conf_z)[0]/2/(1+G.peak_z) < 0.50
      keep_zerr = lambda G: diff(G.conf_z)[0]/(1+mean(G.conf_z)) < 9.10
      keep_sg = lambda G: G.sexdata["CLASS_STAR"] < 0.98
      keep_chi = lambda G: getlog10(G.rchi2[G.match]) <= (log10(1.5e10)+0*(G.mchicut + 5.0*self.scatchi2))
      keep = lambda G: keep_zerr(G) and keep_sg(G) and keep_chi(G)
      self.Gs = [G for G in self.Gs if keep(G)]
      print len(self.Gs)

      self.member_prob = lambda G: sum(self.inside*G.prob_z)
      self.nonmember_prob = lambda G: sum(self.outside*G.prob_z)
      self.nonmembership = lambda G: self.nonmember_prob(G) > 0.90
      self.membership = lambda G: not self.nonmembership(G)
#      self.membership = lambda G: self.member_prob(G) > 0.90

      if z1 == z2:
         z1a,z2a = z1-0.015,z2+0.015
         for it in range(5):
            self.inside = between(Templates.redshifts,z1a,z2a,inc=1)
#            self.inside = between(self.Gs[0].p_z,z1a,z2a,inc=1)
            member = array([self.membership(G) for G in self.Gs])
            peak_z = array([G.peak_z for G in self.Gs])
            err_z = array([diff(G.conf_z)[0] for G in self.Gs])
            wt_z = member*less(err_z,0.03)/err_z**2
            mz = sum(wt_z*peak_z) / sum(wt_z)
            sz = sqrt(sum(wt_z*(peak_z-mz)**2) / sum(wt_z))
            sz = sum(wt_z*(err_z/2)) / sum(wt_z)

            print 1+it,mz,sz
            z1a,z2a = mz-5*sz,mz+5*sz
         z1,z2 = z1a,z2a

      self.inside = 1.0*between(Templates.redshifts,z1,z2,inc=1)
      self.outside = 1.0*logical_or(less(Templates.redshifts,z1),greater(Templates.redshifts,z2))
#      self.inside = between(self.Gs[0].p_z,z1,z2,inc=1)

      self.member = array([self.membership(G) for G in self.Gs])
      self.logt = array([G.peak_t for G in self.Gs])
      self.escale = array([G.escale[G.match] for G in self.Gs])
      print sum(self.member)

      self.speci = array([G.specz[0] for G in self.Gs])
      self.specz = array([G.specz[1] for G in self.Gs])

      self.Templates = Templates
      self.SMF = SMF

      self.z0,self.z1,self.dz = Templates.z0, Templates.z1, Templates.dz
      self.T0,self.T1,self.dT = Templates.T0, Templates.T1, Templates.dT
      self.Z0,self.Z1,self.dZ = Templates.Z0, Templates.Z1, Templates.dZ

      self.check(sys.stdout)

      io = open("temp_mem.dat","w")
      self.check(io,mem=1)
      io.close()
      io = open("temp_non.dat","w")
      self.check(io,mem=0)
      io.close()

      self.open()

      self.NofChi2()
      self.Nofz(z1,z2)
      self.NofT()
      self.Tvsz()
      self.ZvsT(mem=1)
      self.ZvsT(mem=0)
      self.XY()

      if self.sex:
         if sum(greater(self.specz,-99)):
            print "Number of z matches=",sum(greater(self.specz,-99))
            self.check(sys.stdout,red=1)
            self.CompareZ()
         self.RedshiftMag()
         for color in self.sex.colors:
             self.ColorMag(color)
         self.Stars()
         self.ChiMag()

      self.close()

   def check(self,io,mem=1,red=0):
      if mem: check = [G for G in self.Gs if self.membership(G)]
      else: check = [G for G in self.Gs if not self.membership(G)]
      Ggood05 = [G for G in self.Gs if (G.conf_z[1]-G.conf_z[0])/(1+mean(G.conf_z)) < 0.05]
      Ggood10 = [G for G in self.Gs if (G.conf_z[1]-G.conf_z[0])/(1+mean(G.conf_z)) < 0.10]
      Ggood15 = [G for G in self.Gs if (G.conf_z[1]-G.conf_z[0])/(1+mean(G.conf_z)) < 0.15]
      Ggood20 = [G for G in self.Gs if (G.conf_z[1]-G.conf_z[0])/(1+mean(G.conf_z)) < 0.20]
      if red:
         check = [G for G in self.Gs if G.specz[0]]
         Gin = [G for G in self.Gs if G.specz[0] and (G.conf_z[0] < G.specz[1] < G.conf_z[1])]
         Gout = [G for G in self.Gs if G.specz[0] and not (G.conf_z[0] < G.specz[1] < G.conf_z[1])]
         Gmem =[self.membership(G) for G in self.Gs if G.specz[0]]
      fmt = "%7s %5.2f %7.3f %7.3f %7.3f %6.2f %6.2f %6.2f %6.2f %s %s %s\n"
      for G in check:
         result = (G.oname,self.member_prob(G),G.peak_z,G.conf_z[0],G.conf_z[1],G.peak_t,diff(G.conf_t)[0]/2,G.rchi2[G.match],
                G.specz[1],str(G.conf_z[0] < G.specz[1] < G.conf_z[1])[0],str(self.membership(G))[0],G.specz[0])
         io.write(fmt % result)
      if red:
         print len(check),len(Gin),len(Gout)
      print len(self.Gs), len(Ggood05), len(Ggood10), len(Ggood15), len(Ggood20)

   def open(self):
      if self.sex:
         if sum(greater(self.specz,-99)): nx,ny=7,4
         else: nx,ny=7,3
      else:
         nx,ny=4,3
      opendev(PGPLOT_DEV,nx=nx,ny=ny,land=0,asp=1.*ny/nx,size=nx*3)

   def close(self):
      closedev()

   def Nofz(self,z1,z2):
       redshifts = self.Templates.redshifts
       prob_z = sum([divz(G.prob_z,sum(G.prob_z)) for G in self.Gs])
       prob_z_m = sum([self.membership(G)*divz(G.prob_z,sum(G.prob_z)) for G in self.Gs])
       prob_z_1 = sum([(G.peak_t<=9.0)*divz(G.prob_z,sum(G.prob_z)) for G in self.Gs])
       prob_z_2 = sum([(G.peak_t>9.0)*divz(G.prob_z,sum(G.prob_z)) for G in self.Gs])
       newbox([self.z0-self.dz/2,self.z1+self.dz/2],[0,max(prob_z)*1.1],ch=2.2)
       xlabel("z",ch=2.2)
       ylabel("N",ch=2.2)
       drawln(redshifts,prob_z,ls=1,hist=1)
       drawln(redshifts,prob_z_1,ls=1,hist=1,ci=4)
       drawln(redshifts,prob_z_2,ls=1,hist=1,ci=2)
       drawln(redshifts,prob_z_m,ls=1,hist=1,ci=3)

       newbox([z1-(z2-z1),z2+(z2-z1)],[0,max(prob_z)*1.1],ch=2.2)
       xlabel("z",ch=2.2)
       ylabel("N",ch=2.2)
       drawln([z1,z1],[0,2*max(prob_z)],ls=4)
       drawln([z2,z2],[0,2*max(prob_z)],ls=4)
       drawln(redshifts,prob_z,ls=1,hist=1)
       drawln(redshifts,prob_z_1,ls=1,hist=1,ci=4)
       drawln(redshifts,prob_z_2,ls=1,hist=1,ci=2)
       drawln(redshifts,prob_z_m,ls=1,hist=1,ci=3)

       redshifts = self.Gs[0].p_z
       prob_z = sum([divz(G.sprob_z,sum(G.sprob_z)) for G in self.Gs])
       prob_z_m = sum([self.membership(G)*divz(G.sprob_z,sum(G.sprob_z)) for G in self.Gs])
       prob_z_1 = sum([(G.peak_t<=9.0)*divz(G.sprob_z,sum(G.sprob_z)) for G in self.Gs])
       prob_z_2 = sum([(G.peak_t>9.0)*divz(G.sprob_z,sum(G.sprob_z)) for G in self.Gs])
       newbox([self.z0-self.dz/2,self.z1+self.dz/2],[0,max(prob_z)*1.1],ch=2.2)
       xlabel("z",ch=2.2)
       ylabel("N",ch=2.2)
       drawln(redshifts,prob_z,ls=1,hist=1)
       drawln(redshifts,prob_z_1,ls=1,hist=1,ci=4)
       drawln(redshifts,prob_z_2,ls=1,hist=1,ci=2)
       drawln(redshifts,prob_z_m,ls=1,hist=1,ci=3)

       newbox([z1-(z2-z1),z2+(z2-z1)],[0,max(prob_z)*1.1],ch=2.2)
       xlabel("z",ch=2.2)
       ylabel("N",ch=2.2)
       drawln([z1,z1],[0,2*max(prob_z)],ls=4)
       drawln([z2,z2],[0,2*max(prob_z)],ls=4)
       drawln(redshifts,prob_z,ls=1,hist=1)
       drawln(redshifts,prob_z_1,ls=1,hist=1,ci=4)
       drawln(redshifts,prob_z_2,ls=1,hist=1,ci=2)
       drawln(redshifts,prob_z_m,ls=1,hist=1,ci=3)

   def NofT(self):
       logages = self.Templates.logages
       prob_t = sum([divz(G.prob_t,sum(G.prob_t)) for G in self.Gs])
       prob_tm = sum([self.membership(G)*divz(G.prob_t,sum(G.prob_t)) for G in self.Gs])
       newbox([self.T0-self.dT/2,self.T1+self.dT/2],[0,max(prob_t)*1.1],ch=2.2)
       xlabel("log T",ch=2.2)
       ylabel("N",ch=2.2)
       drawln(logages,prob_t,ls=1,hist=1)
       drawln(logages,prob_tm,ls=1,hist=1,ci=3)

   def NofChi2(self):
       lrchi2 = array([log10(G.rchi2[G.match]) for G in self.Gs])

       dx = 0.05
       lx0 = arange(-2-dx/2,2,dx,Float32)
       lnrchi2 = getlog10(diff(searchsorted(sort(lrchi2),lx0)))
       lnrchi2a = getlog10(diff(searchsorted(sort(compress(less_equal(self.logt,9.0),lrchi2)),lx0)))
       lnrchi2b = getlog10(diff(searchsorted(sort(compress(greater(self.logt,9.0),lrchi2)),lx0)))
       lx = diff(lx0)/2 + lx0[:-1]

       newbox([-1.0,1.5],add([-2.5,0.3],max(lnrchi2)),ch=2.2,xlog=1,ylog=1)
       xlabel("\gx\u2\d\\b\d\gn\u",ch=2.2)
       ylabel("N",ch=2.2)
       drawln(lx,lnrchi2,ls=1,hist=1)
       drawln(lx,lnrchi2a,ls=1,hist=1,ci=4)
       drawln(lx,lnrchi2b,ls=1,hist=1,ci=2)

       Terr= array([diff(G.conf_t)[0] for G in self.Gs])
       Zerr= array([diff(G.conf_Z)[0] for G in self.Gs])

       newbox([-1,1.5],[0.0,1.0],ch=2.2,xlog=1)
       xlabel("\gx\u2\d\\b\d\gn\u",ch=2.2)
       ylabel("\gDlog T",ch=2.2)
       x,y,w = compress(less_equal(self.logt,9.0),[lrchi2,Terr,Zerr],1)
       drawpt(x,y,ps=17,ch=1.5,ci=4)
       x,y,w = compress(greater(self.logt,9.0),[lrchi2,Terr,Zerr],1)
       drawpt(x,y,ps=17,ch=1.5,ci=2)

       newbox([-1,1.5],[0.0,1.0],ch=2.2,xlog=1)
       xlabel("\gx\u2\d\\b\d\gn\u",ch=2.2)
       ylabel("\gD[Z]",ch=2.2)
       x,y,w = compress(less_equal(self.logt,9.0),[lrchi2,Terr,Zerr],1)
       drawpt(x,w,ps=17,ch=1.5,ci=4)
       x,y,w = compress(greater(self.logt,9.0),[lrchi2,Terr,Zerr],1)
       drawpt(x,w,ps=17,ch=1.5,ci=2)


   def NofZ(self):
       prob_Z = sum([divz(G.prob_Z,sum(G.prob_Z)) for G in self.Gs])
       newbox([self.Z0-self.dZ/2,self.Z1+self.dZ/2],[0,max(prob_Z)*1.1],ch=2.2)
       xlabel("[Z]",ch=2.2)
       ylabel("N",ch=2.2)
       drawln(Templates.metals,prob_Z,ls=1,hist=1)

   def Tvsz(self):
       newbox([self.z0-self.dz/2,self.z1+self.dz/2],[self.T0-self.dT/2,self.T1+self.dT/2],ch=2.2)
       xlabel("z",ch=2.2)
       ylabel("log T",ch=2.2)
       prob_zts = [add.reduce(G.prob_3d*self.dZ,0) for G in self.Gs]
       pnorms = [add.reduce(add.reduce(p*self.dT,0)*self.dz,0) for p in prob_zts]
       prob_zt = zeros(prob_zts[0].shape,Float64)
       [add(prob_zt,divz(p,pn),prob_zt) for p,pn in zip(prob_zts,pnorms)]
       mp = max(compress(prob_zt.flat,prob_zt.flat))/2.
       cmp = [mp,1]
       print mp,sum(prob_zt.flat*self.dz*self.dT)
       za_trf = array([self.z0-self.dz,self.dz,0.0,self.T0-self.dT,0.0,self.dT])
       cpggray(prob_zt, prob_zt.shape[1],prob_zt.shape[0], 1,prob_zt.shape[1],1,prob_zt.shape[0], mp, cmp[-1], za_trf)
       newbox([self.z0-self.dz/2,self.z1+self.dz/2],[self.T0-self.dT/2,self.T1+self.dT/2],ch=2.2,era=0)

   def ZvsT(self,mem=0):
      newbox([self.T0-self.dT/2,self.T1+self.dT/2],[self.Z0-self.dZ/2,self.Z1+self.dZ/2],ch=2.2)
      xlabel("log T",ch=2.2)
      ylabel("[Z]",ch=2.2)
      if mem:
         prob_tZs = [add.reduce(G.prob_3d*self.dz,-1) for G in self.Gs if self.membership(G)]
      else:
         prob_tZs = [add.reduce(G.prob_3d*self.dz,-1) for G in self.Gs if not self.membership(G)]
      if len(prob_tZs):
         pnorms = [add.reduce(add.reduce(p*self.dT,0)*self.dZ,0) for p in prob_tZs]
         prob_tZ = zeros(prob_tZs[0].shape,Float64)
         [add(prob_tZ,divz(p,pn),prob_tZ) for p,pn in zip(prob_tZs,pnorms)]
         mp = max(prob_tZ.flat)/2.
         cmp = [mp,1]
         print mp,sum(prob_tZ.flat*self.dZ*self.dT)
         tZ_trf = array([self.T0-self.dT,self.dT,0.0,self.Z0-self.dZ,0.0,self.dZ])
         cpggray(prob_tZ, prob_tZ.shape[1],prob_tZ.shape[0], 1,prob_tZ.shape[1],1,prob_tZ.shape[0], mp, cmp[-1], tZ_trf)
         newbox([self.T0-self.dT/2,self.T1+self.dT/2],[self.Z0-self.dZ/2,self.Z1+self.dZ/2],ch=2.2,era=0)

   def XY(self):

      smfj = [self.SMF["id"].index(G.oname) for G in self.Gs]
      smfu = take(self.SMF["u"],smfj)
      smfv = take(self.SMF["v"],smfj)

      newbox([15,-15],[-15,15],ch=2.2)
      xlabel("x (arcmin)",ch=2.2)
      ylabel("y (arcmin)",ch=2.2)
      drawpt(smfu,smfv,ps=1,ch=0.5,ci=1)
      mu,mv = compress(self.member*less_equal(self.logt,9.),[smfu,smfv],1)
      drawpt(mu,mv,ps=17,ch=2.5,ci=4)
      mu,mv = compress(self.member*greater(self.logt,9.),[smfu,smfv],1)
      drawpt(mu,mv,ps=17,ch=2.0,ci=2)

   def ColorMag(self,color):
       bmag = color[-1]
       cmag = color[0]
       blab = "%s (mag)" % (bmag)
       clab = "%s-%s (mag)" % (cmag,bmag)
       bmags3 = array([G.magdata[bmag]["mag3"] for G in self.Gs])
       bmags1 = array([G.magdata[bmag]["mag3"] for G in self.Gs])
       cmags1 = array([G.magdata[cmag]["mag3"] for G in self.Gs])
       color = cmags1-bmags1
       base = bmags3

       ubase,ucolor = compress(between(base,-99,99)*between(bmags1,-99,99)*between(cmags1,-99,99),[base,color],1)

       xm0 = getpct(ubase,2)
       xm1 = getpct(ubase,95)
       xm2 = (xm1-xm0)/4
       xm = xm0-xm2,xm1+xm2

       ym0 = getpct(ucolor,5)
       ym1 = getpct(ucolor,95)
       ym2 = (ym1-ym0)/3
       ym = ym0-ym2,ym1+ym2

       newbox(xm,ym,ch=2.2)
       xlabel(blab,ch=2.2)
       ylabel(clab,ch=2.2)
       drawpt(base,color,ps=1,ci=1)
       x,y = compress(self.member*less_equal(self.logt,9.),[base,color],1)
       drawpt(x,y,ps=17,ci=4)
       x,y = compress(self.member*greater(self.logt,9.),[base,color],1)
       drawpt(x,y,ps=17,ci=2)

   def Stars(self):
       bmag = self.sex.refmag
       blab = "%s (mag)" % (bmag)
       bmags = array([G.magdata[bmag]["mag3"] for G in self.Gs])
       lrchi2 = array([log10(G.rchi2[G.match]) for G in self.Gs])

       sa0 = arange(0,1,0.02,Float32)
       sa = diff(sa0)/2. + sa0[:-1]
       sg = array([G.sexdata["CLASS_STAR"] for G in self.Gs])
       nsa = 1.*diff(searchsorted(sort(sg),sa0))
       nsa = nsa / sum(nsa)

       newbox([0,1],[0,1.1*max(nsa)],ch=2.2)
       xlabel("CLASS_STAR",ch=2.2)
       ylabel("N",ch=2.2)
       drawln(sa,nsa,ls=1,ci=1,hist=1)
       usg = compress(self.member*less_equal(self.logt,9.),sg)
       nsa = 1.*diff(searchsorted(sort(usg),sa0))
       nsa = nsa / sum(nsa)
       drawln(sa,nsa,ls=1,ci=4,hist=1)
       usg = compress(self.member*greater(self.logt,9.),sg)
       nsa = 1.*diff(searchsorted(sort(usg),sa0))
       nsa = nsa / sum(nsa)
       drawln(sa,nsa,ls=1,ci=2,hist=1)

       sg = array([G.sexdata["CLASS_STAR"] for G in self.Gs])

       umags = compress(greater(bmags,-99),bmags)
       xm0 = getpct(umags,2)
       xm1 = getpct(umags,95)
       xm2 = (xm1-xm0)/4
       xm = xm0-xm2,xm1+xm2
       ym = (0,1)

       newbox(xm,ym,ch=2.2)
       xlabel(blab,ch=2.2)
       ylabel("CLASS_STAR",ch=2.2)
       drawpt(bmags,sg,ps=1,ci=1)
       x,y = compress(self.member*less_equal(self.logt,9.),[bmags,sg],1)
       drawpt(x,y,ps=17,ci=4)
       x,y = compress(self.member*greater(self.logt,9.),[bmags,sg],1)
       drawpt(x,y,ps=17,ci=2)

#       xm = (-1,1.5)
#       newbox(xm,ym,ch=2.2,xlog=1)
#       xlabel("\gx\u2\d\\b\d\gn\u",ch=2.2)
#       ylabel("CLASS_STAR",ch=2.2)
#       drawpt(lrchi2,sg,ps=1,ci=1)
#       x,y = compress(self.member*less_equal(self.logt,9.),[lrchi2,sg],1)
#       drawpt(x,y,ps=17,ci=4)
#       x,y = compress(self.member*greater(self.logt,9.),[lrchi2,sg],1)
#       drawpt(x,y,ps=17,ci=2)


   def CompareZ(self):
       bmag = self.sex.refmag
       blab = "%s (mag)" % (bmag)
       bmags = array([G.magdata[bmag]["mag3"] for G in self.Gs])
       peak_z = array([G.peak_z for G in self.Gs])
       err1_z,err2_z = abs(transpose([G.conf_z for G in self.Gs]) - peak_z[NewAxis,::])/peak_z[NewAxis,::]
       spec_z = array([G.specz[1] for G in self.Gs])
       off_z = peak_z - spec_z

       xm = (0,1.2)
       ym = (0,1.2)
       newbox(xm,ym,ch=2.2)
       xlabel("z\dhi\u",ch=2.2)
       ylabel("z\dLDP\u",ch=2.2)
       drawpt(spec_z,peak_z,ps=1,ci=1)
       x,y,e1,e2 = compress(self.member*less_equal(self.logt,9.),[spec_z,peak_z,err1_z,err2_z],1)
       drawpt(x,y,ps=17,ci=4)
       x,y,e1,e2 = compress(self.member*greater(self.logt,9.),[spec_z,peak_z,err1_z,err2_z],1)
       drawpt(x,y,ps=17,ci=2)

       xm = (0,1.2)
       ym = (0,1.2)
       newbox(xm,ym,ch=2.2)
       xlabel("z\dhi\u",ch=2.2)
       ylabel("z\dLDP\u",ch=2.2)
       drawpt(spec_z,peak_z,ps=1,ci=1)
       drawpterr2(spec_z,0*spec_z,0*spec_z,peak_z,err1_z,err2_z,ci=1)

       ym = (-1,1)
       newbox(xm,ym,ch=2.2)
       xlabel("z\dhi\u",ch=2.2)
       ylabel("\gDz",ch=2.2)
       drawpt(spec_z,off_z,ps=1,ci=1)
       x,y,e1,e2 = compress(self.member*less_equal(self.logt,9.),[spec_z,off_z,err1_z,err2_z],1)
       drawpt(x,y,ps=17,ci=4)
       x,y,e1,e2 = compress(self.member*greater(self.logt,9.),[spec_z,off_z,err1_z,err2_z],1)
       drawpt(x,y,ps=17,ci=2)

       newbox(xm,ym,ch=2.2)
       xlabel("z\dhi\u",ch=2.2)
       ylabel("\gDz",ch=2.2)
       drawpt(spec_z,off_z,ps=1,ci=1)
       drawpterr2(spec_z,0*spec_z,0*spec_z,off_z,err1_z,err2_z,ps=1,ci=1)

       umags = compress(between(bmags,-99,99),bmags)

       xm0 = getpct(umags,2)
       xm1 = getpct(umags,95)
       xm2 = (xm1-xm0)/4
       xm = xm0-xm2,xm1+xm2
       ym = (-1,1)

       newbox(xm,ym,ch=2.2)
       xlabel(blab,ch=2.2)
       ylabel("\gDz",ch=2.2)
       drawpt(bmags,off_z,ps=1,ci=1)
       x,y,e1,e2 = compress(self.member*less_equal(self.logt,9.),[bmags,off_z,err1_z,err2_z],1)
       drawpt(x,y,ps=17,ci=4)
       x,y,e1,e2 = compress(self.member*greater(self.logt,9.),[bmags,off_z,err1_z,err2_z],1)
       drawpt(x,y,ps=17,ci=2)


       newbox(xm,ym,ch=2.2)
       xlabel(blab,ch=2.2)
       ylabel("\gDz",ch=2.2)
       drawpt(bmags,off_z,ps=1,ci=1)
       drawpterr2(bmags,0*bmags,0*bmags,off_z,err1_z,err2_z,ps=1,ci=1)

   def RedshiftMag(self):
       bmag = self.sex.refmag
       blab = "%s (mag)" % (bmag)
       bmags = array([G.magdata[bmag]["mag3"] for G in self.Gs])
       peak_z = array([G.peak_z for G in self.Gs])
       err1_z,err2_z = abs(transpose([G.conf_z for G in self.Gs]) - peak_z[NewAxis,::])/peak_z[NewAxis,::]
       umags = compress(between(bmags,-99,99),bmags)

       xm0 = getpct(umags,2)
       xm1 = getpct(umags,95)
       xm2 = (xm1-xm0)/4
       xm = xm0-xm2,xm1+xm2
       ym = (0,1.2)

       newbox(xm,ym,ch=2.2)
       xlabel(blab,ch=2.2)
       ylabel("z",ch=2.2)
       drawpt(bmags,peak_z,ps=1,ci=1)
#       drawpterr2(bmags,0*bmags,0*bmags,peak_z,err1_z,err2_z,ps=1,ci=1)
       x,y,e1,e2 = compress(self.member*less_equal(self.logt,9.),[bmags,peak_z,err1_z,err2_z],1)
       drawpt(x,y,ps=17,ci=4)
#       drawpterr2(x,0*x,0*x,y,e1,e2,ps=17,ci=4)
       x,y,e1,e2 = compress(self.member*greater(self.logt,9.),[bmags,peak_z,err1_z,err2_z],1)
       drawpt(x,y,ps=17,ci=2)
#       drawpterr2(x,0*x,0*x,y,e1,e2,ps=17,ci=2)

       ym = (-0.1,1.2)
       newbox(xm,ym,ch=2.2)
       xlabel(blab,ch=2.2)
       ylabel("F(\gDz\d95\u)",ch=2.2)
       ce = (err2_z + err1_z)/4.
       ma0 = arange(xm[0],xm[1]+0.1,0.3,Float)
       nma = 1.*diff(searchsorted(sort(bmags),ma0))
       ma = diff(ma0)/2. + ma0[:-1]

       x,y,c = compress(less_equal(ce,0.1),[bmags,peak_z,ce],1)
       c010 = divz(1.*diff(searchsorted(sort(x),ma0)),nma)
       x,y,c = compress(less_equal(ce,0.20),[bmags,peak_z,ce],1)
       c020 = divz(1.*diff(searchsorted(sort(x),ma0)),nma)
       drawln(ma,c010,ls=1,ci=1,hist=1)
       drawln(ma+0.02,c020,ls=2,ci=1,hist=1)
       print ma
       print c010
       print c020

       x,y,c = compress(greater(self.logt,9.),[bmags,peak_z,ce],1)
       nma = 1.*diff(searchsorted(sort(x),ma0))
       x,y,c = compress(greater(self.logt,9.)*less_equal(ce,0.1),[bmags,peak_z,ce],1)
       c010 = divz(1.*diff(searchsorted(sort(x),ma0)),nma)
       x,y,c = compress(greater(self.logt,9.)*less_equal(ce,0.20),[bmags,peak_z,ce],1)
       c020 = divz(1.*diff(searchsorted(sort(x),ma0)),nma)
       drawln(ma+0.04,c010,ls=1,ci=2,hist=1)
       drawln(ma+0.06,c020,ls=2,ci=2,hist=1)
       print ma
       print c010
       print c020

       x,y,c = compress(less_equal(self.logt,9.),[bmags,peak_z,ce],1)
       nma = 1.*diff(searchsorted(sort(x),ma0))
       x,y,c = compress(less_equal(self.logt,9.)*less_equal(ce,0.1),[bmags,peak_z,ce],1)
       c010 = divz(1.*diff(searchsorted(sort(x),ma0)),nma)
       x,y,c = compress(less_equal(self.logt,9.)*less_equal(ce,0.20),[bmags,peak_z,ce],1)
       c020 = divz(1.*diff(searchsorted(sort(x),ma0)),nma)
       drawln(ma+0.08,c010,ls=1,ci=4,hist=1)
       drawln(ma+0.10,c020,ls=2,ci=4,hist=1)
       print ma
       print c010
       print c020


#       x,y,c = compress(self.member*less_equal(self.logt,9.),[bmags,peak_z,ce],1)

#       drawpt(bmags,err1_z+err2_z,ps=1,ci=1)
#       x,y,e1,e2 = compress(self.member*less_equal(self.logt,9.),[bmags,peak_z,err1_z,err2_z],1)
#       drawpt(x,e1+e2,ps=17,ci=4)
#       x,y,e1,e2 = compress(self.member*greater(self.logt,9.),[bmags,peak_z,err1_z,err2_z],1)
#       drawpt(x,e1+e2,ps=17,ci=2)

   def ChiMag(self):
       bmag = self.sex.refmag
       blab = "%s (mag)" % (bmag)
       bmags = array([G.magdata[bmag]["mag3"] for G in self.Gs])
       lrchi2 = array([log10(G.rchi2[G.match]) for G in self.Gs])

       xm0 = getpct(bmags,2)
       xm1 = getpct(bmags,95)
       xm2 = (xm1-xm0)/4
       xm = xm0-xm2,xm1+xm2
       ym = (-1,1.5)
       xa = arange(xm[0]-5,xm[-1]+5,0.02,Float32)
       ya = dot(self.chisol,polynom(xa-22,2))

       newbox(xm,ym,ch=2.2,ylog=1)
       xlabel(blab,ch=2.2)
       ylabel("\gx\u2\d\\b\d\gn\u",ch=2.2)
       drawpt(bmags,lrchi2,ps=1,ci=1)
       x,y = compress(self.member*less_equal(self.logt,9.),[bmags,lrchi2],1)
       drawpt(x,y,ps=17,ci=4)
       x,y = compress(self.member*greater(self.logt,9.),[bmags,lrchi2],1)
       drawpt(x,y,ps=17,ci=2)
       drawln(xa,ya,ls=1,ci=3)
       drawln(xa,ya+self.scatchi2,ls=2,ci=3)
       drawln(xa,ya+2*self.scatchi2,ls=3,ci=3)


def Interact(n,onames,Specs,Templates,qe,qe0,SMF,w1=4000,w2=9000,sex=None,flux=0,clean=0,verb=0):

    n0 = -1
    yp,xp = 0,0
    key = None
    newfit = True
    newdisp = True
    onlyshow = 0

    Galaxies = {}

    nx,ny=1,1
    xdev = opendev(PGPLOT_DEV,nx=nx,ny=ny,land=0,size=12,asp=0.66,ask=0)

    while key != "q":
      if n not in Galaxies:
         Galaxies[n] = Galaxy(n,Specs,Templates,qe,qe0,SMF,sex=sex,flux=flux,w1=w1,w2=w2,verb=verb)
      G = Galaxies[n]
      if n != n0:
         onlyshow = 0
      if Templates:
         if n != n0:
            if G.Restore(clean) and Templates.name == G.gridname:
               newfit = False
            G.Pull()
            if G.restored:
               G.Temps.glams = [rflam*gscale[::,NewAxis]+eflam*(etime*gscale*G.escale)[::,NewAxis] for rflam,gscale,etime,eflam in zip(G.Temps.rflams,G.gscales,G.etimes,G.Temps.eflams)]
         if n != n0 or G.match<0:
            newfit = True
         if newfit:
            G.Fit()
      if n != n0 or newfit or newdisp:
         G.display(onlyshow)
      if onlyshow:
         gkeeps = G.okeeps[onlyshow-1]
      else: gkeeps = None
      if newfit:
         G.Save()
      newfit = False
      newdisp = False
      n0 = n
      key,xp,yp = Select(xp,yp)

      if key == "I":
         raise "UserInterrupt"

      if key == "n" and n != onames[-1]: n = onames[onames.index(n)+1]
      if key == "p" and n != onames[0]: n = onames[onames.index(n)-1]
      if key == "M":
         new = raw_input("Input Galaxy ID: ")
         if new in onames:
            print "Switching to galaxy",new
            n = new

      if key == "E":
         mnew = raw_input("Input M Estimator: ").split(",")
         if Templates and [m in globals() for m in mnew] == len(mnew)*[True]:
            Templates.MEST = ",".join(mnew)
            newfit = True

      if key in digits and int(key) <= len(G.where):
         onlyshow = int(key)
         newdisp = True

      if key in "!@#$%^&*()":
         G.down = list("!@#$%^&*()").index(key) + 1
         if G.down == 10: G.down = 0
         newfit = True

      if key == "Y":
         key,xp,yp = Select(xp,yp)
         if key == "t": G.top = yp; newdisp = True
         if key == "T": G.top = 2*G.top; newdisp = True
         if key == "b": G.bot = yp; newdisp = True
         if key == "B": G.bot = 2*G.bot; newdisp = True
         if key == "r": G.bot,G.top = 0,0; newdisp = True

      if key == "W":
         key,xp,yp = Select(xp,yp)
         if key == "1": G.w1 = xp; newfit = True
         if key == "2": G.w2 = xp; newfit = True
         if key == "r": G.w1,G.w2 = G.w10,G.w20; newfit = True

      if key == "C" and flux:
         G.showcurves = not G.showcurves
         newdisp = True

      if key == "A" and G.okay:
         G.Average()
         G.avshow = not G.avshow
         newdisp = True

      if key == "u":
         if G.units == 'counts': G.units = 'acounts'
         elif G.units == 'acounts': G.units = 'flambda'
         elif G.units == 'flambda': G.units = 'counts'
         newdisp = True

      if key == "w":
         if G.pixels == 'lambda': G.pixels = 'pixels'
         elif G.pixels == 'pixels': G.pixels = 'lambda'
         newdisp = True

      if G.pixels == 'lambda': Gxs = G.waves[onlyshow-1]
      elif G.pixels == 'pixels': Gxs = G.xs[onlyshow-1]

      if G.units == 'counts': Gys = G.specs[onlyshow-1]
      elif G.units == 'acounts': Gys = divz(G.specs[onlyshow-1], G.disps[onlyshow-1])
      elif G.units == 'flambda': Gys = divz(G.specs[onlyshow-1], G.disps[onlyshow-1], G.qes[onlyshow-1])

      if onlyshow and key == "D" and len(nonzero(gkeeps)):
         x = Gxs
         y = Gys
         ds = compress(gkeeps,sqrt((x-xp)**2 + 0*(y-yp)**2))
         k = nonzero(gkeeps)[argmin(ds)]
         gkeeps[k] = 0.0
         newdisp = True

      if onlyshow and key == "U" and len(nonzero(1-gkeeps)):
         x = Gxs
         y = Gys
         ds = compress(1-gkeeps,sqrt((x-xp)**2 + 0*(y-yp)**2))
         k = nonzero(1-gkeeps)[argmin(ds)]
         gkeeps[k] = 1.0
         newdisp = True

      if key == "P":
         if Templates:
            G.saveps(xdev)

      if key == "Z":
         einput = raw_input(">>> ")
         while einput:
           try:
              exec(einput)
           except:
              import traceback,sys,cgi,cStringIO
              content = sys.stdout = sys.stderr = cStringIO.StringIO()
              traceback.print_exc()
              sys.stdout = sys.__stdout__
              sys.stderr = sys.__stderr__
              result = "%s" % cgi.escape(content.getvalue())
              print result
           einput = raw_input(">>> ")
      if key == "f": newfit = True

    closedev()

class SexAndMags(Persistent):
   def __init__(self,catalogs):
#      for line in open(catalogs,"r"):
#         if line.strip() and line.strip()[0]!="#":
#            exec(line.strip())
      execfile(catalogs)
      self.readcurves()
      self.magfiles = [self.magbase.replace("%F",f) for f in self.filters]
      self.readsex()
      self.readaliases()
      self.readmags()
      self.readz()
   def readsex(self):
      print "Reading",self.catalog
      sex = readsex(self.catalog)[0]
      self.sex = OOBTree()
      for key in sex:
         self.sex[key] = sex[key]
   def readz(self):
      if hasattr(self,"specz"):
         print "Reading",self.specz
         s = readtable(self.specz,tcols=[0,3,4])
         self.specz = {}
         self.specz["id"] = s[0]
         self.specz["z"] = s[1]
         self.specz["ra"] = array([hrrad(scanSexagesimal(x)) for x in s[3]])
         self.specz["dec"] = array([degrad(scanSexagesimal(x)) for x in s[4]])
      else:
         self.specz = None
   def readaliases(self):
      if hasattr(self,"alias"):
         print "Reading",self.alias
         self.alias = readtable(self.alias,icols=[0,1])
         self.alias = take(self.alias, argsort(self.alias[1]), 1)
      else:
         self.alias = [self.sex["NUMBER"], self.sex["NUMBER"]]
   def readmags(self):
      self.mag = OOBTree()
      self.cor = OOBTree()
      if hasattr(self,"magfiles"):
         for f,m in zip(self.filters,self.magfiles):
            print "Reading",m,"for the",f,"filter"
            magf = readtable(m,header=0,icols=[0])
            self.mag[f] = OOBTree()
            for key in magf:
               self.mag[f][key] = magf[key]
      if hasattr(self,"apcor"):
         print "Reading",self.apcor,"for the aperture corrections"
         cordata = readsex(self.apcor)[0]
         if hasattr(self,"aperture"):
            naps = self.aperture
         else:
            naps = len([key for key in self.mag[self.corfilt] if "mag" in key])
#         apcor = subtract(cordata["MAG_AUTO"],self.mag[self.corfilt]["mag%d" % (naps)])
         apcor = subtract(cordata["MAG_AUTO"],cordata["MAG_APER"])
         aperr = cordata["MAGERR_AUTO"]
         if "VECTOR_ASSOC" in cordata:
            aid = "VECTOR_ASSOC"
         else:
            aid = "NUMBER"
         self.cor["id"] = cordata[aid]
         self.cor["naps"] = naps
         self.cor["apcor"] = apcor
         self.cor["aperr"] = aperr
   def readcurves(self):
      self.trans = {}
      if hasattr(self,"curves"):
         if not hasattr(self,"detector"):
           self.detector = None
         if not hasattr(self,"detectors"):
           self.detectors = len(self.curves)*[None]
         if not hasattr(self,"abs"):
           self.abs = len(self.curves) * [0]
         self.trans = {}
         for ab,filt,curve,detector in zip(self.abs,self.filters,self.curves,self.detectors):
            print "Reading %s,%s for %s (AB=%d)" % (curve,detector,filt,ab)
            if detector == "None": detector = None
            self.trans[filt] = GetFilter(curve,detector,ab=ab)
   def __getitem__(self,item):
       dict = {}
       if item not in self.alias[1]: return dict
       if type(self.alias[1]) == ListType:
          n = self.alias[0][self.alias[1].index(item)]
       elif type(self.alias[1]) == ArrayType:
          n = self.alias[0][searchsorted(self.alias[1],item)]
       n0 = n-1
       dict = {"n":n,"n0":n0,"mag":{},"sex":{},"defs":{}}
       for f,c,d,a in zip(self.filters,self.curves,self.detectors,self.abs):
          dict["defs"][f] = (c,d,a)
          dict["defs"][c] = f
       for f in self.mag:
           dict["mag"][f] = {}
           for m in self.mag[f]: dict["mag"][f][m] = self.mag[f][m][n0]
       for f in self.sex:
           dict["sex"][f] = self.sex[f][n0]
       return dict


