
import math
import time
from BC03 import *
from PhotoZ import *
from basics import *
from FITS import *
from pgplot import *
from VTKHelperFunctions import *
from Multipack import *
from sets import Set
from NumRec import *
from LDP import *
#from LDPFitting import *
from scipy.special import *
from Mestimators import *
from nnls import *
import cPickle
from TwistedHub import *

gX, dummy, gW= gauher(180,eps=3e-20)

class Target:
  def __init__(self,z,lam,flam,rescale=1.0,qe=None):
    self.z = z
    self.lam = lam * (1+z)
    self.flam = rescale * flam / (1+z)
    self.lam = concatenate([[self.lam[0]/2.0],self.lam,[self.lam[-1]*2.0]])
    self.flam = concatenate([[0.0],self.flam,[0.0]])
#    hw = 10500.0
#    self.flam = self.flam*less(self.lam,hw)
    self.qe = qe
    self.fqe = 1
    self.qo = 1
    if self.qe:
       self.qo = between(self.lam,self.qe.qe_wav[0],self.qe.qe_wav[-1],inc=1)
       self.fqe = self.qo*self.qe.interp(self.lam)
    self.ftck = splrep(self.lam,self.fqe*self.flam,s=0,k=3,xb=self.lam[0],xe=self.lam[-1])
  def Instrument(self,sigma,lams,dlams):
    wave = lams
    dwave = dlams
    dtck = splrep(wave,dwave,s=0,k=3)
    bin0 = wave[0]-dwave[0]/2.
    bins = [bin0]
    nbin = 0.5
    while bins[-1] < wave[-1]+nbin*dwave[-1]/2:
       bw0 = 1e38
       bw = bins[-1]
       while abs(bw-bw0)/bw > 1e-6:
          bw0 = bw
          dwdxb = splev(bw,dtck)
          bw = bins[-1]+nbin*dwdxb/2.
       bins.append( bins[-1]+nbin*dwdxb)
    wbins = array(bins)
    dbins = splev(wbins,dtck)

    S = array([splint(w-dw/2.,w+dw/2.,self.ftck)/dw for w,dw in zip(wbins,dbins)])
    S = VTKGauss(S,sigma/nbin,numret=1)
    stck = splrep(wbins,S,s=0,k=3)
    R = array([splint(w-dw/2.,w+dw/2.,stck)/dw for w,dw in zip(wave,dwave)])

#    S = array([ghquad(w,sigma*dw/nbin,self.ftck,gX,gW) for w,dw in zip(wbins,dbins)])
#    stck = splrep(wbins,S,s=0,k=3)
#    R = splev(wave,stck)

    if self.qe:
       qo = between(wave,self.qe.qe_wav[0],self.qe.qe_wav[-1],inc=1)
       fqe = qo*self.qe.interp(wave)
       R = divz(R,fqe)

    return R

def BCModelSpecAndMags(Name,MName,lams,dlams,logage1,metal1,redshifts,zstart,row0,sigma,curves,qfile,csf=True,maraston=0):
   print logage1,metal1
   if qfile: qe = QE(qfile)
   else: qe = None
   Fout = FITS(Name,readonly=0)
   MFout = FITS(MName,readonly=0)
   template = BCTemplate(None,None,maraston=maraston)
   Filters = [GetFilter(curve[0],ab=curve[1]) for curve in curves]
   for jz,z1 in enumerate(redshifts):
       zfac = (1+z1)
       if zstart > 0:
           tstart = log10(tlook(zstart)-tlook(z1)) + 9.0
           tau = 400.0
           tend = logage1
       else:
           tstart = -zstart
           tau = 400.0
           tend = 5.0
       print z1,tstart,tend
       if tstart > tend:
          if csf: template.tau(tstart,tend,tau,metal1,ninc=64,emin=tend)
          else: template.lagelz(logage1,metal1)
          target = Target(z1, template.lam, template.flam, rescale=1e11, qe=qe)
          flam = target.Instrument(sigma,lams,dlams)
          ftemplate = FauxTemplate(template.lam,template.flam*1e11)
          magsout = array([filt.integrate(ftemplate,z1,redfilt=curves[jc][2],logmult=1) for jc,filt in enumerate(Filters)])
       else:
          flam = 0.0*lams
          magsout = array([99.999 for filt in curves])
#       print "writing to",Fout.name(),os.getcwd(),row0+jz
       Fout.writerow(flam,row0+jz)
       MFout.goto(hdu+1)
       MFout.writerow(magsout,row0+jz)
#       Fout.flush()
#       MFout.flush()
   Fout.close()
   MFout.close()
#   print Fout.name(),"closed."
#   print MFout.name(),"closed."

def EmissionSpecAndMags(Name,MName,eline,lams,dlams,redshifts,sigma,curves,qfile):
   if qfile: qe = QE(qfile)
   else: qe = None
   Fout = FITS(Name,readonly=0)
   MFout = FITS(MName,readonly=0)
   Filters = [GetFilter(curve[0],ab=curve[1]) for curve in curves]
   for jz,z1 in enumerate(redshifts):
       zfac = (1+z1)
       print z1
       if 4800 < eline < 5020: dolines = [(4861,0.1),(4959,0.333),(5007,1.0)]
       else: dolines = [(eline,1.0)]
       ewh1 = between(lams,4500.0,11000.0)
       slams,sdlams = compress(ewh1,[lams,dlams],1)
       dtck  = splrep(slams,log10(sdlams),s=0,k=3)
       flam0 = 0.0*lams
       if 4500.0 < zfac*eline < 11000.0:
          eflams = 0.0*flam0
          for jline,(doline,doamp) in enumerate(dolines):
             dline = exp10(splev(zfac*doline,dtck))
             ey = lambda ls: (ls-zfac*doline)/(sigma*dline)
             eprof0 = lambda l1: (abs(ey(l1)) < 10 and exp(-0.5*ey(l1)**2)/sqrt(2*pi*(sigma*dline)**2) or 0.0)
             eprof = array([quad(eprof0,lam1-dlam1/2.,lam1+dlam1/2.)[0] for lam1,dlam1 in zip(lams,dlams)])
             eprof = less(abs(ey(lams)),8.0)*eprof
             eflams = eflams + doamp*eprof/dlams
       else:
          eflams = 0.0*flam0
       flam = eflams

       if 1 or 4500 < zfac*eline < 11000.0:
          dl = 10.0
          elams = arange(1000.0,11000.0,dl,Float64)
          eflams = 0.0*elams
          for jline,(doline,doamp) in enumerate(dolines):
             ey = (elams-doline)/dl
             eprof = exp(-0.5*power(ey,2)*less(abs(ey),10))/sqrt(2*pi)*less(abs(ey),10)
             eflams = eflams + doamp*eprof/dl
          etemplate = FauxTemplate(elams,eflams)
          emagsout = [filt.integrate(etemplate,z1,redfilt=curves[jc][2],logmult=1,straight=1) for jc,filt in enumerate(Filters)]
#          print emagsout 
       else:
          emagsout = len(Filters)*[99.999]
       magsout  = asarray( emagsout )
       mokay = between(magsout,-90,90)
       magsout = where(mokay,magsout,-99)
#       print "writing to",Fout.name(),os.getcwd(),jz
       Fout.writerow(flam,jz)
       MFout.writerow(magsout,jz)
#       Fout.flush()
#       MFout.flush()
   Fout.close()
   MFout.close()
#   print Fout.name(),"closed."
#   print MFout.name(),"closed."


def AGNModelSpecAndMags(Name,MName,lams,dlams,redshift,row,sigma,curves,templates=None):
   Fout = FITS(Name,readonly=0)
   MFout = FITS(MName,readonly=0)
   if not templates:
      templates = [SDSSAGNTemplate(default=True,no3727=True)]
   Filters = [GetFilter(curve[0],ab=curve[1]) for curve in curves]
   z1 = redshift
   jz = row
   print z1
   for jt,template in enumerate(templates):
     target = Target(z1, template.lam, template.flam)
     flam = target.Instrument(sigma,lams,dlams)
     ftemplate = FauxTemplate(lams,flam)
     magsout = array([filt.integrate(template,z1,redfilt=curves[jc][2],logmult=0) for jc,filt in enumerate(Filters)])
#     fmagsout = array([filt.integrate(ftemplate,z1,redfilt=curves[jc][2],logmult=0) for jc,filt in enumerate(Filters)])
#     dmags = (fmagsout-magsout)
#     dmags = compress(between(fmagsout,-90,90),dmags)
#     dmags = median(dmags)
#     magsout = magsout + dmags
     Fout.writerow(flam,len(templates)*jz+jt)
     MFout.writerow(magsout,len(templates)*jz+jt)
   Fout.close()
   MFout.close()

def ldp_import(name,comps=None):
   print "importing",name
   mod = __import__(name)
   if comps == "*":
     for k in dir(mod):
       globals()[k] = getattr(mod,k)
   elif comps:
     for k in comps.split(","):
       globals()[k] = getattr(mod,k)
   else:
     globals()[name] = mod

def NonNegFit(p,j,lbases,ly,lwt,elines=0,phot=0,verb=0):
    ep = exp10(p)
    if elines:
        r = (ly[0]-dot(ep[:1+len(elines)],lbases[0][::,j]))*lwt[0]
    else:
        r = (ly[0]-ep[0]*lbases[0][j])*lwt[0]
    if phot:
       if elines:
          pp = exp10(p[1+len(elines)] + (p[:1+len(elines)] - p[0]))
          r2 = (ly[1]-dot(pp,lbases[1][::,j]))*lwt[1]
       else:
          r2 = (ly[1]-ep[1]*lbases[1][j])*lwt[1]
       r = concatenate([r,r2])
    if verb: print exp10(p)
    return r

def GetScales(b,y):
     scales = divz(add.reduce(b*y,0),add.reduce(b*b,0))
     return scales

def SlitLoss(spar,w):
    k = exp10(spar)
    s = 1 - k[0]*(2.2e4/w) - k[1]*(2.2e4/w)**2
    return s

def Gspar(lspar,w,matchsed,gs,ly,le,lwt,goods,fitting=1,verb=0):
   spar = lspar
   wl  = array([g.wleff for g in gs])
   wb  = array([g.bandpass1 for g in gs])
   ab  = array([g.conv_AB for g in gs])
   fconv = (CLIGHT*1e8)/wl**2
   sloss = SlitLoss(spar,w)
   qsed = divz(matchsed,sloss)
   smags = asarray([f.integrate(FauxTemplate(w,qsed),0) for f in gs])
   sflux = goods * fconv * exp10(-0.4*(smags+ab+48.60))
   ssc = divz(sum(goods*lwt*ly*sflux*le),sum(goods*lwt*sflux*sflux*le))
   sflux = ssc*sflux
   x = compress(goods*lwt,(sflux-ly)*le)
   if verb: print lspar,ssc,sum(goods*lwt),sum(x**2),sum(x**2)/(len(x)-1)
   if fitting: return x
   else: return smags,ssc,sloss

def unnls(lwav,ldwav,lbases,ly,lerr,lwt,full_output=0,verb=0,elines=0,phot=0,lsols=None,it=0):
    if elines:
       nel = len([l for l in elines if type(l) != StringType])
       nsp = len(elines)+1-nel
       if "AGN" in elines: nagn = elines.index("AGN")
       elif "PL" in elines: nagn = elines.index("PL")
       else: nagn = nsp
       m = lbases[0].shape[1]
       k = lbases[0].shape[0]
       gsols = []
       y = concatenate(ly,0)
       e = concatenate(lerr,0)
       w = concatenate(lwt,0)
       #w = concatenate([lwt[0],lwt[1]*(1-phot["minc"])],0)
       for j in range(m):
           asols = []
           if j and not (j % (m/80)): sys.stdout.write("."); sys.stdout.flush()
#           bases = concatenate([lbases[ns][::,j] for ns in range(len(lwt))],1)
           for ns in [0,1]:
               bases = lbases[ns][::,j]
               by = ly[ns]
               be = lerr[ns]
               bw = lwt[ns]
               if ns:
                  lfit = dot(sols,lbases[ns][::,j])
                  bases = [lfit]
#                  bases = concatenate([bases,lbases[ns][:1,j]])
#                  if nel: bases = concatenate([bases,lbases[ns][-nel:,j]])
#                  by = by - lfit
               else:
                  ge = bases[-nel:]
                  ge2 = add.reduce(ldwav[0]*ge,1)
                  ge = divz(ge,ge2[::,NewAxis])
                  ge1 = add.reduce(ldwav[0]*ge*divz(1,lerr[0][NewAxis,::])**2,1)
                  ge3 = add.reduce(ldwav[0]*ge*ge,1)
                  ge = divz(ge1,ge3)
                  ge = getsqrt(ge)
               jsol = arange(len(bases))
               fff = 1
               fp = 0
               while fff:
                  bbases = take(bases,jsol,0)
                  sols2 = trysb(by,bbases,bw,be,lsols,f=0,nit=[3,1][ns])
                  sols = zeros((len(bases),),Float64)
                  put(sols,jsol,sols2)
                  fff = 0
                  fp += 1
#                  print j,fp,jsol
                  if 0:
                     jsol0 = nonzero(sols)
                     sols0 = 1.*sols
                     sols[-nel:] = where(greater(sols[-nel:],1*ge),sols[-nel:],0.0).astype(sols.typecode())
                     if 6563.0 in elines and 5007.0 in elines:
                           j5007 = elines.index(5007.0)+1
                           j6563 = elines.index(6563.0)+1
                           if sols[j6563] and sols[j5007]==0: sols[j6563] = -1.0
                     if 3727.0 in elines and 5007.0 in elines:
                           j3727 = elines.index(3727.0)+1
                           j5007 = elines.index(5007.0)+1
                           if sols[j5007] and sols[j3727]==0: sols[j5007] = -1.0
                     if ("AGN" in elines or "PL" in elines):
                        if "AGN" in elines: jagn = elines.index("AGN")+1
                        else: jagn = elines.index("PL")+1
                        if 3727.0 in elines:
                           j3727 = elines.index(3727.0)+1
                           if sols[jagn]>0 and sols[j3727]<=0: sols[jagn] = -1.0
                        if 5007.0 in elines:
                           j5007 = elines.index(5007.0)+1
                           if sols[jagn]>0 and sols[j5007]<=0: sols[jagn] = -1.0
                        if 6563.0 in elines:
                           j6563 = elines.index(6563.0)+1
                           if sols[jagn]>0 and sols[j6563]<=0: sols[jagn] = -1.0
                        if 2800.0 in elines:
                           j2800 = elines.index(2800.0)+1
                           if sols[jagn]>0 and sols[j2800]<=0: sols[jagn] = -1.0
                           if sols[j2800]>0 and sols[jagn]<=0: sols[j2800] = -1.0
                     fff = sum(less(sols,0)) > 0
                     jsol = nonzero(greater(sols,-1))
                     jsol =  concatenate([ arange(jagn-1),compress(greater_equal(jsol,jagn),jsol)])
                     sols2 = take(sols0,jsol)
                     sols = 0.0*sols
                     put(sols,jsol,sols2)
               asols.append(sols)
#           sols = concatenate(asols)
           gsols.append( concatenate([asols[0],ge,asols[1]]))
       return asarray(gsols)
    else:
       for j in range(m):
           if j and not (j % (m/80)): sys.stdout.write("."); sys.stdout.flush()
           sols0 = nnls((lbases[0][j]*(lwt[0]*lerr[0]))[NewAxis,::],ly[0]*(lwt[0]*lerr[0]))[1]
           if len(ly) > 1:
              sols0 = concatenate([sols0,nnls((lbases[1][j]*(lwt[1]*lerr[1]))[NewAxis,::],ly[1]*(lwt[1]*lerr[1]))[1]])
           gsols.append(sols0)
       return asarray(gsols)

def trysb(y,bases,w,e,lsols,f=1,nit=1):
   for mi in range(nit):
      if mi==0:
          mw = 1
      else:
          if lsols: wsols = lsols
          else: wsols = sols
          fr = dot(wsols,bases)
          r = (y-fr)*e
          if 0 or lsols:
             rfix = compress(r,r)
             if rfix:
               rfix = bwt(rfix)[1]
               if rfix: r = r/rfix
          if 0 and mi > 1:
            mwl1 = Fair(r,w=1)
            mwl2 = L2(r,w=1)
            mw = where(r>0,mwl1,mwl2)
          else:
            mw = Lp(r,w=1)
      mw = mw*w*e
      sols = nnls(bases*mw[NewAxis,::],y*mw)[1]
   if f: return w*(dot(sols,bases)-y)*e
   else: return sols

def NonNeg(j,lbases,ly,lwt,full_output=0,verb=0,elines=0,phot=0):
    if not sum(lbases[0][0,j]):
       sols = asarray(len(lbases[0][::,j])*[0.0])
       return sols
    if elines:
       sols0 = asarray([GetScales(b*lwt[0],ly[0]*lwt[0]) for b in lbases[0][::,j]])
       sols0[1:] = sols0[1:]
       if phot:
          sols1 = asarray([GetScales(b*lwt[1],ly[1]*lwt[1]) for b in lbases[1][:1,j]])
          sols0 = concatenate([sols0,sols1])
    else:
       sols0 = asarray([GetScales(lbases[0][j]*lwt[0],ly[0]*lwt[0]),
                        GetScales(lbases[1][j]*lwt[1],ly[1]*lwt[1])])

    rsols = getlog10(sols0,rep=-999.)
    usols = 1.0*greater(rsols,-999)
    if verb: print sols0
    if sum(usols):
#       nsols = mean(compress(usols,rsols))
#       sols = where(usols,rsols,nsols)
       sols = rsols
       psols = leastsq(NonNegFit,sols,args=(j,lbases,ly,lwt,elines,phot,verb),full_output=full_output)
       if full_output: gcov = GetC(psols)
       if verb:
          if full_output: print psols
          else: print psols
       gsols = psols[0]
       if len(shape(gsols)) == 0: gsols = asarray([gsols])
#       gsols = where(not_equal(gsols,sols),gsols,-99.0)
       bad = 0
    else:
       gsols = -99.0*ones(sols0.shape,Float64)
       gcov = -99.0*ones((sols0.shape[0],sols0.shape[0]),Float64)
    if elines and phot:
       mp = gsols[1+len(elines)] + (gsols[:1+len(elines)] - gsols[0])
       gsols = concatenate([gsols[:1+len(elines)],mp])
    if full_output:
      return gsols,gcov
    else:
      return exp10(gsols)

def FitOneGalaxy(template,ffile,phot,inter=False,lam1=5000.0,lam2=9000.0,individ=0):
    start = time.time()
    etemplate = eval(template)
    print "Fitting",ffile
    F = FITS(ffile)
    objname = F["object"].strip()
    exptime = F["exptime"]
    aperture = F["aperture"]
    data = F.data()
    norms = []
    while F["norm%d" % (len(norms)+1)] != "N/A": norms.append(F["norm%d" % (len(norms)+1)][1:-1].split(","))
    F.close()
    print [os.path.basename(norm[0]) for norm in norms]
    if hasattr(etemplate,"qe"):
       specs = [norm[0].replace("od","ox") for norm in norms]
    else:
       specs = [norm[0].replace("od","oq") for norm in norms]
    Fs = [FITS(spec) for spec in specs]
    sdata = [[F.data()[::,j,::] for j in range(F["naxis2"]) if objname == F["obj%04d" % (j+1)].strip()] for F in Fs]
    stimes = [2*F["exptime"] for F in Fs]
    [F.close() for F in Fs]
    specn = [float(norm[1]) for norm in norms]

    if "obj" in objname: number = int(objname[3:])
    elif "ref" in objname: number = int(objname[3:])
    else: number = int(objname[1:])
    if inter: pgdev = "/xs"
    else:
       odir = os.path.dirname(ffile)
       ofile = os.path.basename(ffile).replace(".fits","_"+template+".ps/vcps")
       pgdev = "/".join([odir,template,ofile])
       print "Writing",pgdev
    if "Galaxy" in template:
      nx,ny = 3,3
    if template == "AGN":
      nx,ny = 3,3
    nit = 1
#    if phot: nit += 2
    ssc = 1.0
    for it in range(nit):
       seds = getattr(etemplate,"seds")
       mags = getattr(etemplate,"mags")
       curves = getattr(etemplate,"curves")
       cab = getattr(etemplate,"conv_AB")
       cwl = getattr(etemplate,"wleffs")
       pshape = getattr(etemplate,"pshape")
       params = getattr(etemplate,"params")
       redshifts = getattr(etemplate,"redshifts")
       allz = getattr(etemplate,"allz")
       if hasattr(etemplate,"Avs"): Avs = getattr(etemplate,"Avs")
       else: Avs = []
       if hasattr(etemplate,"elines"): elines = getattr(etemplate,"elines")
       else: elines = []

       w,dw,s,ds,bg = data
       ds = ds/2.

#       ds = sqrt(ds**2 + (s/sqrt(dw))**2)*greater(ds,0)

       if not it:
          sfix = ones(w.shape,Float32)

       if hasattr(etemplate,"qe"):
          qe = getattr(etemplate,"qe")
          qw = getattr(etemplate,"qw")
       else:
          qe = ones(w.shape,Float)
          qw = w

       swt = 1.0*greater(w,lam1)*less(w,lam2)*greater(ds,0)
       atmos = getattr(etemplate,"atmos")
       swt = swt * atmos
       swt = swt * greater(s,-3*ds)

       if hasattr(etemplate,"qewt"):
          print "Using QE weighting"
          qewt = etemplate.qewt
          qewt = divz(qewt,mean(compress(swt,qewt)))
       else:
          qewt = 1
       swt = swt * qewt

       if len(nonzero(swt)) < 30:
          print len(nonzero(swt))
          return False

       sq = s
       dsq = ds
       bgq = bg

       if not it: opendev(pgdev,nx=nx,ny=ny,size=nx*3.5,land=0,asp=1.0*ny/nx)
       qtck = splrep(qw,qe,s=0,k=3)

       if 1:
          ydat = [sq*1e20]
          yerr = [divz(1,dsq*1e20)]
          ywt = [swt]
          ymod = [seds]
          ywav = [w]
          ydwav = [dw]
          sphot = None

       if phot:
          fs = phot["mag"].keys()
          cs = [phot["defs"][f] for f in fs]
          ci = [curves.index(c[0]) for c in cs]
          gs = [GetFilter(c[0],c[1],ab=c[2]) for jc,c in enumerate(cs)]
          wl  = array([g.wleff for g in gs])
          wb  = array([g.bandpass1 for g in gs])
          ab  = array([g.conv_AB for g in gs])
          wo = argsort(wl)
          cs = [cs[jw] for jw in wo]
          ci = [ci[jw] for jw in wo]
          gs = [gs[jw] for jw in wo]
          fs = [fs[jw] for jw in wo]
          wl,wb,ab = take( [wl,wb,ab], wo, 1)

          fconv = (CLIGHT*1e8)/wl**2
          print fs
          print ab
          print wl
          print wb
          print ci
          uap = getattr(etemplate,"uap")
          if hasattr(etemplate,"uap2"): uap2 = getattr(etemplate,"uap2")
          else: uap2 = None
          magt = array([phot["mag"][f]["mag%d" % (4)] for f in fs])
          mago = array([phot["mag"][f]["mag%d" % (uap)] for f in fs])
          erro = array([phot["mag"][f]["err%d" % (uap)] for f in fs])
          if uap2:
             mago2 = array([phot["mag"][f]["mag%d" % (uap2)] for f in fs])
             erro2 = array([phot["mag"][f]["err%d" % (uap2)] for f in fs])
#             fixap = array([(f in uap2[1:-1]) for f in fs])
#             fixmag = phot["mag"][uap2[0]]["mag%d" % (uap)] - phot["mag"][uap2[0]]["mag%d" % (uap2[-1])]
#             mago = where(fixap,mago2+fixmag,mago)
#             erro = where(fixap,erro2,erro)
             erro2 = sqrt(erro2**2 + 0.030**2)
             umag2 = between(mago2,0,29) * less(erro2,9)
             fluxo2 = fconv * exp10(-0.4*(mago2+ab+48.60))
             sigo2 = (erro2/1.0857)*fluxo2
             fluxo2 = where(between(mago2,-90,90),fluxo2,0.0)
             sigo2 = where(between(mago2,-90,90),sigo2,0.0)

          erro = sqrt(erro**2 + 0.030**2)
          umag = between(mago,0,29) * less(erro,9)
          print magt
          print umag
          print mago
          print erro
          fluxo = fconv * exp10(-0.4*(mago+ab+48.60))
          sigo = (erro/1.0857)*fluxo
          fluxo = where(between(mago,-90,90),fluxo,0.0)
          sigo = where(between(mago,-90,90),sigo,0.0)
          mwt = umag
          print fluxo
          print sigo

          if elines:
             print "Fit magnitudes including these extra lines",elines
             print mags.shape
#             magm = asarray([mags[::,::,ci1] for ci1 in ci])
             magm = take(mags,ci,2)
#             magm = asarray([take(mags1,ci,1) for mags1 in mags])
          else:
             magm = take(mags,ci,1)
          goodm = between(magm,-90,90)
          cfconv = (CLIGHT*1e8)/take(cwl,ci)**2
          cabu = take(cab,ci)
          print [curves[ci1] for ci1 in ci]
          print ci
          print magm.shape,cfconv.shape,cabu.shape, goodm.shape
          fluxes = goodm * exp10(-0.4*(magm+cabu+48.60)) * cfconv
#          print fluxes.shape
#          exit()

          goods = greater(wl-wb/2.,w[0])*less(wl+wb/2.,w[-1])
#          goods = greater(wl,lam1)*less(wl,lam2)

          minc = greater(wl-wb/2.,lam1)*less(wl+wb/2.,lam2)
          print mwt
          print goods

          sphot = {"goods":goods,"gs":gs,"minc":minc}
          ydat.append(fluxo*1e20)
          yerr.append(divz(1,sigo*1e20))
          ywt.append(mwt)
          ywav.append(wl)
          ydwav.append(wb)
          ymod.append(fluxes)

          lu = zeros(ywav[0].shape,Float64)
          [add(lu,less(abs(ywav[0]-wl1),wb1/2.),lu) for wl1,wb1 in zip(wl,wb)]
          lu = lu * swt
          lu = lu*between(ywav[0],lam1,lam2,inc=1)
          lu = lu*greater(ydwav[0],0)
          qmods = zeros(ywav[0].shape,Float64)
          qmode = zeros(ywav[0].shape,Float64)
          qmodn = zeros(ywav[0].shape,Float64)
          for wl1,wb1,wf1,we1,ww1,wg1 in zip(wl,wb,ydat[1],yerr[1],mwt,goods):
             if wg1:
                print wl1,wb1,wf1
                fwt = less(abs(ywav[0]-wl1),wb1/2.)
                fwt = fwt * between(ywav[0],lam1,lam2,inc=1)
                add(qmodn,fwt,qmodn)
                add(qmods,fwt*wf1,qmods)
                add(qmode,fwt*(we1*ww1),qmode)
          qmods = divz(qmods,qmodn)
          qmode = divz(qmode,qmodn)
          qmode = divz(1,sqrt(divz(1,qmode)**2 + divz(1,yerr[0])**2))
          ffix = getattr(etemplate,"ffix")
          fform = getattr(etemplate,"fform")
          if fform == "w": qb = polynom((ywav[0]/1e4),ffix)
          elif fform == "dw": qb = polynom(ydwav[0]/200,ffix)
          elif fform == "iw": qb = polynom(divz(1e4,ywav[0])-1,ffix)
          ulu = lu * greater(ydat[0],-3*divz(1,yerr[0]))
          for uk in range(3):
             qf = svdfit(transpose(qb*(ulu*qmode*qmods)[NewAxis,::]),(ulu*qmode)*ydat[0])
             print qf
             sqf = qf
#             qf = array([1.0])
             iqfix = dot(qf,qb)
             rq = iqfix*qmods-ydat[0]
             rq = rq*yerr[0]
             sq = bwt(compress(swt*ulu*qmode*qmods,rq))[1]
             rq = divz(rq,sq)
             ulu = greater(lu,0)*Lp(rq,w=1)
             ulu = ulu * greater(ydat[0],-3*divz(1,yerr[0]))
          qfix = divz(1,iqfix)
          ydat[0] = ydat[0] * qfix
          yerr[0] = yerr[0] * iqfix

          if uap2:
             ydat[1] = fluxo2*1e20 - ydat[1]
             yerr[1] = divz(1,sigo2*1e20)
             print ydat[1]

#          ywt[-1] = where(greater(wl+wb/2.,lam1)*less(wl-wb/2.,lam2),ywt[-1],ywt[-1])

#       exit()
       gdict = {"Lp":None,"L2":None,"Tukey":None,"Fair":None,"L1":None}
       gdict2 = {"Lp":None,"L2":None,"Tukey":None,"Fair":None,"L1":None}
       if elines:
          lsols = None
          nol = 1+len(elines)
          nel = len([l for l in elines if type(l) != StringType])
          for fiter in range(1):
              print "Fit LDP spectrum including these extra lines",elines
              gsolse = unnls(ywav,ydwav,ymod,ydat,yerr,ywt,elines=elines,phot=sphot,verb=0,lsols=lsols,it=it)
              gsols = gsolse[::,:nol]
              gsole = gsolse[::,nol:nol+nel]
#              gsols[::,nol-nel:nol] = gsols[::,nol-nel:nol]*0
              nspec = len(ywav)
              if phot: nspec = nspec-1
              for ns in range(nspec):
                 g = asarray([dot(gsols[j,:nol],ymod[ns][:nol,j,::]) for j in range(ymod[ns].shape[1])])
                 ge = asarray([dot(greater(gsols[j,nol-nel:nol],0),ymod[ns][nol-nel:nol,j,::]) for j in range(ymod[ns].shape[1])])
#                 g = add(gc,ge)
                 subtract(g,ydat[ns][NewAxis,::],g)
                 multiply(g,yerr[ns][NewAxis,::],g)
                 nbad = greater(g,0)
                 umwt = ywt[ns][NewAxis,::]
                 umwt = umwt * where(less_equal(ge,0.01/sqrt(4*pi)),1.0,0.25)
#                 umwt = umwt * where(ge,1.0,1.0)
                 multiply(g,umwt,g)
                 for M in gdict:
                    gdict[M] = add.reduce(eval(M)(g),1)
                 x2 = gdict["L2"]
                 ndeg = len(nonzero((ywt[ns]*yerr[ns]))) - len(pshape) - 1 - len(elines)
                 extras = gsols.shape[1]-ns+nol
              if phot:
                 gsolm = gsolse[::,nol+nel:]
                 fluxfix = qf
                 extras = fluxfix.shape[0]
                 gc = asarray([dot(gsols[j,:nol],ymod[nspec][::,j,::]) for j in range(ymod[nspec].shape[1])])
                 gp = asarray([gsolm[j,0]*gc[j]+dot(gsolm[j,1:],ymod[nspec][:len(gsolm[j,::])-1,j,::]) for j in range(ymod[nspec].shape[1])])
                 g = add(gp,fluxo)
                 subtract(g,ydat[nspec][NewAxis,::],g)
                 multiply(g,yerr[nspec][NewAxis,::],g)
#                 umwt = (ywt[nspec]*logical_not(goods))
                 umwt = ywt[nspec][NewAxis,::]
#                 umwt = umwt * where(ge,1.0,1.0)
                 multiply(g,umwt,g)
                 for M in gdict2:
                    gdict2[M] = add.reduce(eval(M)(g),1)
                 x2 = x2 + gdict2["L2"]
                 ndeg = ndeg + len(nonzero(ywt[nspec])) - extras
              mx2 = argmin(x2)
              lsols = gsols[mx2,:nol]
       else:
          gsols = asarray([unnls(allz[j],j,ywav,ydwav,ymod,ydat,yerr,ywt,elines=0,phot=phot,verb=0) for j in range(ymod[0].shape[0])])
          nol = 1
          g = gsols[::,0][::,NewAxis]*ymod[0]
          subtract(g,ydat[0][NewAxis,::],g)
          multiply(g,yerr[0][NewAxis,::],g)
          g = Lp(g)
          multiply(g,ywt[0][NewAxis,::],g)
          sx2 = add.reduce(g,1)
          x2 = sx2
          ndeg = len(nonzero((ywt[0]*yerr[0]))) - len(pshape) - 1
          nspec = len(ywav)
          if phot: nspec = nspec-1
          if phot:
             g = gsols[::,1][::,NewAxis]*ymod[1]
             subtract(g,ydat[1][NewAxis,::],g)
             multiply(g,yerr[1][NewAxis,::],g)
             g = Lp(g)
             multiply(g,ywt[1][NewAxis,::],g)
             sx2 = add.reduce(g,1)
             x2 = x2 + sx2
             ndeg = ndeg + len(nonzero((ywt[1]*yerr[1]))) - 1
#       x2 = where((allz>0.880)*(allz<0.910),x2,1e9)
#       x2 = where((allz>=1.00)*(allz<1.2),x2,1e9)
       x2 = 2*x2
       x2 = reshape(x2,pshape)
       mfacs = ones(pshape,Float32)
       if hasattr(etemplate,"bad"):
          bad = reshape(getattr(etemplate,"bad"),pshape)
          x2 = x2 + 1e24*bad
       minm = argmin(x2.flat)
       minz = allz[minm]
       minchi2 = x2.flat[minm]
       dx2 = x2 - minchi2
       p = exp(-0.5*dx2)
       z0 = getattr(etemplate,"z0")
       z1 = getattr(etemplate,"z1")
       dz = getattr(etemplate,"dz")
       if "Galaxy" in template:

          k20 = array([ 0.18009532,-3.19084644, 0.61086963, 0.54542968, 0.31197063,])
          k20p = k20[0]*cosh((allz-k20[1])/k20[2])*exp(-0.5*(allz-k20[3])**2/k20[4]**2)
          k20p = reshape(k20p,pshape)

          logage0 = getattr(etemplate,"logage0")
          logage1 = getattr(etemplate,"logage1")
          dlogage = getattr(etemplate,"dlogage")
          allt = reshape(getattr(etemplate,"allt"),pshape)
          metal0 = getattr(etemplate,"metal0")
          metal1 = getattr(etemplate,"metal1")
          dmetal = getattr(etemplate,"dmetal")
          allm = reshape(getattr(etemplate,"allm"),pshape)
          print allt.shape
#          tp = exp10(allt-9.0)
#          tp = tp * k20p
#          p = p * tp
#          tz = between(reshape(allz,pshape),0.81,0.83)
#          p = p * tz
#          mM = cosmomod(redshifts)
#          if phot:
#             tm = reshape(add.reduce(gsols[::,:6],1),p.shape)*exp10(0.4*mM)*1e11
#             tm = tm * exp10(0.4*median(magt-mago))
#             p = p*between(tm,1e8,5e12)
          zchi2 = minimum.reduce(minimum.reduce(x2,1),0)
          zprob = add.reduce(add.reduce(p,1),0)*dlogage*dmetal
          ztprob = add.reduce(p,0)*dmetal
          zmprob = add.reduce(p,1)*dlogage
          tmprob = add.reduce(p,2)*dz
          ztprob = divz(ztprob,sum(sum(ztprob*dz)*dlogage))*dz*dlogage
          zmprob = divz(zmprob,sum(sum(zmprob*dz)*dmetal))*dz*dmetal
          tmprob = divz(tmprob,sum(sum(tmprob*dmetal)*dlogage))*dmetal*dlogage
          zprob = divz(zprob,sum(zprob*dz))*dz
          lzprob = getlog10(zprob)
          tprob = add.reduce(add.reduce(p,2),0)*dz*dmetal
          mprob = add.reduce(add.reduce(p,2),1)*dz*dlogage
       if template == "AGN":
          zprob = add.reduce(p,1)
          zchi2 = minimum.reduce(x2,1)
          zprob = divz(zprob,sum(zprob*dz))*dz
          lzprob = getlog10(zprob)

       ldil = 4*pi*(CMPMPC*mod2mpc(cosmomod(redshifts)))**2

       maxp = argmax(p.flat)

       maxz = allz[maxp]
       print "z=",minz,maxz
       cz = add.accumulate(zprob)
       ic = clip(searchsorted(cz,[0.025,0.975]),0,len(redshifts)-1)
       print take(redshifts,ic)
       print take(cz,ic)
       print ConfidenceLimits(redshifts,zprob)
       print 
       print "Min chi^2=",minchi2,x2.flat[maxp]
       print "N degrees=",ndeg
       print "Min reduced chi^2=",divz(minchi2,ndeg),divz(x2.flat[maxp],ndeg)
       print gsols[maxp]
       print gsole[maxp]
       if phot:
         print gsolm[maxp]
       gsols = gsols/1e20
       gsole = gsole/1e20
       print gsols[maxp]
       print gsole[maxp]
       if phot:
         gsolm[::,1:] = gsolm[::,1:]/1e20
         print gsolm[maxp]

       xm,ym = [redshifts[0],redshifts[-1]],[max([0,min(zchi2)-ndeg]),median(compress(less(zchi2,1e24),zchi2))]
       newbox(xm,ym,ch=1.8)
       xlabel("z",ch=1.8)
       ylabel("\gx\u2\d",ch=1.8)
       drawln(redshifts,zchi2)
       drawln([minz,minz],ym,ls=2)

       xm,ym = [redshifts[0],redshifts[-1]],[max(lzprob)-5,max(lzprob)+0.3]
       newbox(xm,ym,ch=1.8)
       xlabel("z",ch=1.8)
       ylabel("P(z)dz",ch=1.8)
       drawln(redshifts,lzprob)
       drawln([minz,minz],ym,ls=2)

       cpgpage()

       plam1, plam2 = lam1 - 500, lam2 + 500
       mfac = 1
       mfacb = 1.0
       if phot:
#          mfac = divz(1,gf[maxp])
          mfac = qfix
          mfacb = median(mfac)

       s = mfac*s
       ds = mfac*ds
       bg = mfac*bg

       us,uds = compress(swt,[s,ds],1)
       xm,ym = [plam1,plam2],[-0.3*getpct(us+2*uds,95),1.5*getpct(us+2*uds,95)]
       newbox(xm,ym,ch=1.8)
       xlabel("\gl (\A)",ch=1.8)
       ylabel("DN/s/\A",ch=1.8)
       drawpt(w,s,ps=17,ci=14,ch=1)
       drawpterr(w,dw/2,s,ds,ci=14,ps=17,ch=1.8)
       wp,dwp,sp,dsp = compress(swt,[w,dw,s,ds],1)
       drawpt(wp,sp,ps=17,ch=1)
       drawpterr(wp,dwp/2,sp,dsp,ps=17,ch=1.8)
       if elines:
          ugsols = gsols[maxp,:nol]
          g = dot(ugsols,seds[::,maxp,::])
          drawln(w,g,ci=4)
          ugsols = gsols[maxp,:nol-nel]
          g = dot(ugsols,seds[:-nel,maxp,::])
          drawln(w,g,ci=2)
       else:
          g = seds[maxp]
          drawln(w,g,ci=2)

       sq = s
       dsq = ds
       bgq = bg
       uw,usq,udsq,ubgq = compress(swt,[w,sq,dsq,bgq],1)

       xm,ym = [plam1,plam2],[-0.3*getpct(usq+2*udsq,95),1.5*getpct(usq+2*udsq,95)]
       newbox(xm,ym,ch=1.8)
       xlabel("\gl (\A)",ch=1.8)
       ylabel("F\d\gl\u (ergs s\u-1\d cm\u-2\d \A\u-1\d)",ch=1.8)
       for ns,sdat,snorm,stime in zip(range(len(stimes)),sdata,specn,stimes):
          if 1:
             ss1, se1, sb1, sw1, sd1, sb1, sqe1 = sdat[0]
             sw1,sd1,ss1,se1,sb1,sqe1 = compress(sw1*sqe1,[sw1,sd1,ss1,se1,sb1,sqe1],1)
             if len(sw1):
                ss1 = where(between(ss1,1e-24,1e24),ss1,0.0)
                se1 = where(between(se1,1e-24,1e24),se1,0.0)
                if snorm:
                   if phot:
                      if fform == "w": mfacb = divz(1,dot(qf,polynom((sw1/1e4),len(qf)-1)))
                      elif fform == "dw": mfacb = divz(1,dot(qf,polynom(sd1/200,len(qf)-1)))
                      elif fform == "iw": mfacb = divz(1,dot(qf,polynom(divz(1e4,sw1)-1,len(qf)-1)))
                      mfacb = mfacb*greater(sd1,0)
                   else:
                      mfacb = 1.
                   sq1 = mfacb*snorm*divz(ss1,sd1*sqe1*sb1)/stime
                   dsq1 = mfacb*snorm*divz(se1,sd1*sqe1*sb1)/stime
                   drawpt(sw1,sq1,ps=1,ch=1,ci=1)
                   drawpterr(sw1,sd1/2,sq1,dsq1,ps=17,ch=1.8,ci=1)
       if elines:
          ugsols = gsols[maxp,:nol]
          g = dot(ugsols,seds[::,maxp,::])
          drawln(w,g,ci=4)
          ugsols = gsols[maxp,:nol-nel]
          g = dot(ugsols,seds[:-nel,maxp,::])
          drawln(w,g,ci=2)
       else:
          g = seds[maxp]
          drawln(w,g,ci=2)

       uw,usq,udsq,ubgq = compress(swt*greater(w,5000),[w,sq,dsq,bgq],1)

       xm,ym = [plam1,plam2],[-0.3*getpct(ubgq+2*udsq,95),1.5*getpct(ubgq+2*udsq,95)]
       newbox(xm,ym,ch=1.8)
       xlabel("\gl (\A)",ch=1.8)
       ylabel("F\d\gl\u (ergs s\u-1\d cm\u-2\d \A\u-1\d)",ch=1.8)
       drawln(w,bgq,ci=8)
       drawpt(w,sq,ci=14,ps=17,ch=1)
       drawpterr(w,dw/2,sq,dsq,ci=14,ps=17,ch=1.8)
       wp,dwp,sqp,dsqp,bgqp = compress(swt,[w,dw,sq,dsq,bgq],1)
       drawpt(wp,sqp,ps=17,ch=1)
       drawpterr(wp,dwp/2,sqp,dsqp,ps=17,ch=1.8)
       if phot:
          drawpt(wl,fluxo,ps=17,ci=[3,3][inter])
          drawpterr(wl,wb/2.,fluxo,sigo,ps=17,ci=[3,3][inter])
       if elines:
          ugsols = gsols[maxp,:nol]
          g = dot(ugsols,seds[::,maxp,::])
          drawln(w,g,ci=4)
          ugsols = gsols[maxp,:nol-nel]
          g = dot(ugsols,seds[:-nel,maxp,::])
          drawln(w,g,ci=2)
       else:
          g = seds[maxp]
          drawln(w,g,ci=2)

       if phot:
          plam1 = min([plam1,min(wl-wb)])
          plam2 = max([plam2,max(wl+wb)])

       xm = [plam1,plam2]
       if phot:
          if uap2:
             ym = [-0.3*getpct(usq+1*udsq,85),1.2*max(fluxo2+1*sigo2)]
          else:
             ym = [-0.3*getpct(usq+1*udsq,85),1.2*max(fluxo+1*sigo)]
       else:
          ym = [-0.3*getpct(usq+1*udsq,85),2.0*getpct(usq+1*udsq,85)]
       newbox(getlog10(xm),ym,ch=1.8,xlog=1)
       xlabel("\gl (\A)",ch=1.8)
       ylabel("F\d\gl\u (ergs s\u-1\d cm\u-2\d \A\u-1\d)",ch=1.8)
       drawpt(getlog10(w),sq,ci=14,ps=17,ch=1)
       drawpterr(getlog10(w),dw/2/w/2.303,sq,dsq,ci=14,ps=17,ch=1.8)
       wp,dwp,sqp,dsqp,bgqp = compress(swt,[w,dw,sq,dsq,bgq],1)
       drawln(getlog10(wp),bgqp,ci=8)
       drawpt(getlog10(wp),sqp,ps=17,ch=1)
       drawpterr(getlog10(wp),dwp/2/wp/2.303,sqp,dsqp,ps=17,ch=1.8)
       drawln(getlog10(wp),dsqp,ls=1,ci=5,lw=1)
       if phot:
          if uap2:
             drawpt(getlog10(wl),fluxo2,ps=17,ci=[3,7][inter],ch=2.2)
             drawpterr(getlog10(wl),wb/2./wl/2.303,fluxo2,sigo2,ps=17,ci=[3,7][inter],ch=2.2)
             drawpt(getlog10(wl),fluxo,ps=5,ci=8,ch=2.2)
             drawpterr(getlog10(wl),wb/2./wl/2.303,fluxo,sigo,ps=5,ci=8,ch=2.2)
          else:
             drawpt(getlog10(wl),fluxo,ps=5,ci=[3,7][inter],ch=2.2)
             drawpterr(getlog10(wl),wb/2./wl/2.303,fluxo,sigo,ps=5,ci=[3,7][inter],ch=2.2)
          ugsols = gsols[maxp,:nol]
          ugsolm = gsolm[maxp,:nol]
          g = dot(ugsols,fluxes[::,maxp,::])
          g = fluxo + ugsolm[0]*g + dot(ugsolm[1:],fluxes[:len(ugsolm)-1,maxp,::])
          drawpt(getlog10(wl),g,ps=17,ci=4,ch=2.5)
       if elines:
          ugsols = gsols[maxp,:nol]
          g = dot(ugsols,seds[::,maxp,::])
          drawln(getlog10(w),g,ci=4)
          ugsols = gsols[maxp,:nol-nel]
          g = dot(ugsols,seds[:-nel,maxp,::])
          drawln(getlog10(w),g,ci=2)
       else:
          g = seds[maxp]
          drawln(getlog10(w),g,ci=2)


       if "Galaxy" in template:

          xm,ym = [z0,z1+dz],[logage0,logage1+dlogage]
          newbox(xm,ym,ch=1.8)
          xlabel("z",ch=1.8)
          ylabel("log t\dend\u",ch=1.8)
          xfm = array([z0-dz/2.,dz,0.0,logage0-dlogage/2.,0.0,dlogage])
          am = max(ztprob.flat)
          if inter: a1,a2 = -0.5*am,0.5*am
          else: a1,a2 = 0.5*am,-0.5*am
          cpggray(ztprob,ztprob.shape[1],ztprob.shape[0],1,ztprob.shape[1],1,ztprob.shape[0],a1,a2,xfm)
          newbox(xm,ym,ch=1.8,era=0)

          xm,ym = [logage0,logage1+dlogage],[metal0,metal1+dmetal]
          newbox(xm,ym,ch=1.8)
          xlabel("log t\dend\u",ch=1.8)
          ylabel("[Fe/H]",ch=1.8)
          xfm = array([logage0-dlogage/2.,dlogage,0.0,metal0-dmetal/2.,0.0,dmetal])
          am = max(tmprob.flat)
          if inter: a1,a2 = -0.5*am,0.5*am
          else: a1,a2 = 0.5*am,-0.5*am
          cpggray(tmprob,tmprob.shape[1],tmprob.shape[0],1,tmprob.shape[1],1,tmprob.shape[0],a1,a2,xfm)
          newbox(xm,ym,ch=1.8,era=0)

    closedev()

    odir = os.path.dirname(ffile)
    ofile = os.path.basename(ffile).replace(".fits","_"+template+".fits")
    ofile = "/".join([odir,template,ofile])
    print "Writing",ofile

    F = FITS(ofile,create=1,template=ffile)

    gsolsb = asarray([reshape(gs,pshape) for gs in transpose(gsols)])
    gsolsc = asarray([reshape(gs,pshape) for gs in transpose(gsole)])
    gnaxes = asarray(gsolsb.shape)[::-1].tolist()
    gnaxes2 = asarray(gsolsc.shape)[::-1].tolist()
    F.resize(gnaxes)
    F.modkey("object","gsols")
    if phot:
       F.modkey("phot",1)
       F.modkey("uap",uap)
       if uap2:
          F.modkey("uap2",uap2)
#         F.modkey("uap2",str(uap2).replace(" ",""))
    else:
       F.modkey("phot",0)
    if hasattr(etemplate,"elines"):
       elines = getattr(etemplate,"elines")
       for jline,eline in enumerate(elines):
          if eline: F.modkey("eline%d" % (jline),eline)

    F.modkey("minchi2",minchi2)
    F.modkey("ndeg",ndeg)
    F.modkey("lam1",lam1)
    F.modkey("lam2",lam2)
    if phot:
       [F.modkey("qf%d" % (j),qf[j]) for j in range(len(qf))]
    [F.modkey(par,params[par]) for par in params]
    F.putdata(gsolsb)

    F.addhdu()
    F.newhead("T",-32,len(gnaxes2),gnaxes2)
    F.putdata(gsolsc)

    F.addhdu()
    naxes = list(asarray(pshape)[::-1])+[5,]
    F.newhead("T",-32,len(naxes),naxes)
    Ms = ["Lp","L2","Tukey","Fair","L1"]
    [F.modkey("Mest%d" % (j),Ms[j]) for j in range(len(Ms))]
    garray = asarray([gdict[M] for M in Ms])
    F.putdata(garray)

    if phot:
       gsolsd = asarray([reshape(gs,pshape) for gs in transpose(gsolm)])
       gnaxes3 = asarray(gsolsd.shape)[::-1].tolist()

       F.addhdu()
       F.newhead("T",-32,len(gnaxes3),gnaxes3)
       F.putdata(gsolsd)

       F.addhdu()
       F.newhead("T",-32,len(naxes),naxes)
       [F.modkey("Mest%d" % (j),Ms[j]) for j in range(len(Ms))]
       garray2 = asarray([gdict2[M] for M in Ms])
       F.putdata(garray2)

    F.close()

    finish = time.time()
    print "Time = %.3fs" % (finish-start)


def ConfidenceLimits(x,y,k=3):
    x,y = take([x,y],argsort(x),1)
    tck = splrep(x,y,s=0,k=3)
    g = lambda x1: splint(0,x1,tck)
    iy = array([g(x1) for x1 in x])
    if iy[-1]: iy = iy / iy[-1]
#    tck = splrep(x,iy,s=0,k=k)
    ps = [0.025,0.05,0.16,0.5,0.84,0.95,0.975]
    tx = concatenate([k*[x[0]],x,k*[x[-1]]])
    g = lambda r,p: splev(r,[tx,iy,k])-p
    cs = [fsolve(g,[x[clip(searchsorted(iy,p),0,len(x)-1)]],args=(p,)) for p in ps]
    m = argmax(y)
    p = x[m]
    return concatenate([[p],cs])

def PConfidenceLimits(x,y,k=3):
    x,y = compress(y,[x,y],1)
    x,y = take([x,y],argsort(x),1)
    iy = add.accumulate(y)
    if iy[-1]: iy = iy / iy[-1]
    sx,sy = compress(x,[x,y],1)
    if sx and sum(sy): s = 1e-4*exp10(sum(sy*log10(sort(sx)))/sum(sy))
    else: s = 1e-6
    x,iy = adjspac([x,iy],s=s,verb=1)
#    t,c,k = splrep(x,iy,s=0,k=3)
    ps = [0.025,0.05,0.16,0.5,0.84,0.95,0.975]
    tx = concatenate([k*[x[0]],x,k*[x[-1]]])
    g = lambda r,p: splev(r,[tx,iy,k])-p
    cs = [fsolve(g,[x[clip(searchsorted(iy,p),0,len(x)-1)]],args=(p,)) for p in ps]
    m = argmax(y)
    p = x[m]
    return concatenate([[p],cs])

def adjspac(A,s=1e-6,extras=None,verb=1):
   x = A[0]
   o = argsort(x)
   A = take(A,o,1)
   if extras: extras = [take(extra,o) for extra in extras]
   x = A[0]
   dx = diff(x)
   while sometrue(equal(dx,0)):
      if verb: print "adjusting spacings...", sum(equal(dx,0.0))
      p = concatenate([[0.0],equal(dx,0)])
      x = x + s * p * (random(p.shape)-0.5)
      A[0] = x
      o = argsort(x)
      A = take(A,o,1)
      if extras: extras = [take(extra,o) for extra in extras]
      x = A[0]
      dx = diff(x)
   if extras: return A,extras
   else: return A



def ObjInfo(ffile):
    F = FITS(ffile)
    objname = F["object"].strip()
    exptime = F["exptime"]
    aperture = F["aperture"]
    F.close()
    if "obj" in objname: n = int(objname[3:])
    elif "ref" in objname: n = int(objname[3:])
    else: n = int(objname[1:])
    info = {"exptime":exptime,"aperture":aperture,"n":n,"object":objname}
    return info

