#!/usr/bin/env python

import shutil
import copy
import os
from time import time
from sys import stdout
from basics import *
from pgplot import *
from FITS import *
from GalPhot import *

class nphot:
  def __init__(self):
    self.galphot = GalPhot()
    self.galphot.initparams()
    self.justtheneighbors = 0
    self.removethemall = 0
    self.minmag = 99.9
    self.satmax = 65535.0
    self.nfit = 9999
    self.minr = 1.0
    self.maxr = 10.0
    self.shapr = 1.0
    self.cenr = 1.0
    self.seeing = 4.0
    self.starcut = 0.95
    self.galphots = [self.galphot]
  def readcat(self,catalog):
    catdat = readtable(catalog,icols=[0],header=0)
    objids = catdat["n"].tolist()
    xs, ys = catdat["x"], catdat["y"]
    xs = xs + 1
    ys = ys + 1
    rimage = catdat["r"]
    elimage = catdat["ell"]
    thimage = catdat["pa"]
    aimage = rimage/sqrt(elimage)
    bimage = power(rimage,2)/aimage
#    aimage, bimage = catdat[3], catdat[4]
#    aimage, bimage = 3*aimage, 3*bimage
    ordering = argsort(ys)
    xs,ys,aimage,bimage,elimage,thimage = \
            take([xs,ys,aimage,bimage,elimage,thimage],ordering,1)
    objids = [objids[o] for o in ordering]
    self.xs = xs
    self.ys = ys
    self.aimage = aimage
    self.bimage = bimage
    self.elimage = elimage
    self.thimage = thimage
    self.objids = objids
  def getcenter(self,objid):
    self.objid = objid
    self.obj = self.objids.index(self.objid)
    self.objx = self.xs[self.obj]
    self.objy = self.ys[self.obj]
    self.obja = self.aimage[self.obj]
    self.objb = self.bimage[self.obj]
    print "Object center=",self.objx,self.objy
    self.nomask = []
  def getimage(self,image,box=None,photdir="GALPHOT",suff="res"):
    self.image = image
    self.photdir = photdir
    if self.photdir[-1] != "/": self.photdir += "/"
    self.resimage = replace(self.image,".fits",suff+".fits")
    self.boxsize = box
    self.box = int(box*self.obja+0.5)
    f = FITS(self.image)
    self.naxes = f.naxes()
    if box:
      self.ll = [int(self.objx)-self.box/2, int(self.objy)-self.box/2]
      self.ur = [int(self.objx)+self.box/2, int(self.objy)+self.box/2]
      print "Extracting box:", self.ll, self.ur
      self.origimage = f.readsec(self.ll,self.ur)
      self.galphot.xcen = self.objx - (int(self.objx)-self.box/2)
      self.galphot.ycen = self.objy - (int(self.objy)-self.box/2)
      yi,xi=indices(self.origimage.shape) + 1.0
      self.xi = (xi + (int(self.objx)-self.box/2)).astype(Float32)
      self.yi = (yi + (int(self.objy)-self.box/2)).astype(Float32)
    else:
      self.origimage = f.data()
      self.galphot.xcen = self.objx
      self.galphot.ycen = self.objy
      yi,xi=indices(self.origimage.shape)
      self.xi = (xi + 1.0).astype(Float32)
      self.yi = (yi + 1.0).astype(Float32)
    f.close()
    qdump("sub.fits",self.origimage)
    self.rawimage = copy.copy(self.origimage)
  def createout(self,newsize=0):
     if not newsize:
        shutil.copy(self.image,self.resimage)
     else:
        f=FITS(self.resimage,create=1,template=self.image)
        f.blank()
        f.close()
  def pasteimage(self):
    print "Writing %s." % (self.resimage)
    if self.rawimage.shape == (self.naxes[1],self.naxes[0]):
       f = FITS(self.resimage,readonly=0)
       f.putdata(self.rawimage)
       f.close()
    else:
       f = FITS(self.resimage,readonly=0)
       f.writesec(self.rawimage,self.ll,self.ur)
       f.close()
  def defobjects(self,objlist):
    inside = greater(self.xs-4.5*self.aimage-2*self.seeing,self.xi[0,0])
    inside *= greater(self.ys-4.5*self.aimage-2*self.seeing,self.yi[0,0])
    inside *= less(self.xs+4.5*self.aimage+2*self.seeing,self.xi[-1,-1])
    inside *= less(self.ys+4.5*self.aimage+2*self.seeing,self.yi[-1,-1])
    use = nonzero(inside)
    self.xs,self.ys,self.aimage,self.bimage,self.elimage,self.thimage = \
       take([self.xs,self.ys,self.aimage,self.bimage,self.elimage,self.thimage],use,1)
    self.objids = [self.objids[o] for o in use]

    useful = greater(self.aimage,self.seeing/2)
    useful *= greater(self.bimage,self.seeing/2)

#    overlapx = less((self.xs - self.objx)/(self.maxr*self.aimage+0.5*self.maxr*self.obja),1)
#    overlapy = less((self.ys - self.objy)/(self.maxr*self.aimage+0.5*self.maxr*self.obja),1)
#    overlap = overlapx * overlapy

    distance = sqrt((self.xs - self.objx)**2 + (self.ys - self.objy)**2)
    overlap = less(distance,2*self.maxr*self.obja+2*self.maxr*self.aimage)
    useful *= overlap

    self.ordering = argsort(self.aimage)[::-1]
#    self.ordering = argsort(divz(distance,self.aimage))
    useful = take(useful,self.ordering)
    self.ordering = compress(useful,self.ordering)
    self.tofit = [self.objids[o] for o in self.ordering]
    if self.justtheneighbors: self.tofit = self.tofit[1:]
    if objlist:
       objlistobj = readtable(objlist,icols=[0])[0]
       if (self.removethemall or self.justtheneighbors):
          oo = nonzero([o in objlistobj for o in self.tofit])
          self.tofit = [self.tofit[o] for o in oo]
       else:
          oo = nonzero([o not in objlistobj for o in self.tofit])
          self.tofit = [self.tofit[o] for o in oo]
    print "Doing objects=",self.tofit
    self.nomask = []
    if objlist:
       self.nomask = readtable(objlist,icols=[0])[0].tolist()
       print self.nomask

  def mkmask(self,simple=0,verb=0):
    clobber = greater(self.aimage,1) * greater(self.bimage,1)
    maskn = nonzero(clobber)
    maskim = zeros(self.rawimage.shape,Float32)
    if len(self.nomask) == 1: mc = open("maskobj.cat","w")
    for n in maskn:

      if self.objids[n] in self.nomask:
          y1,x1 = indices((4*self.seeing,4*self.seeing))
          x2 = x1 + self.xs[n] - 2*self.seeing
          y2 = y1 + self.ys[n] - 2*self.seeing
          x1 = x2 - self.xs[n]
          y1 = y2 - self.ys[n]
          m1 = less(sqrt(power(x1/self.seeing,2)+power(y1/self.seeing,2)),2.0)
          x2i = int(self.xs[n]-2*self.seeing-self.xi[0,0])
          y2i = int(self.ys[n]-2*self.seeing-self.yi[0,0])
          x3i = x2i + m1.shape[1]
          y3i = y2i + m1.shape[0]
          maskim[y2i:y3i,x2i:x3i] += (maskim[y2i:y3i,x2i:x3i] + m1).astype(Float32)
          if verb:
            print "%7d %7.1f %7.1f %7.1f %7.1f" % \
             (self.objids[n],self.xs[n]-self.xi[0,0],self.ys[n]-self.yi[0,0],self.aimage[n],self.bimage[n])
          if len(self.nomask) == 1:
            print >>mc, "%7d %7.1f %7.1f %7.1f %7.1f" % \
             (self.objids[n],self.xs[n]-self.xi[0,0],self.ys[n]-self.yi[0,0],self.aimage[n],self.bimage[n])
          stdout.flush()
      else:
          y1,x1 = indices((6*self.aimage[n],6*self.aimage[n]))
          x2 = x1 + self.xs[n] - 3*self.aimage[n]
          y2 = y1 + self.ys[n] - 3*self.aimage[n]
          x1 = x2 - self.xs[n]
          y1 = y2 - self.ys[n]
          pa = self.thimage[n] * (pi/180) + pi/2
          cpa, spa = cos(pa), sin(pa)
          x1,y1 = x1*cpa + y1*spa, -x1*spa + y1*cpa
          m1 = less(sqrt(power(x1/self.aimage[n],2)+power(y1/self.bimage[n],2)),2.0)
          m1 = m1 + less(sqrt(power(x1,2)+power(y1,2)),1.5*self.seeing)
          x2i = int(self.xs[n]-3*self.aimage[n]-self.xi[0,0])
          y2i = int(self.ys[n]-3*self.aimage[n]-self.yi[0,0])
          x3i = x2i + m1.shape[1]
          y3i = y2i + m1.shape[0]
          maskim[y2i:y3i,x2i:x3i] += (maskim[y2i:y3i,x2i:x3i] + m1).astype(Float32)
          if verb:
            print "%7d %7.1f %7.1f %7.1f %7.1f" % \
             (self.objids[n],self.xs[n]-self.xi[0,0],self.ys[n]-self.yi[0,0],self.aimage[n],self.bimage[n])
          if len(self.nomask) == 1:
            print >>mc, "%7d %7.1f %7.1f %7.1f %7.1f" % \
             (self.objids[n],self.xs[n]-self.xi[0,0],self.ys[n]-self.yi[0,0],self.aimage[n],self.bimage[n])
          stdout.flush()
    if len(self.nomask) == 1: mc.close()
    maskim += greater(self.rawimage,self.satmax).astype(Float32)
    if simple:
      currobj = self.nomask[0]
    else:
      currobj = self.nomask[-1]
    if currobj in self.objids:
        n = self.objids.index(currobj)
        y1,x1 = indices((4*self.seeing,4*self.seeing))
        x2 = x1 + self.xs[n] - 2*self.seeing
        y2 = y1 + self.ys[n] - 2*self.seeing
        x1 = x2 - self.xs[n]
        y1 = y2 - self.ys[n]
        m1 = less(sqrt(power(x1/self.seeing,2)+power(y1/self.seeing,2)),2.0)
        x2i = int(self.xs[n]-2*self.seeing-self.xi[0,0])
        y2i = int(self.ys[n]-2*self.seeing-self.yi[0,0])
        x3i = x2i + m1.shape[1]
        y3i = y2i + m1.shape[0]
        maskim[y2i:y3i,x2i:x3i] = where(m1,0,maskim[y2i:y3i,x2i:x3i]).astype(maskim.typecode())
    self.galphot.image = where(maskim,VALBAD,self.rawimage).astype(Float32)
    qdump("fit.fits",self.galphot.image)

  def photfit(self,obj):
    tablename = self.photdir+"/galphot"+zfill(obj,5)+".dat"
    if os.path.exists( tablename): os.remove( tablename)
    print "Running GALPHOT."
    t0 = time()
    self.galphot.photfit()
    t1 = time()
    print "Time to run GALPHOT was %.2f seconds." % (t1-t0)
    self.galphot.x[::,0] = (self.galphot.x[::,0] + self.xi[0,0] - 2).astype(Float32)
    self.galphot.y[::,0] = (self.galphot.y[::,0] + self.yi[0,0] - 2).astype(Float32)
    print "Writing to table %s." % (tablename)
    self.galphot.writetable(tablename)

  def rmobj(self,obj):
    n = self.objids.index(obj)
    tablename = self.photdir+"galphot"+zfill(obj,5)+".dat"
    print "Reading %s." % (tablename)
    self.galphot.readtable(tablename)
    self.galphot.x[::,0] = (self.galphot.x[::,0] - self.xi[0,0] + 2).astype(Float32)
    self.galphot.y[::,0] = (self.galphot.y[::,0] - self.yi[0,0] + 2).astype(Float32)
    self.galphot.x0[::,0] = (self.galphot.x0[::,0] - self.xi[0,0] + 2).astype(Float32)
    self.galphot.y0[::,0] = (self.galphot.y0[::,0] - self.yi[0,0] + 2).astype(Float32)
    self.galphot.xcen = mean(self.galphot.x[::,0])
    self.galphot.ycen = mean(self.galphot.y[::,0])
    self.galphot.image = self.rawimage
    self.galphot.justres = 1
    self.galphot.tabellipfit = 1
    self.galphot.ellipsefit = 1
    print "Subtracting Object #%d" % (self.tofit.index(obj)+1)
    print "Object is described by (%.1f,%.1f,%.1f)" % (self.aimage[n],self.bimage[n],self.thimage[n])
    print "Object center is (%.1f,%.1f)" % (mean(self.galphot.x[::,0]), mean(self.galphot.y[::,0]))
    t0 = time()
    self.galphot.photfit()
    t1 = time()
    print "Time to run GALPHOT was %.2f seconds." % (t1-t0)
    self.rawimage = (self.galphot.resim+self.galphot.i[-1,0])
    qdump("res.fits",self.rawimage)

  def mkobj(self,obj,pos=0):
    n = self.objids.index(obj)
    tablename = self.photdir+"galphot"+zfill(obj,5)+".dat"
    print "Reading %s." % (tablename)
    self.galphot.readtable(tablename)
    if pos:
       self.galphot.x[::,0] = (ones(self.galphot.x[::,0].shape)*pos).astype(Float32)
       self.galphot.y[::,0] = (ones(self.galphot.y[::,0].shape)*pos).astype(Float32)
       self.galphot.x0[::,0] = (ones(self.galphot.x0[::,0].shape)*pos).astype(Float32)
       self.galphot.y0[::,0] = (ones(self.galphot.y0[::,0].shape)*pos).astype(Float32)
       self.galphot.xcen = pos
       self.galphot.ycen = pos
    else:
       self.galphot.x[::,0] = (self.galphot.x[::,0] - self.xi[0,0] + 2).astype(Float32)
       self.galphot.y[::,0] = (self.galphot.y[::,0] - self.yi[0,0] + 2).astype(Float32)
       self.galphot.x0[::,0] = (self.galphot.x0[::,0] - self.xi[0,0] + 2).astype(Float32)
       self.galphot.y0[::,0] = (self.galphot.y0[::,0] - self.yi[0,0] + 2).astype(Float32)
       self.galphot.xcen = mean(self.galphot.x[::,0])
       self.galphot.ycen = mean(self.galphot.y[::,0])
    self.galphot.image = self.rawimage
    self.galphot.justres = 1
    self.galphot.tabellipfit = 1
    self.galphot.ellipsefit = 1
    print "Making Object #%d" % (self.tofit.index(obj)+1)
    print "Object is described by (%.1f,%.1f,%.1f)" % (self.aimage[n],self.bimage[n],self.thimage[n])
    print "Object center is (%.1f,%.1f)" % (mean(self.galphot.x[::,0]), mean(self.galphot.y[::,0]))
    t0 = time()
    self.galphot.photfit()
    t1 = time()
    print "Time to run GALPHOT was %.2f seconds." % (t1-t0)
    self.rawimage = self.rawimage-(self.galphot.resim+self.galphot.i[-1,0])
    qdump("mod.fits",self.rawimage)

  def fitobj(self,obj,objlist):
    n = self.objids.index(obj)
    r = sqrt(self.aimage[n]*self.bimage[n])
    self.nomask += [obj]
    self.galphot.initparams()
    self.galphot.xcen = self.xs[n]-self.xi[0,0]+1
    self.galphot.ycen = self.ys[n]-self.yi[0,0]+1
    xedge = min([self.galphot.xcen, self.rawimage.shape[1] - self.galphot.xcen])
    yedge = min([self.galphot.ycen, self.rawimage.shape[0] - self.galphot.ycen])
    edge = min([xedge,yedge])
    self.galphot.rmin = self.minr
    self.galphot.rmax = min([r*self.maxr,edge])
    self.galphot.rshap = min([r*self.shapr,edge])
    self.galphot.rcen = min([r*self.cenr,edge])
    print "Object #%d" % (self.tofit.index(obj)+1)
    print "Fitting %d at (%.1f,%.1f)" % (obj,self.galphot.xcen,self.galphot.ycen)
    print "Object is described by (%.1f,%.1f,%.1f)" % (self.aimage[n],self.bimage[n],self.thimage[n])
    print "Not Masking",self.nomask
    if (self.removethemall or self.justtheneighbors) and objlist:
       self.mkmask(simple=1)
    else:
       self.mkmask(simple=0)
    self.photfit(obj)
    try: self.rmobj(obj)
    except:
       print "Problem removing object %d. Killing it." % (obj)
       tablename = self.photdir+"galphot"+zfill(obj,5)+".dat"
       if os.path.exists( tablename): os.remove( tablename)

