
from sys import stdout
from types import *
from pgplot import *
from masks import *
from basics import *
from pgapack import *

class MaskGenerate:
   def __init__(self,nmasks,sim_in,mask_pre,remain_pre,pgadat,arguments,paus=0):
     self.nmasks = nmasks
     self.sim_in = sim_in
     self.mask_pre = mask_pre
     self.remain_pre = remain_pre
     self.pgadat = pgadat
     self.paus = paus
     self.arguments = arguments
     self.gout = open(self.pgadat,"w")
     self.gpars = []
     self.gfits = []

   def Finalize(self):
      x = self.gpars[argmax(self.gfits)]
      self.Generate(x,keep=1)
      self.gout.close()

   def Command(self,csra,csdec,cspa,sim_in,mask_out,sim_out):
      command = ["sim2mask",sim_in,mask_out,sim_out]
      command += self.arguments
      command += ["-ra",csra]
      command += ["-dec",csdec]
      command += ["-pa",cspa]
      command = " ".join(map(str,command))
      return command

   def Generate(self,x,keep=0):
      prs = []
      self.gpars.append(x)
      mask_outs = []
      sim_outs = []
      for jmask in range(self.nmasks):
          ra,dec,pa = x[3*jmask:3*(jmask+1)]
          csra = formatSexagesimal(radhr(ra),4,3600000)
          csdec = formatSexagesimal(raddeg(dec),4,3600000)
          cspa = raddeg(pa)
          if jmask: sim_in = "%s%s.sim" % (self.remain_pre,letters[jmask-1])
          else: sim_in = self.sim_in
          if self.nmasks > 1:
            mask_out = "%s%s.sim" % (self.mask_pre,letters[jmask])
            sim_out = "%s%s.sim" % (self.remain_pre,letters[jmask])
          else:
            mask_out = "%s.sim" % (self.mask_pre)
            sim_out = "%s.sim" % (self.remain_pre)
          command = self.Command(csra,csdec,cspa,sim_in,mask_out,sim_out)
          mask_outs.append(mask_out)
          sim_outs.append(sim_out)
          verbex(command)
          try:
              mask = readsim(mask_out)[0]
              obj = mask[0]
              priors = array([mask[1][j] for (j,o) in enumerate(obj) if o != "CENTER" and mask[1][j] > 0])
              print "Priorities"
              print priors
          except:
              priors = []
              n = 0
              mp = 0.0
          prs.append(priors)
      if not keep:
        [clean(mask_out) for mask_out in mask_outs]
        [clean(sim_out) for sim_out in sim_outs]
      prs = concatenate(prs)
      n = len(prs)
#      mp = n * pow(10,log10(multiply.reduce(prs))/n)
      mp = sum(prs)
      print "M(Priorities)=%f" % (mp)
      print >> self.gout, "%4d %15.5f" % (n,mp),
      print "%4d %15.5f" % (n,mp),
      for jmask in range(self.nmasks):
          ra,dec,pa = x[3*jmask:3*(jmask+1)]
          csra = formatSexagesimal(radhr(ra),4,3600000)
          csdec = formatSexagesimal(raddeg(dec),4,3600000)
          cspa = raddeg(pa)
          print >> self.gout, "%13s %13s %7.3f" % (csra,csdec,cspa),
          print "%13s %13s %7.3f" % (csra,csdec,cspa),
      print >> self.gout
      print
      self.gout.flush()
      self.gfits.append(mp)
      if self.paus:
         key = raw_input(" << paused >> ")
         if key == "quit": os._exit(0)
         if key == "go": self.paus = 0
      return mp

def clean(f):
  if os.path.exists(f): verbex("rm %s" % (f))


