
import copy
from sets import Set
from sys import stdout,exit
from scipy.special import *
from MLab import *
from FITS import *
from Multipack import *
from VTKHelperFunctions import *
from basics import *
from pgplot import *

argv,drtol = rdarg(argv,"-tol",float,1e-10)
ldrtol = log10(drtol)

def gaussian_resid(params, w, g, var):
   w0,sigma,amp=params
   sigma=pow(10,sigma)
   kernel = -0.5*((w-w0)**2)/(sigma**2)
   resid = g-amp*exp(kernel*between(kernel,-6,6))*between(kernel,-6,6)
   chi = divz(resid,getsqrt(var))
   return chi

def gaussians(params, w, y, var, n, fit=1):
   wavs = params[::3]
   sigs = power(10,params[1::3])
   amps = params[2::3]
   kernels = -0.5*((w[::,NewAxis]-wavs[NewAxis,::])**2)/(sigs[NewAxis,::]**2)
   fitted = amps[NewAxis,::]*exp(kernels*between(kernels,-10,10))*between(kernels,-10,10)
   fitted = sum(fitted,1)
   if fit:
       resid = y-fitted
       chi = divz(resid,getsqrt(var))
       return chi
   else:
       return fitted

def fitgaussian(xdata, ydata, yvariance, amp='guess', sigma='guess', pos='guess'):
   if (amp=='guess'): amp=max(ydata)
   if (sigma=='guess'): sigma=1+abs(xdata[-1]-xdata[0])/6.0
   if (pos=='guess'): pos=(xdata[-1]+xdata[0])/2.0    
   guess=[pos,log10(sigma),amp]
   sol=leastsq(gaussian_resid,guess,args=(xdata,ydata,yvariance),full_output=1,factor=0.1,ftol=1e-12,xtol=1e-12)
   errs = divz(1,getsqrt(diag(cov(transpose(sol[1]['fjac'])))))
   errs = take(errs,argsort(sol[1]["ipvt"]))
   return sol[0][0],errs[0],10**sol[0][1],errs[1]*10**sol[0][1]/2.303,sol[0][2],errs[2]

def uspv(l, tck, m):
   d = splev(l,tck)
   return asarray([m-d],Float64)

def splpeaky(tck, l, m):
   sol = leastsq(uspv, l, args=(tck,m), factor=0.001, full_output=1,
        epsfcn=drtol, ftol=drtol, xtol=drtol, gtol=0.0, maxfev=100000)
   return sol

def edgy(vtck,l):
        z = sproot(vtck,mest=len(vtck[1]))
        if sum(less(z,l)): z1 = sort(compress(less(z,l),l-z))
        else: z1 = None
        if sum(greater(z,l)): z2 = sort(compress(less(l,z),z-l))
        else: z2 = None
        if z1 and z2: return l-z1[0],l+z2[0]
        else: return l,l

def centery(s,m):
   f = edgy(s,m)
   c = mean(f)
   return c, m-f[0], f[1]-m

def trebin(z,twave,tspec,gwave,tk=3,mask=0):
#   if z>0: rwave = twave * (1.0+abs(z))
#   else: rwave = twave / (1.0+abs(z))
   rwave = twave * (1.0+z)
   rspec = tspec
   if not mask:
      dr = diff(rwave)
      dr = concatenate([dr,[dr[-1]]])
      dg = diff(gwave)
      dg = concatenate([dg,[dg[-1]]])
      rspec = rspec/dr
   spl = splrep(rwave,rspec,s=0,k=tk,xb=rwave[0],xe=rwave[-1])
   rspec = splev(gwave,spl)
   del spl
   ruse = between(gwave,rwave[0],rwave[-1])
   rspec = rspec*ruse
   if not mask: rspec = rspec*dg
   return rspec,ruse

def getpoly(x,npoly):
   pbase = legendre(x,npoly)
   return pbase
#  pbase=zeros((int(npoly+1),x.shape[0]),Float64)
#  for i in range(1+npoly):
#    pbase[i] = power(x,i)
#  return pbase

def getharm(x,nharm):
  hbase=zeros((int(2*nharm+1),x.shape[0]),Float64)
  hbase[0]=ones(x.shape,Float64)
  for i in range(1,1+nharm):
    hbase[2*i-1] = sin(i*x)
    hbase[2*i]   = cos(i*x)
  return hbase

def Fnormalize(spec,wt,ffilt,norm=0):
   l = len(spec)
   x = arange(0,l,1,Float64)
   x = 2*pi*x/l
   ux,us,uw = compress(wt,[x,spec,wt],1)
#   us = log(us)
   if not len(us): os._exit(1)
   basis = getharm(ux,ffilt)
   sol = svdfit(transpose(basis)*uw[::,NewAxis],us*uw)
   basis = getharm(x,ffilt)
   continuum = dot(sol,basis)
#   continuum = getexp(continuum)
   if norm: normalized = where(continuum,spec/(continuum+equal(continuum,0)),0.0) - 1
   else: normalized = where(greater(wt,0),spec,continuum)
   del basis
   return normalized, continuum

def Pnormalize(spec,wt,pfilt,norm=0):
   l = len(spec)
   x = arange(0,l,1,Float64)
   x = (x-l/2.0)/(l/2.0)
   ux,us,uw = compress(wt,[x,spec,wt],1)
#   us = log(us)
   if not len(us): os._exit(1)
   basis = getpoly(ux,pfilt)
   sol = svdfit(transpose(basis)*uw[::,NewAxis],us*uw)
   basis = getpoly(x,pfilt)
   continuum = dot(sol,basis)
#   continuum = getexp(continuum)
   if norm: normalized = where(continuum,spec/(continuum+equal(continuum,0)),0.0) - 1
   else: normalized = where(greater(wt,0),spec,continuum)
   del basis
   return normalized, continuum

def Gnormalize(spec,wt,gfilt,norm=0):
   wt2 = power(wt,2)
   numer = VTKGauss(spec*wt2,gfilt,numret=1)
   denom = VTKGauss(wt2,gfilt,numret=1)
   continuum = divz(numer,denom)
   if norm: normalized = where(continuum,spec/(continuum+equal(continuum,0)),0.0) - 1
   else: normalized = where(greater(wt,0),spec,continuum)
   return normalized, continuum

def Bnormalize(spec,wt,bfilt,norm=0):
   l = len(spec)
   x = arange(0,l,1,Float64)
   ux,us,uw = compress(wt,[x,spec,wt],1)
   ux0 = ux[0]
   ux1 = ux[-1]
   uxm = int(0.5*(ux0+ux1))
   tw1 = arange(ux0,uxm-1,bfilt,Float64)
   tw2 = arange(uxm,ux1+bfilt/2,bfilt,Float64)[1:]
   twf = sort(concatenate([tw1,tw2])).astype(Int32)
   tw = compress(between(twf,ux0,ux1,inc=1),twf)
   tw = compress(take(wt,tw),tw)
   tw11 = [tw[0]-bfilt]
   tw22 = [tw[-1]+bfilt]
   tw = sort(concatenate([tw11,tw,tw22])).astype(Int32)
   if len(tw) <= 3: tw = concatenate([3*[tw[0]],tw,3*[tw[-1]]])
   else: tw = concatenate([2*[tw[0]],tw,2*[tw[-1]]])
   tw = sort(tw)
   tw = tw.astype(Float64)
   tck = splrep(ux,us,w=uw,xb=ux0,xe=ux1,k=3,t=tw,
        nest=len(tw),task=-1,s=None,full_output=0,quite=1)
   continuum = splev(x,tck)
   del tck
   if norm:
      normalized = where(continuum,spec/(continuum+equal(continuum,0)),0.0) - 1
   else:
      normalized = where(greater(wt,0),spec,continuum)
   return normalized, continuum

def crossprep(z,zfilt,twave,tspec,temis,gwave,gspec,gemis,vspec,gfilt,bfilt,ffilt,pfilt,masks,tframe,tk=3):
   amask,bmask,emask,umask = masks
   nv = nonzero(vspec)
#   if tframe:
#     tbad = logical_not(between((1+z)*twave,gwave[nv[0]],gwave[nv[-1]]))
#   else: tbad = zeros(twave.shape)
   tbad = zeros(twave.shape)
   if amask:
      tbad += between(twave,6845,6970)
      tbad += between(twave,7580,7665)
   tokay = 1.0*equal(tbad+temis,0)
   tokay = greater_equal(trebin(z,twave,tokay,gwave,tk=1,mask=1)[0],0.8)
   vspec = vspec * tokay
   gbad = zeros(gwave.shape)
   if amask:
      gbad += between(gwave,6845,6970)
      gbad += between(gwave,7580,7665)
   gokay = equal(gbad+gemis,0)
   vspec = vspec * gokay
   if umask != []:
       for um in umask:
          gbad = between(gwave,um[0],um[1],inc=1)
          gokay = equal(gbad,0)
          vspec = vspec * gokay
   rspec,ruse = trebin(z,twave,tspec,gwave,tk)
   wt = where(greater(vspec,0),ruse/(vspec+equal(vspec,0)),0.0)
   wt = getsqrt(wt)
   if gfilt != 0:
      if gfilt > 0: gfilt2 = int((1.0+zfilt)*gfilt/mean(diff(gwave)))
      else: gfilt2 = int(abs(gfilt)/mean(diff(gwave)))
      crspec, rcont = Gnormalize(rspec,wt,gfilt2,norm=1)
      cgspec, gcont = Gnormalize(gspec,wt,gfilt2,norm=1)
   elif bfilt != 0:
      if bfilt > 0: bfilt2 = int((1.0+zfilt)*bfilt/mean(diff(gwave)))
      else: bfilt2 = int(abs(bfilt)/mean(diff(gwave)))
      crspec, rcont = Bnormalize(rspec,wt,bfilt2,norm=1)
      cgspec, gcont = Bnormalize(gspec,wt,bfilt2,norm=1)
   elif ffilt != 0:
      if ffilt > 0: ffilt2 = int(ffilt/(1.0+zfilt))
      else: ffilt2 = abs(ffilt)
      wt = wt * greater(rspec,0)*greater(gspec,0)
      crspec, rcont = Fnormalize(rspec,wt,ffilt2,norm=1)
      cgspec, gcont = Fnormalize(gspec,wt,ffilt2,norm=1)
      wt = wt * greater(rcont,0)*greater(gcont,0)
   else:
      wt = wt * greater(rspec,0)*greater(gspec,0)
      crspec, rcont = Pnormalize(rspec,wt,pfilt,norm=1)
      cgspec, gcont = Pnormalize(gspec,wt,pfilt,norm=1)
      wt = wt * greater(rcont,0)*greater(gcont,0)
   return crspec, rcont, cgspec, gcont, wt

def crosscor1( crspec, cgspec, rcont, gcont, wt):
   cc = sum(cgspec*crspec*wt*power(gcont,2))
   return cc

#def deconvolve(crspec,cgspec,wt,verb=0):
#   shiftn1 = VTKImageShift(crspec,+1,0,numret=1)
#   shiftp1 = VTKImageShift(crspec,-1,0,numret=1)
#   basis = transpose([shiftn1,crspec,shiftp1])
#   decon = svdfit(basis*wt[::,NewAxis],cgspec*wt)
#   decon[0], decon[2] = (decon[0] + decon[2])/2.0, (decon[0] + decon[2])/2.0
#   if verb: print decon
#   crspec = add.reduce(basis*decon,1)
#   return crspec

def crosscor(z,zfilt,twave,tspec,temis,gwave,gspec,gemis,vspec,gfilt,bfilt,ffilt,pfilt,masks,tframe,tk=3,fitting=0,verb=0):
   if fitting: z = z[0]
   crspec,rcont,cgspec,gcont,wt=crossprep(z,zfilt,twave,tspec,temis,gwave,gspec,gemis,vspec,gfilt,bfilt,ffilt,pfilt,masks,tframe,tk=tk)
   cc = crosscor1(crspec,cgspec,rcont, gcont, wt)
   if fitting:
      cm = fitting
      cm = crosscor1(cgspec,cgspec,rcont,gcont, wt)
      if verb: print z,cm,cc
      cc = [sqrt(2*cm-2*cc)]
   elif verb: print z,cc
   return cc

def iterclip(s,w0,f,th=5,iter=1,sm=1):
   f = int(f)
   t = Fnormalize(s,w0,f,norm=0)[1]
   n = 1*t
   w = 1*w0
   for j in range(iter):
      r = 1.49*Fnormalize(abs(s-n),w,f/3,norm=0)[1]
      m = less(s-t,th*r)
#      print sum(m)
      w = w0 * m
      n = Fnormalize(s,w,f,norm=0)[1]
   r = 1.49*Fnormalize(abs(s-n),w,f/3,norm=0)[1]
   w = greater(s-n,th*r)
   w = greater(funcfilt(sum,w,sm,0),0)
   g = where(w,n,s)
   return g,n

def GetRedshift(gwave,gspec,nspec,twave,tspec,w1,w2,guess,zrange,nz,masks,template="",inspec="",\
      estnoise=0,rescale=0,gfilt=0,bfilt=0,ffilt=0,pfilt=0,tk=3,efit=0,efilt=25,citer=4,efind=5,trlim=4,\
      prompt=0,pgdev="/null",pgsize=8,pu=0,dps=0,tps=0,dump=0,inctitle=None):

    [amask,bmask,emask,umask] = masks

    guse = ones(gwave.shape)
    if umask != []:
      for um in umask:
         gbad = between(gwave,um[0],um[1],inc=1)
         guse = guse * equal(gbad,0)

    vspec = power(nspec,2)
    vspec_orig = 1*vspec

    temis = zeros(tspec.shape)
    gemis = zeros(gspec.shape)

    if emask:
        gbad = between(gwave,6830,6970)+ between(gwave,7585,7665)
        for um in umask: gbad = gbad + between(gwave,um[0],um[1],inc=1)
        gokay = equal(gbad,0)
        uset = 1.0*logical_not(between(twave,6830,6970)+ between(twave,7585,7665))*greater(tspec,0)
        useg = divz(gokay,nspec,out=Float64)*greater(gspec,0)
        tefilt = efilt * mean(diff(gwave))/mean(diff(twave))
        stspec,stcont = iterclip(tspec,uset,tefilt,th=emask,iter=citer)
        sgspec,sgcont = iterclip(gspec,useg,efilt,th=emask,iter=citer)
        vspec = vspec*equal(gspec,sgspec)
        if efit:
            temis = zeros(tspec.shape)
            gemis = zeros(gspec.shape)
        else:
            tspec = stspec
            gspec = sgspec
            temis = zeros(tspec.shape)
            gemis = zeros(gspec.shape)
            vspec = vspec*equal(gspec,sgspec)
    #    gspec = sgspec

    nw = nonzero(nspec)
    if w1 < 0 and w2 < 0:
       tframe = 1
       w1,w2 = abs(w1),abs(w2)
       twave,tspec,temis = compress( between(twave,w1,w2,inc=1), [twave,tspec,temis], 1)
    else:
       tframe = 0
       if w1: w1 = max([w1,(1+guess+zrange/2)*twave[0]])
       if w2: w2 = min([w2,(1+guess-zrange/2)*twave[-1]])
       if w1: n1 = max([nw[0],searchsorted(gwave,w1)]); vspec[:n1] = 0*vspec[:n1]
       if w2: n2 = min([nw[-1],searchsorted(gwave,w2)]); vspec[n2:] = 0*vspec[n2:]

    if w2 < w1: return {}

    print "Using (w1,w2)=(%f,%f)" % (w1,w2)
    #if tframe:
    #  w1 = max([gwave[0],w1*(1+redshift)])
    #  w2 = min([gwave[-1],w2*(1+redshift)])

    #w1,w2 = extrema(gwave)

    if nz:
       zres = zrange/nz
    else:
       tws,tvs,tgs = compress(vspec,[gwave,vspec,gspec],1)
       tsn = sqrt(sum(tgs/tvs)/sum(1/sqrt(tvs)))
#       if gdcflag: zres = gcdelt1*log(10)
#       else: zres = gcdelt1/(sum(divz(tws,sqrt(tvs),out=Float64))/sum(divz(1.0,sqrt(tvs),out=Float64)))
       zres = mean(diff(gwave))/(sum(divz(tws,sqrt(tvs),out=Float64))/sum(divz(1.0,sqrt(tvs),out=Float64)))
       nz = int(zrange/zres/2)
       print "Using %d bins (%.1f)." % (nz,zres*CLIGHTK)

    if nz == 1 and zrange == 0:
      ztests = [guess]
      just1 = 1
    else:
      z0 = guess-zrange/2.0
      z1 = guess+zrange/2.0
      dz = (z1-z0)/nz
      ztests = arange(z0,z1+dz/2.0,dz,Float64)
      just1 = 0

      cc = zeros(ztests.shape,Float64)
      gcc = zeros(ztests.shape,Float64)

      for j in range(len(ztests)):
         stdout.write("."); stdout.flush()
         cc[j] = crosscor(ztests[j],ztests[j],twave,tspec,temis,gwave,gspec,gemis,vspec,gfilt,bfilt,ffilt,pfilt,masks,tframe,tk=tk)
      print
      print "Totals = %8.4f" % (sum(cc))

      ccpeak = max(cc)
      cmax = 1.5*ccpeak
      redmax = ztests[argmax(cc)]

    if just1:

       redshift = guess
       opendev(pgdev,land=0,size=pgsize,asp=1.0,nx=1,ny=3)

    else:

       fmt = "   %14.11f %8.3f %8.3f %14.11f %9.3f"
       rederr1 = 1.0e24
       rederr2 = 1.0e24
       while (rederr1==rederr2):
          dr = 1e38
          redshift1 = ztests[argmax(cc)]
          pfac = 2.0
          niter = 0
          while abs(dr) > drtol and niter < 50:
             niter += 1
             tspl = splrep(ztests,cc,s=0,k=3)
             dr = splpeaky(tspl,redshift1,(1+pfac)*max(cc))[0] - redshift1
             redshift = redshift1 + dr
             if abs(dr) > drtol:
                ccpeak = crosscor(redshift,redmax,twave,tspec,temis,gwave,gspec,gemis,vspec,gfilt,bfilt,ffilt,pfilt,masks,tframe,tk=tk)
                ztests = concatenate([ztests,[redshift]])
                cc = concatenate([cc,[ccpeak]])
                ztests,cc = take([ztests,cc],argsort(ztests),1)
                print fmt % (redshift,ldrtol,getlog10(abs(dr)),dr,ccpeak)
                redshift1 = redshift
                pfac = pfac/2.0
             ccpeak = crosscor(redshift,redmax,twave,tspec,temis,gwave,gspec,gemis,vspec,gfilt,bfilt,ffilt,pfilt,masks,tframe,tk=tk)
          print fmt % (redshift,ldrtol,getlog10(abs(dr)),dr,ccpeak)
          print

          if not pu:
             if log10(ccpeak) > 3: pu = 4.0
             elif log10(ccpeak) > 2: pu = 4.0
             elif log10(ccpeak) > 1.5: pu = 4.0
             elif log10(ccpeak) > 1.0: pu = 4.0
             else: pu = 4.0

          dr1 = 1e38
          dr2 = 1e38
          ddrtol = 1e-3
          rederr1o = 1e38
          rederr2o = 1e38
          niter = 0
          while niter < 50 and (abs(dr1) > ddrtol or abs(dr2) > ddrtol):
             niter += 1
             b = less_equal(diff(ztests),0)
             if sum(b): ztests,cc = compress(equal(b,0),[ztests,cc],1)
             tspl = splrep(ztests,cc,s=0,k=3)
             espl = splrep(ztests,cc-(ccpeak-pu/2.0),s=0,k=3)
             credshift, rederr1, rederr2 = centery(espl,redshift)
             if rederr1 and rederr2:
                 dr1 = (rederr1-rederr1o)/rederr1
                 dr2 = (rederr2-rederr2o)/rederr2
             else:
                 dr1 = (rederr1-rederr1o)
                 dr2 = (rederr2-rederr2o)
             if abs(dr1) > ddrtol or abs(dr2) > ddrtol:
                if rederr1 > 0 and rederr2 > 0:
                  ccpeak1 = crosscor(redshift-rederr1,redmax,twave,tspec,temis,gwave,gspec,gemis,vspec,gfilt,bfilt,ffilt,pfilt,masks,tframe,tk=tk)
                  ztests = concatenate([ztests,[redshift-rederr1]])
                  cc = concatenate([cc,[ccpeak1]])
                  if (rederr1 > 0.1 and rederr2 > 0.1): rrs = [4,2,0.6666]
                  else: rrs = [3]
                  for rr in rrs:
                     ccpeak1b = crosscor(redshift-rederr1/rr,redmax,twave,tspec,temis,gwave,gspec,gemis,vspec,gfilt,bfilt,ffilt,pfilt,masks,tframe,tk=tk)
                     ztests = concatenate([ztests,[redshift-rederr1/rr]])
                     cc = concatenate([cc,[ccpeak1b]])
                  ccpeak2 = crosscor(redshift+rederr2,redmax,twave,tspec,temis,gwave,gspec,gemis,vspec,gfilt,bfilt,ffilt,pfilt,masks,tframe,tk=tk)
                  ztests = concatenate([ztests,[redshift+rederr2]])
                  cc = concatenate([cc,[ccpeak2]])
                  for rr in rrs:
                     ccpeak2b = crosscor(redshift+rederr2/rr,redmax,twave,tspec,temis,gwave,gspec,gemis,vspec,gfilt,bfilt,ffilt,pfilt,masks,tframe,tk=tk)
                     ztests = concatenate([ztests,[redshift+rederr2/rr]])
                     cc = concatenate([cc,[ccpeak2b]])
                  print credshift, rederr1, rederr2, ccpeak1, ccpeak2
                  rederr1o = rederr1
                  rederr2o = rederr2
                  ztests,cc = take([ztests,cc],argsort(ztests),1)
                else:
                  print credshift, rederr1, rederr2
                  return {}
          print

          rederr1 = rederr1/sqrt(pu)
          rederr2 = rederr2/sqrt(pu)
          print redshift, rederr1, rederr2
          print

          if (rederr1==rederr2==0):
             rednew = redshift+zres
             ccnew = crosscor(rednew,redmax,twave,tspec,temis,gwave,gspec,gemis,vspec,gfilt,bfilt,ffilt,pfilt,masks,tframe,tk)
             ztests = concatenate([ztests,[rednew]])
             cc = concatenate([cc,[ccnew]])

       if dps:
          pgdev = replace(inspec,".fits","_cc1.ps/vcps")
          opendev(pgdev,land=0,size=pgsize,asp=1.0/3,nx=1,ny=1)
       else:
          opendev(pgdev,land=0,size=pgsize,asp=1.2,nx=1,ny=3)
       cpgsvp(0.1,0.9,0.15,0.9)
       newbox([ztests[0],ztests[-1]],[-cmax/5,cmax],ch=2,ylog=0,manvp=1)
       xlabel("\\fiz",ch=2)
       ylabel("\\fiC\uw\d(z)",ch=2)
       drawln(ztests,cc,ls=1,hist=1)

    crspec,rcont,cgspec,gcont,wt = crossprep(redshift,redmax,twave,tspec,temis,gwave,gspec,gemis,vspec,gfilt,bfilt,ffilt,pfilt,masks,tframe,tk=tk)
    ##crspec = deconvolve(crspec,cgspec,wt,verb=1)
    ccmax = crosscor1( crspec,cgspec,rcont, gcont, wt)

    print "Value at redshift=",ccmax
    #resid = gspec - (crspec+1)*gcont
    #chi = compress(wt,divz(resid,nspec,out=Float64))
    resid = cgspec - crspec
    shortwt = compress(wt,wt)
    sumwt = sum(1.0*shortwt)
    chi = compress(wt,divz(resid,divz(nspec,gcont),out=Float64))
    if gfilt: kappa = (w2-w1)/abs(gfilt)
    elif bfilt: kappa = (w2-w1)/abs(bfilt)
    elif ffilt: kappa = 2*abs(ffilt)+1
    else: kappa = pfilt+1
    print "Number of pixels,kappa=",(chi.shape[0],int(kappa))
    chi2 = power(chi,2)
    ndeg = (chi2.shape[0]-int(kappa)-1)
    chi2 = sum(chi2)/ndeg
    print "ndeg=%d" % (ndeg)
    print "sumwt=%f" % (sumwt)
    print "Reduced \chi^2=%4.2f" % (chi2)

    wt1,rms1 = compress(wt,[wt,cgspec])
    wt2,rms2 = compress(wt,[wt,crspec])
    rms1 = sum(wt1*rms1**2)/sum(wt1)
    rms2 = sum(wt2*rms2**2)/sum(wt2)
    rrms = 0.5*(rms2+rms1)/rms1
    print "Frac of rms=%4.2f" % (rrms)

    if not just1:
       if 1 or (not estnoise and rescale):
            rederr1 = rederr1 * sqrt(chi2)
            rederr2 = rederr2 * sqrt(chi2)
    #   rederr2 = rederr2 * rrms
    #   rederr2 = rederr2 * rrms
       if inctitle:
          subtitle (inspec+" Redshift Peak = %.8f -%.8f +%.8f" % (redshift,rederr1,rederr2),ch=2)

       if redshift > guess: cpgsvp(0.17,0.35,0.5,0.85)
       else: cpgsvp(0.65,0.83,0.5,0.85)
       subwid = max([4*(rederr1+rederr2),0.9e-3])
       newbox([redshift-subwid,redshift+subwid],[-9.0,0.5],ch=1.6,era=0,manvp=1)
       ylabel("\\fiC\uw\d(z)-\\fiC\uw\d(z\dpeak\u)",ch=1.6)
       xlabel("\\fiz",ch=1.6)
       xz = arange(redshift-subwid,redshift+1.2*subwid,rederr1/20)
       drawln(xz,splev(xz,tspl)-ccpeak,ls=1,hist=0)
       drawpt(ztests,cc-ccpeak,ps=17,ch=2)
       drawln([xz[0],xz[-1]],[ 0.0, 0.0],ls=4)
       drawln([xz[0],xz[-1]],[-0.5,-0.5],ls=4)
       drawln([xz[0],xz[-1]],[-2.0,-2.0],ls=4)
       drawln([xz[0],xz[-1]],[-8.0,-8.0],ls=4)

       if 0:
           cpgsvp(0.1,0.9,0.15,0.9)
           dz2 = (rederr1+rederr2)/2.0
           xz = arange(ztests[0],ztests[-1]+dz2,dz2,Float32)
           schi2 = chi2*ndeg
           print schi2, extrema(schi2-2*(cc-ccpeak))

           zprobs = chdtrc(ndeg,schi2-2*(cc-ccpeak))
           probs = chdtrc(ndeg,schi2-2*(splev(xz,tspl)-ccpeak))

           pnorm = sum(dz2*probs)
           if pnorm:
             probs = probs/pnorm
             zprobs = zprobs/pnorm
           newbox([ztests[0],ztests[-1]],[0.00,1.05*max(dz2*zprobs)],ch=2,ylog=0,manvp=1)
           xlabel("\\fiz",ch=2)
           ylabel("P(\\fiz\\fr)d\\fiz",ch=2)
           xz2 = concatenate([xz,ztests])
           pz2 = concatenate([probs,zprobs])
           xz2,pz2 = take([xz2,pz2],argsort(xz2),1)
           drawln(xz2,dz2*pz2,ls=1,hist=1)

           sz2 = Set(xz2)
           ixz = sort([xz1 for xz1 in sz2])
           ipz = array([compress(equal(xz2,xz1),pz2)[0] for xz1 in ixz])
           pspl = splrep(ixz,ipz,s=0,k=3)

           ipz = array([splint(0,xz1,pspl) for xz1 in ixz])

           if redshift > guess: cpgsvp(0.17,0.35,0.5,0.85)
           else: cpgsvp(0.65,0.83,0.5,0.85)
           subwid = max([4*(rederr1+rederr2),0.9e-3])

           newbox([redshift-subwid,redshift+subwid],[0.00,1.05*max(dz2*zprobs)],ch=1.6,era=0,manvp=1,rax=0)
           xlabel("\\fiz",ch=1.6)
           ylabel("P(\\fiz\\fr)d\\fiz",ch=2)
           drawln(xz,dz2*probs,ls=1)

           newbox([redshift-subwid,redshift+subwid],[0.00,1.05*max(ipz)],ch=1.6,era=0,manvp=1,lax=0,rax=1)
           xlabel("\\fiz",ch=1.6)
           rlabel("\(2268)P(\\fiz\\fr)d\\fiz",ch=2)
           drawln(ixz,ipz,ls=2)

           drawln([redshift,redshift],[0.0,1.05*max(ipz)],ls=1)

           ispl = splrep(ixz,ipz,s=0,k=3)

           iconf1,iconf2 = [0.16,0.84]
           conf1 = fsolve(lambda x: splev(x,ispl)-iconf1,redshift)
           conf2 = fsolve(lambda x: splev(x,ispl)-iconf2,redshift)
           drawln([conf1,conf1],[0.0,1.05*max(ipz)],ls=2)
           drawln([conf2,conf2],[0.0,1.05*max(ipz)],ls=2)
           print redshift-conf1, conf2-redshift
           print rederr1, rederr2

           iconf1,iconf2 = [0.025,0.975]
           conf1 = fsolve(lambda x: splev(x,ispl)-iconf1,redshift)
           conf2 = fsolve(lambda x: splev(x,ispl)-iconf2,redshift)
           drawln([conf1,conf1],[0.0,1.05*max(ipz)],ls=4)
           drawln([conf2,conf2],[0.0,1.05*max(ipz)],ls=4)

    nwt = nonzero(wt)
    if dps:
       closedev()
       pgdev = replace(inspec,".fits","_cc2.ps/vcps")
       opendev(pgdev,land=0,size=pgsize,asp=1.0/3,nx=1,ny=1)
    cpgsvp(0.1,0.9,0.15,0.9)
    xmar = [w1,w2]
#    xmar = [gwave[nwt[0]],gwave[nwt[-1]]]
    newbox(xmar,[-1.0,1.0],ch=2,manvp=1)
    xlabel("Observed Wavelength (\A)",ch=2)
    ylabel("Normalized",ch=2)
    if dump:
        qdump("wt.fits",divz(nspec,gcont)*greater(wt,0),inspec)
        qdump("crspec.fits",crspec*greater(wt,0),inspec)
        qdump("cgspec.fits",cgspec*greater(wt,0),inspec)
        qdump("resids.fits",(cgspec-crspec)*greater(wt,0),inspec)
        qdump("chi.fits",divz(cgspec-crspec,divz(nspec,gcont))*greater(wt,0),inspec)
    drawln(gwave,cgspec*guse,ls=1)
    drawln(gwave,divz(nspec,gcont,out=Float64),ci=4,ls=1)
    drawln(gwave,crspec,ci=2,ls=1)
    if not just1:
       print "Results for",inspec
       print "Using template",template
       print "Peak Value",ccpeak
       print "Redshift Peak = %10.8f" % (redshift)
       print "               -%10.8f" % (rederr1)
       print "               +%10.8f" % (rederr2)

    if dps:
       closedev()
       pgdev = replace(inspec,".fits","_cc3.ps/vcps")
       opendev(pgdev,land=0,size=pgsize,asp=1.0/3,nx=1,ny=1)
    rspec = (crspec+1)*rcont
    ymar = [0.0,getpct(gspec,97)*1.5]
    #if efit: xmar = [gwave[0],gwave[-1]]
    #else: xmar = [gwave[nwt[0]],gwave[nwt[-1]]]
    xmar = [w1,w2]
#    xmar = [gwave[nwt[0]],gwave[nwt[-1]]]
    cpgsvp(0.1,0.9,0.15,0.9)
    newbox( xmar,ymar,ch=2,manvp=1)
    xlabel("Observed Wavelength (\A)",ch=2)
    ylabel("Counts",ch=2)
    mgspec = 1*gspec
    if umask != []:
      for um in umask:
         gbad = between(gwave,um[0],um[1],inc=1)
         mgspec = where(gbad,gcont,mgspec)
    drawln(gwave,mgspec,ls=1)
    drawln(gwave[nwt[0]:nwt[-1]],gcont[nwt[0]:nwt[-1]],ls=1,ci=4)
    drawln(gwave[nwt[0]:nwt[-1]],((crspec+1)*gcont+mean(compress(vspec,gcont))/4.0)[nwt[0]:nwt[-1]],ci=2,ls=1)
    drawln(gwave[nwt[0]:nwt[-1]],(rcont*mean(compress(vspec,gcont))/mean(compress(vspec,rcont)))[nwt[0]:nwt[-1]],ls=1,ci=3)
    #drawln(gwave,rspec,ls=1)
    #drawln(gwave,rcont,ls=1,ci=4)

    matched = asarray([ gwave[nwt[0]:nwt[-1]], mgspec[nwt[0]:nwt[-1]], gcont[nwt[0]:nwt[-1]], \
            ((crspec+1)*gcont)[nwt[0]:nwt[-1]], \
            (rcont*mean(compress(vspec,gcont))/mean(compress(vspec,rcont)))[nwt[0]:nwt[-1]] ])

    del tspl,espl

    if dump: qdump("crap.fits",[(crspec+1)*gcont,gspec])

    print
    wtmn = 0.0
    wterr = 0.0
    wtme = 0.0
    nel = 0
    lnhd = []

    if efit:
       gtck = splrep(gwave,gspec,s=0)
       trydirs=split(os.environ['PYTHONPATH'],":")
       wavdir=nonzero(map(lambda d: "emission.dat" in os.listdir(d), trydirs))
       if len(wavdir) > 0: wavefile=trydirs[wavdir[0]]+"/emission.dat"
       galaxylines = readtable(wavefile,tcols=[2],icols=[1])
       restlines = asarray([galaxylines[0][j] for j in nonzero(equal(galaxylines[1],1))])
       linenames = [galaxylines[2][j] for j in nonzero(equal(galaxylines[1],1))]
       gbad = between(gwave,6830,6970)+ between(gwave,7585,7665)
       for um in umask: gbad = gbad + between(gwave,um[0],um[1],inc=1)
       llocs = searchsorted(gwave,restlines*(1.0+redshift))
       ga = arange(gwave.shape[0])
       for ll in llocs:
          if ll and ll < len(gwave)-1: gbad = gbad + between(ga,ll-5,ll+5)
       gokay = equal(gbad,0)
       useg = divz(gokay,nspec,out=Float64)
       sgspec,sgcont = iterclip(gspec,useg,efilt,th=efind,iter=citer)
       if dump: qdump("gbad.fits",[1.0*gbad,gokay])
       print "Attempting to calculate redshift based on known emission lines..."
       lineonly=gspec-sgcont
       linewidth=2*efilt
       calcz=[];zerr=[];fline=[];gfits=[];trs=[]
       sigs=[];serr=[]
       if umask != []:
          for um in umask:
             gbad = between(gwave,um[0],um[1],inc=1)
             lineonly = lineonly * equal(gbad,0)
       if dump: qdump("linesonly.fits",[gspec,gspec*gokay,sgcont,lineonly],inspec)
       for j in range(len(restlines)):
          if restlines[j] in [3726.2,3728.9]: restlines[j] = mean([3726.2,3728.9])
          predwave=restlines[j]*(redshift+1.0)
          if gwave[0] < predwave < gwave[-1]:
             lowindex=searchsorted(gwave,predwave-linewidth/2.0)
             highindex=searchsorted(gwave,predwave+linewidth/2.0)+1
             te=sum(lineonly[lowindex:highindex])
             tv=getsqrt(sum(vspec_orig[lowindex:highindex]))
             tr=(divz(te,tv))
             if tr > trlim/2:
                fitted=fitgaussian(gwave[lowindex:highindex],lineonly[lowindex:highindex],vspec_orig[lowindex:highindex],pos=predwave)
                lowindex=searchsorted(gwave,predwave-fitted[2])
                highindex=searchsorted(gwave,predwave+fitted[2])+1
                te=sum(lineonly[lowindex:highindex])
                tv=getsqrt(sum(vspec_orig[lowindex:highindex]))
                tr=(divz(te,tv))
                if tr > trlim:
                    calcz.append(divz(fitted[0],restlines[j])-1)
                    zerr.append(divz(fitted[1],restlines[j]))
                    sigs.append(fitted[2])
                    serr.append(fitted[3])
                    fline.append(j)
                    gfits.append(fitted)
                    lowy=splev(fitted[0],gtck)
#                    drawln([fitted[0],fitted[0]],[lowy+0.1*diff(ymar)[0],ymar[1]],ls=1,ci=7)
                    trs.append(tr)

       del gtck

       if len(calcz):
          print "Found %i emission lines...." % (len(calcz))
          print rjust("ID",13), center("Rest",10), center("Observed",16),
          print center("Sigma",13), center("z",11), center("z_err",11),
          print rjust("signif",10)
          for lz in fline:
              print " %12s"  % linenames[lz],
              print "%9.2f  "  % restlines[lz],
              ulz = fline.index(lz)
              print "%9.2f+-%4.2f" % (gfits[ulz][0],gfits[ulz][1]),
              print "%6.2f+-%4.2f" % (gfits[ulz][2],gfits[ulz][3]),
              print "  %.8f" % (calcz[ulz]),
              print "  %.8f" % (zerr[ulz]),
              print "  %6.2f" % (trs[ulz])

          calcz,zerr = asarray( [calcz,zerr])
          ocalcz,ozerr = asarray( [calcz,zerr])
          trs,sigs,serr = asarray( [trs,sigs,serr])
          otrs,osigs,oserr = asarray( [trs,sigs,serr])
          ofline = 1*fline
          ogfits = copy.copy(gfits)
          if len(calcz) > 3:
             dbz = 100
             bz = 0
             bs = 0
    #         wsig = sum(divz(sigs/serr,serr))/sum(divz(1,serr))
             mzerr1,mzerr2 = bwt(log10(zerr))
             print mzerr1,mzerr2
             mzerr = power(10,mzerr1+5*mzerr2)
             bz1,bs = bwt(calcz)
             while dbz > 0.5*bs:
                dbz = abs(bz1 - bz)
                bz = bz1
                keep = ones(ocalcz.shape)
                if bz1 < 1:
                   keep = keep*between(ocalcz,bz-5*ozerr,bz+5*ozerr)
                keep = keep*less(ozerr,mzerr)
                keep = keep*less(oserr,osigs/5)
                keep = keep*greater(otrs,efind)
                calcz,zerr = compress(keep,[ocalcz,ozerr],1)
                trs,sigs,serr = compress(keep,[otrs,osigs,oserr],1)
                fline = compress(keep,ofline).tolist()
                gfits = [ogfits[j] for j in nonzero(keep)]
                print "Keeping %i emission lines...." % (len(calcz))
                bz1 = sum(divz(calcz,zerr))/sum(divz(1,zerr))
                bs = sqrt(sum(divz(power(calcz-bz1,2),zerr))/sum(divz(1,zerr)))
          print rjust("ID",13), center("Rest",10), center("Observed",16),
          print center("Sigma",13), center("z",11), center("z_err",11),
          print rjust("signif",10)
          lnhd = []
          for lz in fline:
              print " %12s"  % linenames[lz],
              print "%9.2f  "  % restlines[lz],
              ulz = fline.index(lz)
              print "%9.2f+-%4.2f" % (gfits[ulz][0],gfits[ulz][1]),
              print "%6.2f+-%4.2f" % (gfits[ulz][2],gfits[ulz][3]),
              print "  %.8f" % (calcz[ulz]),
              print "  %.8f" % (zerr[ulz]),
              print "  %6.2f" % (trs[ulz])
              lnh1 = [linenames[lz],str(restlines[lz]),"%.2f" % (trs[ulz])]
              lnhd.append(join(lnh1,"/"))
          print
          print
          stmn = mean(calcz)
          if len(calcz) > 1:
              wtmn = sum(divz(calcz,zerr))/sum(divz(1,zerr))
              wtme = sum(divz(zerr,zerr))/sum(divz(1,zerr))
              wtme = wtme/getsqrt(len(calcz))
              wtrms = sqrt(sum(divz(power(calcz-wtmn,2),zerr))/sum(divz(1,zerr)))
              wterr = max([wtme/getsqrt(len(calcz)), wtrms/getsqrt(len(calcz))])
          else:
              wtmn = calcz[0]
              wtme = zerr[0]
              wtrms = zerr[0]
              wterr = zerr[0]
          print "Mean = %.8f   Weighted Mean = %.8f  Mean Error = %.8f Error in the Mean = %.8f" % (stmn,wtmn,wtme,wterr)
          if len(calcz) > 2:
             bz,bs = bwt(calcz)
             print "Biweight location/scale estimator= %.8f %.8f" % tuple(bwt(calcz))

       else: print "no useful lines."
    print

    closedev(1)
    if prompt: raw_input("hit <cr> ")
    print

    return {"redshift":redshift,"credshift":credshift,"rederr1":abs(rederr1),"rederr2":rederr2,
             "ccpeak":ccpeak,"chi2":chi2,"wtmn":wtmn,"wterr":wterr,"wtme":wtme,"lnhd":lnhd,
             "cc": asarray([ztests,cc]),"matched":matched}


