
#import psyco
#psyco.full()

import MA
from glob import glob
from FITS import FITS
from scipy.special import jn,jv
from Numeric import *
from bisect import bisect
from pgplot import *
#from scipy.optimize import leastsq, fsolve
from Multipack import leastsq, fsolve
from scipy.integrate import quad
from LinearAlgebra import *
import MLab
import FFT
from umath import *
from string import *
from random import *
import ranlib
from RandomArray import *
from sys import exit, argv
import os
import re
import time

if os.environ.has_key("SEEDS"):
  seed1,seed2 = tuple(eval(os.environ["SEEDS"]))
  seed(x=seed1,y=seed2)

GRAV=6.672e-8
ERGPEV=1.602177E-12
PLANCK=6.6260755e-27
CLIGHT=2.99792458e10
CLIGHTK=2.99792458e5
H0 = 100*1e5/3.09e24
SECPYR = 3.15576E7
CMPMPC = 3.09e24
CMPKPC = 3.09e21
CMPPC = 3.09e18
MPPC = CMPPC/100
MPKPC = MPPC*1e3
MPMPC = MPPC*1e6
KMPPC = CMPPC/1e5
KMPKPC = KMPPC*1e3
KMPMPC = KMPPC*1e6
RSUN = 6.9599e10
MSUN = 1.989e33
LSUN = 3.845e33

MEARTH = 6e27

AU = 1.4957E13

SUN_BV = 0.68
SUN_UB = 0.17
SUN_VMAG = 4.82
SUN_gAB = 5.15
SUN_rAB = 4.67
SUN_iAB = 4.48

ONEK = 2048
TWOK = 2048
FOURK = 4096
ONEK2 = (ONEK,ONEK)
TWOK2 = (TWOK,TWOK)
FOURK2 = (FOURK,FOURK)

MMPINCH = 25.4

shifted_digits="!@#$%^&*()"

def newrdarg(argv,key,conv=None,default=None,single=0):
   retval = default
   if key in argv:
     if single:
        j = argv.index(key)
        retval = not default
        del argv[j]
     else:
       vals = []
       while argv.count(key):
          j = argv.index(key)
          if conv: vals.append(apply(conv,(argv[j+1],)))
          else: vals.append(argv[j+1])
          del argv[j:j+2]
       retval = vals
       if len(retval) == 1: retval = vals[0]
   return argv,retval

def rdarg(argv,key,conv=None,default=None,single=0):
   val = default
   if key in argv:
    if not single:
      val=argv[argv.index(key)+1]
      del argv[argv.index(key):argv.index(key)+2]
      if conv: val=map(conv,[val])[0]
    else:
      del argv[argv.index(key):argv.index(key)+1]
      val = 1
      if default == 1: val=0
   else:
      if conv and val: val=apply(conv,(val,))
   return argv,val

def oprl(f):
  return open(f,"r").readlines()

def polynom(x,nl):
 x = asarray(x,Float64)
 l = (nl+1,)+x.shape
 p = zeros(l,Float64)
 p[0] = ones(x.shape,Float32)
 if nl > 0:
   p[1] = x
   for j in range(1,nl): p[j+1] = x*p[j]
 return p

def bessels(r,nl):
 b=[]
 for j in range(nl+1): b+= [jn(j,r)]
 return asarray(b)

def legendre(x,nl):
 x = asarray(x,Float64)
 l = (nl+1,)+x.shape
 p = zeros(l,Float64)
 p[0] = ones(x.shape,Float64)
 if nl > 0: p[1] = x
 if nl > 1:
#    twox = 2*x
    for j in range(1,nl):
        p[j+1] = ((2*j+1)*x*p[j] - j*p[j-1])/(j+1)
#       p[j+1] = twox*p[j] - p[j-1] - (x*p[j]-p[j-1])/(j+1)
 return p

def hermite(x,nl,norm=0):
 x = asarray(x)
 hn = zeros((nl+1,)+x.shape,Float64)
 hn[0] = ones(x.shape,Float64)
 if nl > 0:
    twx = 2*x
    hn[1] = twx
 if nl > 1:
   for n in range(1,nl):
     hn[n+1] = twx*hn[n] - 2*n*hn[n-1]
 if norm:
   hnorm = lambda n: pow(2,n)*sqrt(pi)*fact(n)
   if hn.shape == (1,):
      hn = hn/hnorm(0)
   else:
      if len(hn.shape) == 1:
         hn = asarray([divide(hn[j],hnorm(j)) for j in range(hn.shape[0])])
      else:
         [divide(hn[j],hnorm(j),hn[j]) for j in range(hn.shape[0])]
 return hn

def chebyshev(x,nl):
 x = asarray(x)
 tn = [ones(x.shape,Float64)]
 if nl > 0:
    tn .append( x.astype(Float64))
 if nl > 1:
   twx = 2*x
   for j in range(1,nl):
     tn.append(twx*tn[j] - tn[j-1])
 return asarray(tn)

def harmonics(x,nharm):
  hbase=zeros((2*nharm+1,)+x.shape,Float64)
  hbase[0] = ones(x.shape,Float64)
  for i in range(1,nharm+1):
    amp = 1.0
    hbase[2*i-1] = amp*cos(2*pi*i*x/len(x))
    hbase[2*i] = amp*sin(2*pi*i*x/len(x))
  return hbase

def getdata(input,col):
	f=open(input,'r')
	l=f.readlines()
	f.close()
	x=[]
	y=[]
	for i in l:
	 if i[0] != '#':
	  k=split(i)
	  x.append(float(k[col]))
	return array(x,'f')

def isanum(c):
    tc=add.reduce(map(lambda c: upper(c) not in digits+".-+E|",c)) >0
    return tc

def readtable(input,tcols=[],icols=[],start=0,finish=0,repl=-99.999,header=-1,format=None,lists=0,splitchar=None):
    F=open(input)
    L=F.readlines()
    F.close()
    if header>=0: K=split(L[header].replace("\n","").replace("#","").replace("'",""),splitchar)
    else: K=None
    if header == start: start+=1
    if finish!=0: L=L[:finish]
    if start>0: L=L[start:]
    if format:
       def rf(l): return map(lambda c: l[c[0]:c[1]], format)
       U=map(rf,L)
    else:
       U=[split(l,splitchar) for l in L]
    if K:
       G=nonzero(map(lambda l: (l[0][0] != "#" and len(l) == len(K)), U))
    else:
       G=nonzero(map(lambda l: l[0][0] != "#", U))
    U=map(lambda g: U[g], G)
    def P(l,c,k=None):
      if c in tcols or (k and k in tcols) or lists: return l
      else: return asarray(l)
    def Q(s,c,k=None):
      if c in tcols or (k and k in tcols): return s.strip()
      elif c in icols: return int(s)
      else: 
        try: return float(s)
        except: return repl
    if U != []:
        if K:
           D=map(lambda C,Ck: P(map(lambda R: Q(R[C],C,Ck), U),C,Ck), range(len(U[0])), K)
           T={}
           for j in range(len(D)): T[K[j]] = D[j]
        else:
           T=map(lambda C: P(map(lambda R: Q(R[C],C), U),C), range(len(U[0])))
        return T

def writetable(output,d):
   f=open(output,"w")
   h=join(["#"]+d.keys())
   print >>f, h
   x1 = d[d.keys()[0]]
   for j in range(len(x1)):
      print >>f, join([str(d[k][j]) for k in d.keys()])
   f.close()

def readtables(inputs): return map(readtable,inputs)

def readgrid(input,tcols=[],icols=[],start=0,finish=-1):
	t0=os.times()[0]
	f=open(input,'r')
	data=f.readlines()
	f.close()

	ending = -1
	if finish >= 0:
	    dg = data[start:ending]
	else:
	    dg = data[start:]
        ending = len(dg)
	ml = map(lambda l: len(l)*(l[0] != "#"), dg)
	template = argmax(ml)+start

	nc=len(split(data[template]))

	grid=[]
	for i in range(0,nc):
	  coldat=[]
	  for d in dg:
	   if d[0] != '#' and len(d) > 1:
	    v = split(d)[i]
	    if i not in tcols:
	     if isanum(v) == 0:
	      if i in icols: v=int(v)
	      else: v=float(v)
	     elif upper(v) == 'INDEF': v=-99.999
#             elif i == 1: print v,isanum(v),transpose(v)
	    coldat.append(v)
	  grid.append(coldat)
	t1=os.times()[0]

	return (grid,t1-t0)

def mod2mpc(mod):
	return pow(10,0.2*mod-5)

def mpc2mod(mpc):
	return 5*log10(mpc*1e6)-5

def sec2cm(z,h,om,tol=1E-6):
#	har=2*(2-om+om*z-(2-om)*sqrt(1+om*z))/(om**2*(1+z))
	har=Har(z,h,om,tol)
	h0=h*100*1e5/3.09e24
	f0=(1+z)/har
	return (pi/180/3600)*CLIGHT/h0/f0

def cosmomod(z,h=0.7,om=[0.3,0.7],tol=1E-8):
    nz = len(shape(z))
    if nz:
       har=asarray([Har(a,h,om,tol) for a in z])
       z = asarray(z)
    else: har=Har(z,h,om,tol)
    return 25.0 + 5.0*log10(3000*(1+z)*har)-5*log10(h)

def newdivz(x,y=1,repl=0.0):
   if type(x) in [ArrayType,ListType] and type(y) in [ArrayType,ListType]:
      u = divide(x,add(y,equal(y,0)))
      return where(y*ones(u.shape),u,repl)
   elif type(y) in [ArrayType,ListType]:
      return where(y,divide(x,add(y,equal(y,0))),repl)
   else:
      if y != 0: return divide(x,y)
      else: return 0.0

def divz(x,y=1,repl=0.0,out=Float32):
   if len(shape(y)) or len(shape(x)):
      if len(shape(y)): bad = equal(y,0.0)
      else: bad = ones(x.shape)*(y==0.0)
      not_bad = 1-bad
      numer = (x*not_bad)
      denom = (y+bad)
      a = (repl*bad).astype(out)
      b = (numer/denom).astype(out)
      return (a + b).astype(out)
   else:
      if y == 0: return repl
      else: return x/y

def absdev( x,y=None):
  x = asarray(x)
  if y == None: y=median(x)
  m=median(abs(x-y))
  if len(m) > 0: m=m.astype(Float32)
  return m

def mean( x,axis=0):
  if len(x): return MLab.mean(asarray(x),axis)
  else: return 0.0

def average( x,axis=0):
  return mean(x,axis=axis)

def rms( x,y=None):
  if len(x) > 2:
    if y == None: y=average(x)
    return sqrt(mean(power(subtract(x,y),2)))
  else: return 0.0

def middle(x):
   if len(x.shape)> 1: x=sort(x,0)
   else: x=sort(x)
   return (x[len(x)/4]+ x[3*len(x)/4])/2.0

def statrej(x,q,f,sorted=0):
   if not sorted: x=sort(x,0)
   s = f(x[q:-q]) 
   return s

def pclip(x,q,sorted=0):
   if not sorted:
      if len(x.shape)> 1: x=sort(x,0)
      else: x=sort(x)
   i1=int(q[0]*len(x)/100.0)
   i2=int(q[1]*len(x)/100.0)-1
   return x[i1:i2+1]

def getrej(x,q):
   if len(x.shape)> 1: x=sort(x,0)
   else: x=sort(x)
   i1=int(q[0]*len(x)/100.0)
   i2=int(q[1]*len(x)/100.0)-1
   x= x[i1:i2+1]
   n=len(x)*q[2]/100
   if n%1: return (x[n-1] + x[n])/2.0
   else: return x[n]

def getpct(x,q,axis=0):
   x=asarray(x)
   if len(x.shape)> 1: x=sort(x,axis)
   else: x=sort(x)
   j = min([len(x)-1,int(0.01*q*len(x))])
   if len(x) > 2:
      if axis == 0: return x[j]
      else: return x[::,j]
   elif len(x): return sum(x)/(1.0*len(x))
   else: return 0.0

def get25(x):
   if len(x.shape)> 1: x=sort(x,0)
   else: x=sort(x)
   return x[len(x)/4]

def get75(x):
   if len(x.shape)> 1: x=xort(x,0)
   else: x=sort(x)
   return x[3*len(x)/4]

def quartiles(x):
   if len(x.shape)> 1: x=sort(x,0)
   else: x=sort(x)
   return array([x[1*len(x)/4], x[2*len(x)/4], x[3*len(x)/4]])

def median( x, sorted=0, nullval = 0.0, axis=0):
  x = asarray(x)
  n = x.shape[axis]
  median=0.0
  if n:
     if not sorted: x = sort( x,axis)
     if n%2:
        if axis == 0: median = x[(n-1)/2]
        else: median = take(x,[(n-1)/2],axis)
     else:
        median = add.reduce(take(x,[n/2-1,n/2],axis),axis)/2.0
     return median
  else: return nullval

def mad( x, y=0):
  x = asarray(x)
  if y==0: y=median(x)
  return median(abs(subtract(x,y)))

def bwttest( x, iter=3):
 ns = sqrt(len(x))
 M = median(x)
 if len(x) <= 2:
  S = 0*M
 else:
  MAD = median(abs(x-M))
  if not MAD: MAD = rms(x-M)*0.6745
  for i in range(iter):
   u6 = divz(x-M,MAD)/6.0
   omuu62 = power(1-power(u6,2),2)
   g6 = less(abs(u6),1.0)
   M = M + divz(add.reduce(g6*(x-M)*omuu62),add.reduce(g6*omuu62))
   u9 = divz(x-M,MAD)/9.0
   omuu9 = 1-power(u9,2)
   omuu59 = 1-5*power(u9,2)
   g9 = less(abs(u9),1.0)
   S = ns * getsqrt( divz( add.reduce(g9*power(x-M,2)*power(omuu9,4)),\
                        abs(add.reduce(g9*omuu9*omuu59)) ))
   MAD = 0.6745*S
 return array([M,S])

def lsq(x,iter=0):
  return [mean(x),rms(x)]

def mamedian(x):
  if len(x.shape) == 1:
      a = MA.sort(x)
      a = a[:len(a)-sum(MA.getmaskarray(a))]
      return a[len(a)/2]
  else:
    b = array((x.shape[1],),x.typecode())
    for j in range(len(b)):
      a = MA.sort(x[::,j])
      a = a[:len(a)-sum(MA.getmaskarray(a))]
      b[j] = a[len(a)/2]
    return b

def mabwt0( x, iter=3):
 x = MA.asarray(x)
 x = MA.sort(x)
 x = x[:len(x)-sum(MA.getmaskarray(x))]
 ns = sqrt(len(x))
 if len(x)>0: M = mamedian(x)
 else: M=0.0
 if len(x) <= 2:
  S = 0*M
 else:
  MAD = mamedian(abs(x-M))
  if not MAD: MAD = rms(x-M)*0.6745
  for i in range(iter):
   u6 = divz(x-M,MAD)/6.0
   omuu62 = power(1-power(u6,2),2)
   g6 = less(abs(u6),1.0)
   M = M + add.reduce(g6*(x-M)*omuu62)/add.reduce(g6*omuu62)
   u9 = divz(x-M,MAD)/9.0
   omuu9 = 1-power(u9,2)
   omuu59 = 1-5*power(u9,2)
   g9 = less(abs(u9),1.0)
   try: S = ns * sqrt(add.reduce(g9*power(x-M,2)*power(omuu9,4))) / \
                        abs(add.reduce(g9*omuu9*omuu59))
   except: S = 0.0
   MAD = 0.6745*S
 return [M,S]

def mabwt(x, iter=3):
  x = MA.asarray(x)
  if len(x.shape) == 1: return mabwt0(x)
  else:
    x = MA.transpose(x)
    b = MA.transpose(MA.asarray(map(mabwt0,x)))
    return b
#    b = zeros((2,x.shape[1]),x.typecode())
#    for j in range(x.shape[1]): b[::,j] = mabwt0(x[::,j])


def bwt( x, iter=3, sorted=0):
 x=asarray(x)
 ns = sqrt(len(x))
 if len(x)>0: M = median(x,sorted=sorted)
 else: M=0.0
 if len(x) <= 2:
  S = 0*M
 else:
  MAD = median(abs(x-M))
  if not MAD: MAD = rms(x-M)*0.6745
  if MAD:
     for i in range(iter):
        u6 = divz(x-M,MAD)/6.0
        omuu62 = power(1-power(u6,2),2)
        g6 = less(abs(u6),1.0)
        M = M + add.reduce(g6*(x-M)*omuu62)/add.reduce(g6*omuu62)
        u9 = divz(x-M,MAD)/9.0
        omuu9 = 1-power(u9,2)
        omuu59 = 1-5*power(u9,2)
        g9 = less(abs(u9),1.0)
        S = ns * divz(sqrt(add.reduce(g9*power(x-M,2)*power(omuu9,4))), abs(add.reduce(g9*omuu9*omuu59)))
        MAD = 0.6745*S
  else: M,S = mean(x),0.0
 return [M,S]

def garray(s, l):
  rho = random( l)
  r=sqrt(-2*log(1-rho))
  th = pi*random( l)
  return s*r*cos(th)

def quadratic( a,b,c):
	xn = (-b-sqrt(b**2-4*a*c))/(2*a)
	xp = (+b-sqrt(b**2-4*a*c))/(2*a)
	return (xn,xp)

def verbex( command,dummy=None,leave=0,pout=False,wd=None): 
     if wd:
        print "Changing working directory to",wd
        os.chdir(wd)
     if pout:
        pid = os.getppid()
        mach = os.uname()[1].split(".")[0]
        fout = "%s_%d.out" % (mach,pid)
        if not dummy:
           fo = open(fout,"a")
           fo.write(72*"#"+"\n")
           fo.write(command+"\n")
           fo.close()
        command = command+" >> "+fout
     print command
     if not dummy:
         perf=os.system(command)
     if leave: exit()
     elif not dummy: return perf

def getchi(x,ex,y,ey,a,iter=0):
    d = y-a*x
    if add.reduce(ex*ey) == 0: denom = ones(x.shape); noerr=1
    else: denom = a*a*ex*ex+ey*ey; noerr = 0
    w = 1/denom
    for i in range(iter+1):
        o = add.reduce(d*w)/add.reduce(w)
        r = power(d-o,2)
	scat = sqrt(add.reduce(r*w)/add.reduce(w))
	exscat = sqrt(add.reduce(denom*w)/add.reduce(w))
	inscat = 0.0
	if exscat < scat: inscat = sqrt(scat**2-exscat**2)
#        chi2 = add.reduce(r/denom)/(len(r)-2)
        chi2 = add.reduce(r*w)/(len(r)-2)
	if noerr: dit = scat**2
	else: dit = (1+r/scat**2/3)*denom
	w = 1/dit
    return chi2, o, scat, exscat

def oldminchi(x,ex,y,ey,a,da,iter):
   ii=array([-1,0,1])
   a0 = a
   mchi=0.01/(len(x)-2)
   dchi=1e24
   mm = 99
   a0n = a0
   while dchi > mchi:
        s0 = 9e24
        olda = a0
        for i in ii:
          a = a0 + i*da
          chi2,o,sc,ec = getchi(x,ex,y,ey,a,iter)
          if chi2 < s0:  dchi=s0-chi2; s0 = chi2; a0n = a; mm=i
        if mm == 0: da=da/2
        a0 = a0n
   return a0

def planefit( xo, yo, a, iter=5, perp=0, wts=None, verb=0, boot=0, minfunc=bwt, arg=None, sigma=1.):
    def mp(s,x,y,minfunc):
      m=add.reduce(s*x,1)
      if minfunc==bwt:
        u,v = bwt(y-m,iter=iter)
      elif minfunc==rms:
        v = sqrt(mean(power(y-m,2)))
      elif minfunc=="mead":
        v = mean(abs(y-m))
      elif minfunc==mad:
        v = median(abs(y-m))
      elif minfunc==getpct:
        v = getpct(abs(y-m),arg)
      else:
        v = sum(minfunc((y-m)/sigma))
      if perp!=0:
        v = v/sqrt(1+add.reduce(s*s))
      if len(shape(v)) == 0:
         ouv=array(len(y)*[v])
      else:
         ouv=v
      if wts:
         ouv = ouv * wts
      if verb:
        print s,v
      return ouv
    sl=[]
    for j in range(boot+1):
      if j > 0: i=randint(0,len(xo),(len(xo),))
      else: i=arange(len(xo))
      x=take(xo,i); y=take(yo,i)
      ms=leastsq(mp,a,args=(x,y,minfunc),full_output=1,epsfcn=1e-8,factor=0.1)
      if verb: print ms[0]
      ns = ms[0]
      sl.append(1*ms[0])
    sl=asarray(sl)
    ns,serr=bwt(sl)
    sl0 = ns
    return [sl0,serr]

def minbwt( xo, yo, a, iter=5, perp=0, verb=0,boot=0):
    xo,yo = asarray([xo,yo])
    sl=[]
    for j in range(boot+1):
      if j > 0: i=randint(0,len(xo),xo.shape)
      else: i=arange(len(xo))
      x=take(xo,i); y=take(yo,i)
      mp = lambda s,x,y: [bwt(y-s[0]*x,iter=iter)[1]/sqrt(1+(perp!=0)*s[0]**2)]
      ms=leastsq(mp,[a],args=(x,y),full_output=1)
      if verb: print ms
      ns = ms[0]
      sl.append(ns)
      if j == 0: sl0=ns
    sl=asarray(sl)
    ns,serr=bwt(sl)
    return [sl0]+bwt(y-ns*x,iter=iter)+[serr]

def minrms( xo, yo, a, iter=5, perp=0, verb=0,boot=0):
    sl=[]
    for j in range(boot+1):
      if j > 0: i=randint(0,len(xo),xo.shape)
      else: i=arange(len(xo))
      x=take(xo,i); y=take(yo,i)
      ms=leastsq(lambda s: [rms(y-s*x)/sqrt(1+(perp!=0)*s[0]**2)], a)
      if verb: print ms
      ns = ms[0]
      sl.append(ns)
      if j == 0: sl0=ns
    sl=asarray(sl)
    ns,serr=bwt(sl)
    return [sl0]+bwt(y-sl0*x,iter=iter)+[serr]

def minmad( xo, yo, a=-0.5, iter=5, perp=0, verb=0,boot=0):
    sl=[]
    for j in range(boot+1):
      if j > 0: i=randint(0,len(xo),xo.shape)
      else: i=arange(len(xo))
      x=take(xo,i); y=take(yo,i)
      ms=leastsq(lambda s: [median(abs(y-s*x-median(y-s*x)))/sqrt(1+(perp!=0)*s[0]**2)], a)
      if verb: print ms
      ns = ms[0]
      sl.append(ns)
      if j == 0: sl0=ns
    sl=asarray(sl)
    ns,serr=bwt(sl)
    return [sl0]+bwt(y-sl0*x,iter=iter)+[serr]

def bootsample(x): return take(x,randint(0,len(x),x.shape[0]),0)

def bootstats(x,n,f1=bwt,f2=None):
    r = asarray([f1(bootsample(x)) for j in xrange(n)])
    if f2: return f2(r)
    else: return r

def oldminbwt( x, y, a, da, iter=5, perp=0, verb=0):
   ii = array([-1,0,1])
   a0 = a
   a0n = a
   p = 1.0
   nth = 1
   if perp: nth=2
   for jth in range(nth):
      nit = 0
      perp2=(jth==1)
      da = 1.0
      ds = 1000.0
      mm = 1
      while da > 1E-5 and nit < 500:
         nit = nit + 1
         s0=bwt((y-a0*x),iter=iter)[1]
         if perp2: s0=s0/sqrt(1+a0*a0)
         mm = 0
         for i in ii:
           a = a0+i*da
           b,s = bwt((y-a*x),iter=iter)
           if perp2: s=s/sqrt(1+a*a)
           if s < s0:
             if s!= 0: ds=abs(s-s0)/s
             else: ds=abs(s-s0)
             s0=s;a0n=a;mm=i
           if verb: print jth,nit,a0,i,a,s,ds
         if mm == 0: da=da/2
         a0 = a0n
   return [a0]+bwt(y-a0*x,iter=iter)

def fitb( xi, yi, a=0.0, da=0.5, boot=100):
   n =len(xi)
   bf = minbwt(xi,yi,a,5)
   se = 0; ze = 0; ce = 0
   sa = 0.0
   if boot:
     ba=[]
     for b in range(boot):
	print b,
        sel = map(lambda n: choice(range(n)), n*ones((n,)))
        x = take(xi,sel); y = take(yi,sel)
        ba.append(minbwt(x,y,bf[0],0.2*bf[0]))
     ba = transpose(ba)
     se = mad(ba[0])
     ze = mad(ba[1])
     ce = mad(ba[2])
     print
   return bf+[se,ze,ce]

def fit2( xi, exi, yi, eyi, iter=0, a=0, da=0.5, boot=0):
   n = len(xi)
   a = minchi(xi,exi,yi,eyi,a,da,0)
   a = minchi(xi,exi,yi,eyi,a,0.1*a,iter)
   sa = 0.0
   if boot:
     ab=[]
     for b in range(boot):
        sel = map(lambda n: choice(range(n)), n*ones((n,)))
	x = take(xi,sel); ex = take(exi,sel)
	y = take(yi,sel); ey = take(eyi,sel)
	ab.append(minchi(x,ex,y,ey,a,da,iter))
#	print b,
     sa = rms(ab)
   x = xi; ex = exi
   y = yi; ey = eyi
   cm,om,s,e = getchi(x,ex,y,ey,a-sa,iter)
   c0,o0,scat,ec = getchi(x,ex,y,ey,a,iter)
   cp,op,s,e = getchi(x,ex,y,ey,a+sa,iter)

   chi2 = c0
   o = o0
   so = abs(op-om)/2
   if ec < scat: ic = sqrt(scat**2-ec**2)
   else: ic = 0.0

   return chi2, a, sa, o, so, scat, ec, ic

def linfit(x,y,ey=None):
#  y=a+b*x
   if ey: x,y,ey= asarray([x,y,ey])
   else: x,y= asarray([x,y]); ey=ones(y.shape)
   S=sum(1/ey**2)
   Sx=sum(x/ey**2)
   Sy=sum(y/ey**2)
   Sxx=sum(x**2/ey**2)
   Sxy=sum(x*y/ey**2)
   d=S*Sxx-Sx**2
   a=divz(Sxx*Sy-Sx*Sxy,d)
   b=divz(S*Sxy-Sx*Sy,d)
   if len(x) > 2: n2 = (len(x)-2)
   else: n2 = 1.
   r = y-(a+b*x)
   R = sum((r/ey)**2)
   ea=sqrt(R*(Sxx/d))
   eb=sqrt(R*S/d)
   return a,ea,b,eb

def lfac(x):
  if x < 100:
    f = 1.0
    for j in range(x,1,-1): f=f*j
    f = log(f)
  else:
    f = stirling(x)
  return f
     
fact=lambda n:n-1 + abs(n-1) and fact(n-1)*n or 1

def factorial(x):
  if not len(shape(x)):
    f = reduce(lambda x,y: x*y, range(x,0,-1))
  else:
    f=x.astype(Float32)
    while add.reduce(x) > len(x):
      x=(x-1)+less_equal(x,1)
      f=f*x
    f = f + equal(f,0)
  return f
     
def stirling(x):
  return log(2*pi)/2+(x+0.5)*log(x)-x

def trapzd(f,a,b,s,n):
  def func(f,x): return eval(f)
  if n == 1:
    s = 0.5*(b-a)*(func(f,a)+func(f,b))
  else:
    d = (b-a)/float(n)
    x = arange(a,b,d)+d/2
    s = (s+(b-a)*add.reduce(eval(f))/n)/2
  return s

def midpnt(f,a,b,s,n):
  def func(f,x): return eval(f)
  if n == 1:
    s = 0.5*(b-a)*(func(f,a)+func(f,b))
  else:
    d = (b-a)/float(3*n)
    dd = d+d
    sum = 0
    x = a+d/2
    for j in range(n):
      sum = sum + func(f,x)
      x = x + dd
      sum = sum + func(f,x)
      x = x + d
    s = (s+(b-a)*add.reduce(sum)/n)/3
  return s

def integrate(f, a, b, tol=1E-6):
  s = 0
  for n in range(1,50000):
    os = 1.*s
    s = (4.*trapzd(f,1.*a,1.*b,s,n)-os)/3
    if abs(s-os) < tol* abs(os): break
  return s

def zlook(t,h=0.7,omega=[0.3,0.7],guess=0.1,tol=1E-8):
  def resid(z,t,h,omega,tol):
     tg = tlook(z[0],h,omega,tol)
     return [tg-t]
  results = leastsq( resid, [guess], args=(t,h,omega,tol),xtol=tol,ftol=tol)
  return results[0]

def tlook(z,h=0.7,omega=[0.3,0.7],tol=1E-6):
  om=1.*omega[0]
  ol=1.*omega[1]
  os=1.-om-ol
  k=-os
  def infun(x,om,ol,os):
       return 1./((x+1)*sqrt(om*pow(x+1,3)+os*pow(x+1,2)+ol))
  nz = len(shape(z))
  if not nz:
     result=quad( infun,0.0,z,args=(om,ol,os),epsabs=tol,full_output=1)
     y = result[0]
  else:
    y=[]
    for r in z:
        y.append( quad( infun,0.0,r,args=(om,ol,os),epsabs=tol)[0])
    y = array(y)
  return y/(H0*h)/SECPYR/1e9

def har_infun(x,om,ol,k): return 1/(x*sqrt(om/x-k+ol*x*x))

def Har(z,h,omega,tol=1E-6):
  om,ol=omega
  os=1.-om-ol
  k=-os
  d=CLIGHT/(H0*h)/CMPMPC
  y = quad( har_infun,1./(z+1.),1.,args=(om,ol,k),epsrel=tol,epsabs=tol,limit=10000)[0]
  if k == 0: har=y
  elif k > 0: har=(1/sqrt(k))*sin(sqrt(k)*y)
  else: har=(1/sqrt(-k))*sinh(sqrt(-k)*y)
  return har

def angsize(z=1.0,h=0.7,omega=[0.3,0.7],units="kpc",tol=1E-6):
  if upper(units)=="MPC": dun=CMPMPC
  if upper(units)=="KPC": dun=CMPKPC
  if upper(units)=="PC": dun=CMPPC
  if len(shape(z)) == 0: zs=[z]
  else: zs=asarray(z)
  ang = []
  for zz in zs:
    f0 = (1+zz)/Har(zz,h,omega,tol)
    ang += [(pi/180/3600)*CLIGHT/(H0*h)/f0/dun]
  ang=asarray(ang)
  if len(shape(z)) == 0: ang=ang[0]
  return ang

def mkarr( x,kind):
  if find(str(type(x)),'array')+1: x=x.astype(kind)
  if find(str(type(x)),'list')+1: x=array(x,kind)
  return x

def genpt( a, kind='float'):
  if upper(kind) == 'INT': recast,ty=Int32,'int'
  elif upper(kind) == 'FLOAT': recast,ty=Float32,'float'
  elif upper(kind) in ['STRING','CHAR']: p,ty=a,'string'
  else: p='YOU DID SOMETHING IDIOTIC'

  if ty in ['float','int']:
     a=mkarr(a,recast)
     p = '_'+re.sub('0x','',str(hex(a.buffer_info()[0])))+'_'+ty+'_p'
  return a,p

def make_normal_eq( A, Y):
   N=A.shape[1]
   M=A.shape[0]

   T = transpose(A)
   XX = dot(T,A)
   XY = add.reduce(T*Y,1)

   return XX, XY

def isnan(x): return strip(str(x)) == 'NaN'

def getsqrt(x,rep=0):
    y = less_equal(x,0)
    z = multiply(x,logical_not(y))
    return sqrt(z)+rep*y

def getlog10(x,rep=-99):
    y = greater(x,0)
    z = where(y,x,1)
    return where(y,log10(z),rep)

def getexp(x):
    y = greater(x,-200)
    return exp(y*x)*y

def getlog(x,rep=-99):
    y = less_equal(x,0)
    z = multiply(x,logical_not(y))
    return log(z+y)+rep*y

def interparray(x,x0,f0):
  u0 = clip(searchsorted(x0,x),0,x0.shape[0]-1)
  u1 = clip(u0 + 1,0,x0.shape[0]-1)
  a,fa = take([x0,f0],u0,1)
  b,fb = take([x0,f0],u1,1)
  dx = x - a
  e = equal(b,a)
  f = where(e,fa, fa + (x-a) * (fb-fa) / (b-a))
  return f

def simpinterp(x,r,d,rev=0,ulog=1,sorted=0):
   if not sorted:
      d,r=take(d,argsort(r)),sort(r)
#   if rev: d,r=take(d,argsort(r)),sort(r)
   if x < r[-2] and x > r[0]:
      i = bisect(r, x)
      r0 = r[i-1]; r1 = r[i]
      l0 = d[i-1]; l1 = d[i]
      if ulog:
        if x and r0 and r1 and l0 and l1:
            try: onu = pow(10,log10(l0) + log10(x/r0)*log10(l1/l0)/log10(r1/r0))
            except: onu = l0 + (x-r0)*(l1-l0)/(r1-r0)
        else: onu = l0 + (x-r0)*(l1-l0)/(r1-r0)
      else:
        onu = l0 + (x-r0)*(l1-l0)/(r1-r0)
   else: onu = 0.0
   return onu

def bwtfilt(d,boxx,boxy,circle=0,verb=0,ring=None,worksize=0.5):
    def bwt0(x): return bwt(x)[0]
    return funcfilt(bwt0,d,boxx,boxy,circle=circle,ring=ring,verb=verb,worksize=worksize)

def medfilt(d,boxx,boxy,circle=0,ring=0,verb=0,worksize=0.5):
    return funcfilt(median,d,boxx,boxy,circle=circle,ring=ring,verb=verb,worksize=worksize)

def sumfilt(d,boxx,boxy,circle=0,ring=0,verb=0,worksize=0.5):
    return funcfilt(sum,d,boxx,boxy,circle=circle,verb=verb,ring=ring,worksize=worksize)

def meanfilt(d,boxx,boxy,circle=0,ring=0,verb=0,worksize=0.5):
    return funcfilt(mean,d,boxx,boxy,circle=circle,verb=verb,ring=ring,worksize=worksize)

def funcfilt(func,d,boxx,boxy,verb=0,worksize=30,js=None,arg=(),circle=None,ring=None,cut=None):
    if func == quartiles: e = zeros((3,d.shape[0],d.shape[1]),Float32)
    else: e = zeros(d.shape,Float32)
    sh = d.shape
    boxx = int(2*boxx); boxy = int(2*boxy)
    n2 = len(d)
    if len(sh) == 2:
      n1 = sh[1]; n2 = sh[0]; nx = n1; xax = 1
    elif len(sh) == 1:
      n1 = 1; n2 = sh[0]; nx = n2; xax = 0
    if not js:
       if circle:
          area=pi*boxx*boxy/4
          if ring: area=area-pi*(boxx+1-ring)*(boxy+1-ring)/4
       else:
          area = (boxx+1)*(boxy+1)
          if ring: area=area-ring**2
       if xax: sb = (int(area+1.0) * nx)*8
       else: sb = boxx
       js = int(worksize*1E6/(1+sb))
    js=max((1,js))
    if verb: print "Stepping",js
    jss=range(0,n2+js,js)
    time0 = os.times()[0]
    jdi = arange(n2)
    for jb in jss[:-1]:
       js0=jb; js1=min((jss[jss.index(jb)+1],n2))
       if verb: print "--> Doing ",[js0,js1]
       jt = []
       jdu = []
       for jd in range(-boxy/2,boxy/2+1):
          if jd < 0: jtj = arange(jd,jd+n2)
          else: jtj= arange(jd-n2,jd) + n2
          jtj = clip(jtj,0,n2-1)
          jtj = take(jdi,jtj)
          jtu = map(lambda jj: d[jj], jtj[js0:js1])
          jt.append(jtu)
          jdu.append(jd)
       it = []
       for ijd in range(len(jt)):
          if circle: uboxx=int(boxx*sqrt(1.0-jdu[ijd]**2/(boxy**2/4.)))
          else: uboxx=boxx
          if ring: uid = range(-uboxx/2,-uboxx/2+ring)+ range(uboxx/2+1-ring,uboxx/2+1)
          else: uid= range(-uboxx/2,uboxx/2+1)
          for id in uid:
              if id < 0: iji = arange(id,id+nx)
              else: iji = arange(id-nx,id)+nx
              iji = clip(iji,0,nx-1)
              it.append(take(jt[ijd],iji,xax))
       it = asarray(it)
       if arg: locres = func(it,arg)
       else:
         if func==bwt: locres = func(it)[0]
         else: locres = func(it)
       locres = MLab.squeeze(asarray(locres).astype(Float32))
       if func == quartiles: e[::,js0:js1] = locres
       else: e[js0:js1] = locres
       if verb:
         time1=os.times()[0]; dt = (time1-time0)/js1
         tot = n2*dt; rem = tot - (time1-time0)
         print "--> Estimated time/time remaining= %9.3f/%9.3f" % (tot,rem)
    return e

def fourfilt(b,kcuts,t):
#   if len(t) != len(kcuts)-1: return -1
   ny0,nx0 = b.shape
   nx = nx0 + nx0%2; ny = ny0 + ny0%2
   lx = pow(2,int(log(nx)/log(2)+1.0)); ly = pow(2,int(log(ny)/log(2)+1.0))
   x=arange(0,nx/2+1,1,Float32); y=arange(0,ny/2+1,1,Float32)
   u = zeros((ny,nx),b.typecode())
   transpose(u[:ny0])[:nx0] = transpose(b)
   u0 = 1.0*u
   for n in range(len(kcuts)):
     kx,ky = kcuts[n]
     print "--> Using",kx,ky
     if kx<0: fx=greater_equal(x,-kx); fx[0]=1.0
     else: fx=less_equal(x,kx)
     if ky<0: fy=greater_equal(y,-ky); fy[0]=1.0
     else: fy=less_equal(y,ky)
     if kx != 0: s = FFT.inverse_real_fft2d( FFT.real_fft2d(u)*fx)
     if ky != 0: s = transpose(FFT.inverse_real_fft2d( FFT.real_fft2d(transpose(s))*fy))
     d = abs(s-u)/s
     if n < len(kcuts)-1: u = greater(d,t[n])*s + less_equal(d,t[n])*u0
     s=take(take(s,range(0,ny0),0),range(0,nx0),1).astype(Float32)
   return s

def paste(x,m,n=0,c=0):
  if len(x.shape) == 1:
     y=c*ones( (m), x.typecode()).astype(x.typecode())
     y[:x.shape[0]] = x
  elif len(x.shape) == 2:
     y=c*ones( (n,m), x.typecode()).astype(x.typecode())
     transpose(y[:x.shape[0]])[:x.shape[1]] = transpose(x)
  return y

def unpaste(x,m,n): return x[:n,:m]

def shiftim(d,x=0,y=0):
  if len(d.shape) == 2: ny,nx = d.shape; y=int(y)
  else: nx = d.shape[0]
  x=int(x)
  if x > 0: xr=range(-nx+x,x)
  else: xr=range(x,nx+x)
  if len(d.shape) == 2:
     if y > 0: yr=range(-ny+y,y)
     else: yr=range(y,ny+y)
     if y: return take(take(d,xr,1),yr,0)
     else: return take(d,xr,1)
  else: return take(d,xr,0)

def log2(x): return log(x)/log(2)

def conv1d(f,g,shift=None,inverse=None,verb=None):
   if shift and not inverse:
     if verb: print "Shifting."
     g=shiftim(g,g.shape[0]/2)
   if not inverse:
     if verb: print "FFTing."
     result=FFT.inverse_fft(FFT.fft(f)*FFT.fft(g)).real
   else:
     if verb: print "FFTing."
     result=FFT.inverse_fft(FFT.fft(f)/FFT.fft(g)).real
   if shift and inverse:
      if verb: print "Shifting."
      result=shiftim(result,result.shape[0]/2)
   return result

def conv2d(f0,g0,shift=None,inverse=None,verb=None,pad=None):
   fty = f0.typecode()
   if shift and not inverse:
     if verb: print "Shifting."
     g0=shiftim(g0,g0.shape[1]/2,g0.shape[0]/2)
   if pad:
      sf = list(f0.shape); sg = list(g0.shape)
      sf2 = power(2,log2(subtract(sf,1)).astype(Int32)+1)
      sg2 = power(2,log2(subtract(sg,1)).astype(Int32)+1)
      print "Padding",sf,sg,"-->", sf2,sg2
      f=paste(f0,sf2[1],sf2[0]); g=paste(g0,sg2[1],sg2[0])
   else: f,g=f0,g0
   ff = FFT.real_fft2d(f).astype(fty)
   fg = FFT.real_fft2d(g).astype(fty)
   if inverse:
     if verb: print "FFTing."
     result=FFT.inverse_real_fft2d((ff/fg).astype(fty)).astype(fty)
   else:
     if verb: print "FFTing."
     result=FFT.inverse_real_fft2d((ff*fg).astype(fty)).astype(fty)
   if shift and inverse:
      if verb: print "Shifting."
      result=shiftim(result,result.shape[1]/2,result.shape[0]/2)
   return result[:f0.shape[0],:f0.shape[1]]

def gauss(x,s=1.0):
  rho = random( x.shape)
  r=sqrt(-2*log(1-rho))
  th = pi*random( x.shape)
  return s*r*cos(th)

def aint(x):
   if len(shape(x)): return x.astype(Int32)
   else: return int(x)

def gauss_conv1d(x,f=1.0,a=1):
      s = f/2.35
      n2n = 2**(aint(log(subtract(x.shape,1))/log(2)) + a)
      n2f = n2n/2 + 1
      ppp = 0.5/(s)**2
      gga = zeros(n2n,Float64)
      ggw = zeros((n2n[0]),Float32)
      if len(x.shape) == 1:
         gga[0:x.shape[0]] = x.astype(Float32)
         ggfa = FFT.fft( gga)
      elif len(x.shape) == 2:
         gga[0:x.shape[0],0:x.shape[1]] = x.astype(Float32)
         ggfa = FFT.fft2d( gga)
      ggw[0:n2f[0]-1] = (2*pi*arange(0,n2f[0]-1,1,Float32)/float(n2n[0])).astype(Float32)
      ggw[n2f[0]-1:n2n[0]] = (2*pi*arange(n2f[0]-1,0,-1,Float32)/float(n2n[0])).astype(Float32)
      ggk = -ggw**2/(4*ppp)
      ggu = greater(ggk,-50)
      if len(x.shape) == 1:
         ggb = FFT.inverse_fft(ggfa*exp(ggk*ggu)*ggu).real
         return ggb[0:x.shape[0]]
      elif len(x.shape) == 2:
         ggb = FFT.inverse_fft2d(ggfa*exp(ggk*ggu)*ggu).real
         return ggb[0:x.shape[0],0:x.shape[1]]

def gauss_conv2d(x,f=1.0,a=1):
    xtype = x.typecode()
    s= f/2.35
    n2n = 2**(aint(log(subtract(x.shape,1))/log(2)) + a)
    n2f = n2n/2 + 1
    ppp = 0.5/(s)**2
    gga = zeros(n2n,xtype)
    gga[0:x.shape[0],0:x.shape[1]] = x
    ggw0 = zeros((n2n[0]),xtype)
    ggw0[0:n2f[0]-1] = (2*pi*arange(0,n2f[0]-1,1,xtype)/n2n[0]).astype(xtype)
    ggw0[n2f[0]-1:n2n[0]] = (2*pi*arange(n2f[0]-1,0,-1,xtype)/n2n[0]).astype(xtype)
    ggw1 = zeros((n2n[1]),xtype)
    ggw1[0:n2f[1]-1] = (2*pi*arange(0,n2f[1]-1,1,xtype)/n2n[1]).astype(xtype)
    ggw1[n2f[1]-1:n2n[1]] = (2*pi*arange(n2f[1]-1,0,-1,xtype)/n2n[1]).astype(xtype)
    gga = FFT.fft2d( gga)
    gga = gga * exp(-ggw0**2/(4*ppp))
    gga = transpose( transpose(gga) * exp(-ggw1**2/(4*ppp)))
    return FFT.inverse_fft2d( gga).real.astype(xtype)[:x.shape[0],:x.shape[1]]


def boxcar_conv1d(x,f=1.0):
   hwid = int((f-1)/2)
   nn = len(x)
   outd = zeros( (nn), Float32)
   for bi in range(-hwid,hwid+1,1):
       if bi < 1: br = range(bi,nn+bi)
       else: br = range(bi-nn,bi)
       outd = outd + take(x,br)/(2*hwid+1)
   return outd

def qstats(x):
   return asarray([mean(x.flat),rms(x.flat),min(x.flat),max(x.flat),median(x.flat),mad(x.flat)])

def extrema(x):
   return asarray([minimum.reduce(x),maximum.reduce(x)])

def between(x,l,h,inc=0):
   if inc: return greater_equal(x,l)*less_equal(x,h)
   else: return greater(x,l)*less(x,h)

def apoisson(a):
    x=a.flat
    s=map(ranlib.poisson, x)
    return reshape(s,a.shape)

def distance(p1,p2): return sqrt( (p1[0]-p2[0])**2+ (p1[1]-p2[1])**2)

def orl(f,l=0): return open(f,"r").readlines()[l:]

def qtable(f,a):
    g=open(f,"w")
    for l in a:
       if len(shape(l)) == 0: l=[l]
       map(lambda w: g.write(" "+str(w)), l)
       g.write("\n")
    g.close()

def svdfit(b,y):
      decomp = singular_value_decomposition(b)
      sol1 = transpose(decomp[2])
      sol2 = divz(1.0,decomp[1])
      sol3 = dot(transpose(decomp[0]),y)
      if sometrue(sol3):
        solr = (sol2*sol3)
        soll = dot(sol1,solr)
      else:
        soll = zeros(sol3.shape)
      return soll

def pearson(x,y):
    xc = mean(x)
    yc = mean(y)
    n = sum((x-xc)*(y-yc))
    b1 = sum((x-xc)**2)
    b2 = sum((y-yc)**2)
    b = sqrt(b1*b2)
    r = n/b
    return r

def fitgaussian(p,x,y,f=0,v=0,wt=None):
   c,s = p
   s = pow(10,s)
   k = -0.5*power((x-c)/s,2)
   g = getexp(k)
   u = 1.0*greater(y,median(y))*greater(g,1e-4)
   if wt: w = wt
   else: w = divz(u,getsqrt(y))
   a = sum(compress(w,divz(w*y,g)))/sum(compress(w,w))
   m = a*g
   u = 1.0*logical_or(greater(x,c)*greater(y,median(y)),less_equal(x,c))
   u = u * greater(m,0)
   if wt: w = wt
   else: w = divz(u,getsqrt(y))
   if f:
     r = (y-m)*u*w
     if v > 1: print a,c,s,add.reduce(r**2)
     return r
   else: return m

def getgausky(D,dn,verb=0):
   D = sort(D)
   p40 = D[40*len(D)/100+1]
   p10 = D[10*len(D)/100+1]
   se = 1.49*median(abs(D-p40))
#   se = p40-p10
   kD = between(D,p40-4*se,p40+2*se)
   uD = compress(kD,D)
   b = MLab.diff(extrema(uD))[0]/dn
   H = arange(min(uD),max(uD)+b/2.,b,Float)
   dS = MLab.diff(searchsorted(uD,H))
   H = MLab.diff(H)/2. + H[:-1]
   guess = [p40,log10(se)]

   if verb > 1: print "Fitting:",guess
   H,dS = compress(greater(dS,0),[H,dS],1)
   if verb >= 3:
      newbox(extrema(H),[-0.1*max(dS),1.2*max(dS)],ch=2)
      xlabel("H",ch=2)
      ylabel("dS",ch=2)
      drawln(H,dS,hist=1,ls=1)
   gsol,gmsg = leastsq(fitgaussian,guess,args=(H,dS,1,verb),xtol=1e-6,ftol=1e-6,epsfcn=1e-7,factor=10.0,maxfev=5000)
   gsol[1] = 10**gsol[1]
   if verb >= 2: print " %9.1f %9.1f" % tuple(gsol)
   if verb >= 3:
       drawln([gsol[0],gsol[0]],[-max(dS),10*max(dS)],ls=2,ci=7)
       drawln([gsol[0]+gsol[1],gsol[0]+gsol[1]],[-max(dS),10*max(dS)],ls=4,ci=7)
       drawln([gsol[0]-gsol[1],gsol[0]-gsol[1]],[-max(dS),10*max(dS)],ls=4,ci=7)
   return gsol[0]

def GaussianBG(data,dn,verb=0,check=1):
   if check: data = compress(not_equal(data.flat,0),data.flat)
   data = sort(data)
   bg = getgausky(data.flat,dn,verb=verb)
   return bg

def ShrinkN(D,good,mx,my,func=median,nret=0):
   n2,n1 = D.shape
   ny = (n2-1)/my + 0
   nx = (n1-1)/mx + 0
   if nret:
     o = zeros((nret,ny,nx),Float64)
   else:
     o = zeros((ny,nx),Float64)
   for j in range(ny):
     for i in range(nx):
       i0,i1 = clip([i*mx,(i+1)*mx],0,n1)
       j0,j1 = clip([j*my,(j+1)*my],0,n2)
       d = ravel(D[j0:j1,i0:i1])
       u = ravel(good[j0:j1,i0:i1])
       result = func(compress(u,d))
       if nret: o[::,j,i] = result
       else: o[j,i] = result
   return o

def orthofit(fx,y,w=None,verb=0):
     N = len(fx)
     if not w: w = ones(y.shape)
     if N == 1:
        s = svdfit(transpose(fx[0])*w[::,NewAxis],y*w)
        p = dot(s,fx[0])
     elif N == 2:
        s = []
        p = zeros(y.shape,Float64)
        for i in range(len(fx[0])):
           if verb: sys.stdout.write("(%d)" % (i)); sys.stdout.flush()
           b = fx[0][i]*fx[1]
           s1 = svdfit(transpose(b)*w[::,NewAxis],(y-p)*w)
           add(p,dot(s1,b),p)
           s = concatenate([s,s1])
     if verb: print
     return s,p

def GetC(result):
   ipvt = result[1]['ipvt']
   n = len(ipvt)
   fjac = result[1]['fjac']
   perm = take(MLab.eye(n), ipvt -1)
   r = MLab.triu(transpose(fjac)[:n,:])
   R = MLab.dot(r,perm)
   try:
      C = generalized_inverse(MLab.dot(transpose(R),R))
   except:
      C = None
      sys.stderr.write("Error!  Covariance matrix is singular\n")
   return C

def LinearLeastSquares(X,Y,verb=1):
   if verb: print "Setting up the normal equations."
   start = time.time()
   ndata,ndimen = X.shape
   if verb: print "(ndata,ndimen)=(%d,%d)" % (ndata,ndimen)
   if verb: print "Creating XY and XX"
   xy = zeros((ndimen,),Float64)
   xx = zeros((ndimen,ndimen),Float64)
   for i in range(ndimen):
      if verb: sys.stdout.write("(%d)" % (i+1)); sys.stdout.flush()
      xy[i] = sum(X[::,i]*Y)
      for j in range(i,ndimen):
        xx[i,j] = sum(X[::,i]*X[::,j])
        xx[j,i] = xx[i,j]
   sol = svdfit(xx,xy)
   finish = time.time()
   if verb: print "\nTime for solution = %.3fs" % (finish-start)
   return sol

def sglob(g,sort=0):
   f = glob(g)
   if sort: f.sort()
   return f

def CosmicVariance(z,dz,A,r0,g,verb=0):
   S = angsize(z,units="Mpc")*3600.0
   m1 = mod2mpc(cosmomod(add(z,-0.5*dz)))
   m2 = mod2mpc(cosmomod(add(z,+0.5*dz)))
   L = (m2-m1)
   if verb: print "Depth=",L
   V = multiply(L*pow(S,2),A)
   if verb: print "Volume=",V/1e6
   R = pow(3*V/(4*pi),1/3.)
   J2 = 72.0/((3-g)*(4-g)*(6-g)*pow(2,g))
   sv2 = J2 * pow(r0/R,g)
   sv = sqrt(sv2)
   if verb: print "Cosmic Variance=",sv
   return sv

def nonexist(f):
   if type(f) == StringType: return not (os.path.exists(f) and os.stat(f)[6])
   else: return map(nonexist,f)


