#############################################
#
#	This is Dan Kelson's FITSIO
#		Python Module
#
#############################################

import gzip
from math import *
from umath import *
import time
import os
from socket import *
import cPickle
from string import *
import sys
import Numeric

SOCKET="/tmp/SOCKET"
# 1MB buffer for SOCKET
BUFLEN = 1048576

def gethead(fitsfile):

	if fitsfile[-3:] == '.gz': fitsfile=fitsfile[0:-3]
	gzipped = 0
	if not os.path.exists(fitsfile):
	  gzfitsfile = fitsfile+".gz"
	  if not os.path.exists(gzfitsfile):
	   print "File not found: ", fitsfile; sys.exit()
	  else: gzipped = 1; fitsfile=gzfitsfile

	if gzipped: file=gzip.open(fitsfile,'r')
	else: file=open(fitsfile,'r')

	header=[]
	for i in range(0,36000):
	 header.append(str(file.read(80)))
	 if header[i][0:3] == 'END': iend = i*80; break

	file.close()

	return header

def getimport(head):
	bitpix= int(parsehead(head,'bitpix'))
	naxis = int(parsehead(head,'naxis'))
	naxes =[int(parsehead(head,'naxis1'))]
	if naxis > 1: naxes.append(int(parsehead(head,'naxis2')))
	if naxis > 2: naxes.append(int(parsehead(head,'naxis3')))
	bscale= 1.0; bzero= 0.0
	if parsehead(head,'bscale') != 'N/A': bs=float(parsehead(head,'bscale'))
	if parsehead(head,'bzero') != 'N/A': bz=float(parsehead(head,'bzero'))

	return [bitpix, naxis, naxes, bscale, bzero]

def parsehead(header,keyword,full=0):

	value='N/A'
	for h in header:
	   keyw =   ljust(upper(keyword),8)
	   keyw2 =   h[0:8]
	   if keyw == keyw2:
		value=strip(h[10:31])
		if value=="": value=strip(h[10:51])
		if full: value=strip(h[10:80])
		c=add.reduce(map(lambda c: lower(c) not in digits+'.e-', value))
		if c == 0: value=atof(value)
		break

	return value

def readfits(fitsfile):

	if fitsfile[-3:] == '.gz': fitsfile=fitsfile[0:-3]
	gzipped = 0
	if not os.path.exists(fitsfile):
	  gzfitsfile = fitsfile+".gz"
	  if not os.path.exists(gzfitsfile):
	   print "File not found: ", fitsfile; sys.exit()
	  else: gzipped = 1; fitsfile=gzfitsfile

	header=gethead(fitsfile)

	bscale = 1
	bzero = 0
	crval1=1.0; crval2=1.0; crpix1=1.0; crpix2=1.0
	cdelt1=1.0; cdelt2=1.0
	ctype1=''; ctype2=''
	cd1_1=1.0; cd1_2=0.0; cd2_1=0.0; cd2_2=1.0
	dcflag=0

	bitpix=int(parsehead(header,'bitpix'))
	naxis=int(parsehead(header,'naxis'))
	naxes=[int(parsehead(header,'naxis1'))]
	if naxis == 2: naxes.append(int(parsehead(header,'naxis2')))
	if naxis == 3: naxes.append(int(parsehead(header,'naxis3')))

	irafmade = 0
	if parsehead(header,'irafname') != 'N/A': irafmade=1

	if parsehead(header,'crval1') != 'N/A': crval1=float(parsehead(header,'crval1'))
	if parsehead(header,'crpix1') != 'N/A': crpix1=float(parsehead(header,'crpix1'))
	if parsehead(header,'cdelt1') != 'N/A': cdelt1=float(parsehead(header,'cdelt1'))
	if parsehead(header,'cd1_1') != 'N/A': cd1_1=float(parsehead(header,'cd1_1'))
	if parsehead(header,'cd1_2') != 'N/A': cd1_2=float(parsehead(header,'cd1_2'))

	if parsehead(header,'bscale') != 'N/A': bscale=float(parsehead(header,'bscale'))
	if parsehead(header,'bzero') != 'N/A': bzero=float(parsehead(header,'bzero'))
	if parsehead(header,'dc-flag') != 'N/A': dcflag=int(parsehead(header,'dc-flag'))

	coordsys={'type':[ctype1,ctype2], 'cd1':(cd1_1,cd1_2), 'cd2':(cd2_1,cd2_2)}

	iend = len(header)*80
	nrec = iend/2880

	if ( iend/2880.0 > iend/2880): nrec = nrec + 1

	if ( naxis < 3) : naxes.append(1)
	if ( naxis < 2) : naxes.append(1)

	numpix = naxes[0]
	if len(naxes) > 1: numpix = numpix*naxes[1]

	if bitpix == 16: intype=Numeric.Int16
	if bitpix == 32: intype=Numeric.Int32
	if bitpix == -32: intype=Numeric.Float32

	lenstr = numpix*abs(bitpix)/8

	if gzipped: file=gzip.open(fitsfile,'r')
	else: file=open(fitsfile,'r')
	data = file.read()[nrec*2880:nrec*2880+lenstr]
	file.close()

	data = Numeric.fromstring(data,intype)
	if os.uname()[0]=='OSF1' or os.uname()[4]=='i686':
		data = data.byteswapped()

	if bscale != 1.0: data = multiply(data,bscale)
	if bzero != 0.0: data = add(data,bzero)

	data=data.astype(Numeric.Float32)
#	print len(data), lenstr

#	return {'naxis':naxis,'naxes':naxes, 'coords':coordsys,\
	return {'naxis':naxis,'naxes':naxes,
	'crval1':crval1, 'crpix1':crpix1, 'cdelt1':cdelt1, \
	'dcflag':dcflag,\
	'crval2':crval2, 'crpix2':crpix2, 'cdelt2':cdelt2, \
	'input':fitsfile,\
	'bitpix':bitpix,\
	'header':header,\
	'data':data}

def writefits(outfile,output, bitpix=-32):

	header = ""
	reg = ljust(" ",20)
	blank = ljust(" ",80)
	ender = ljust("END",80)
	naxis = 1
	n1 = output['naxes'][0]; n2=0
	if len( output['naxes']) > 1:
		n2 = output['naxes'][1]
		if n2 > 1: naxis = naxis + 1
		else: n2 = 0

	if bitpix == 16:
	  hasbsc=0
	  hasbz=0
	  bzero=pow(2,bitpix-1)
	  bscale=1.0
	  for h in output['header']:
	   if h[0:6] == 'BSCALE': hasbsc=1
           if h[0:5] == 'BZERO': hasbz=1
	  if not hasbsc: output['header'].insert(7,['BZERO'])
	  if not hasbsc: output['header'].insert(7,['BSCALE'])

	fn1 = 0
	fn2 = 0
	for h in output['header']:
	   glue = 1
	   if h[0:6] == 'BITPIX':
	     h="BITPIX  = "+ rjust(str(bitpix),20)
	     h=ljust(h,80)
	   if h[0:6] == 'NAXIS ':
	     h="NAXIS   = "+ rjust(str(naxis),20)
	     h=ljust(h,80)
	   if h[0:6] == 'NAXIS1':
	     h="NAXIS1  = "+ rjust(str(n1),20)
	     h=ljust(h,80)
	     fn1 = 1
	   if h[0:6] == 'NAXIS2':
	    if n2:
	     h="NAXIS2  = "+ rjust(str(n2),20)
	     h=ljust(h,80)
	     fn2 = 1
	    else: h=''
	   if h[0:6] == 'BSCALE':
	     if bitpix == 16:
	      h="BSCALE  = "+ rjust(str(bscale),20)
	      h=ljust(h,80)
	     else:
	      glue = 0
           if h[0:5] == 'BZERO':
	     hasbz=1
	     if bitpix == 16:
	      h="BZERO   = "+ rjust(str(bzero),20)
	      h=ljust(h,80)
	     else:
	      glue = 0
	   if not h == ender and not h == blank and glue:
	     header = header + h

	if naxis == 2 and not fn2 and n2:
	     h="NAXIS2  = "+ rjust(str(n2),20)
	     h=ljust(h,80)
	     fn2 = 1
	     header = header + h

	nhl = len(header)/2880
	nhr = (2880-len(header)%2880)/80

	if nhr < 36:
	 for i in range(0,nhr):
	   header = header + ljust(" ",80)
	for i in range(0,35):
	   header = header + ljust(" ",80)
	header = header + ender

	if bitpix == 16: outtype=Numeric.Int16
	if bitpix == 32: outtype=Numeric.Int32
	if bitpix == -32: outtype=Numeric.Float32

	if bitpix == 16:
	  output['data']=((output['data']-bzero)/bscale)

	output['data']=output['data'].astype(outtype)

	file=open(outfile,'wb')
	file.write(header)
	if os.uname()[0]=='OSF1' or os.uname()[4]=='i686':
	  file.write( output['data'].byteswapped().tostring())
	else:
	  file.write( output['data'].tostring())
	file.close()

def fits2sock(fits,SOCKET):
	while 1:
	  if os.path.exists(SOCKET): break
	sock=socket(AF_UNIX,SOCK_STREAM)
	sock.connect(SOCKET)
	sock.send( cPickle.dumps(fits,1))
	sock.close()

def sock2fits(fitsfile):
	if os.path.exists( fitsfile): os.remove(fitsfile)
	sock=socket(AF_UNIX,SOCK_STREAM)
	sock.bind(fitsfile)
	sock.listen(1)
	conn, addr = sock.accept()
	inlist=[]
	while 1:
	  indat=conn.recv(BUFLEN)
	  if indat:
	    inlist.append(indat)
	  else:
	    break
	conn.close()
	if os.path.exists( fitsfile): os.remove(fitsfile)
	return 	cPickle.loads( join(inlist,''))

def getsec(data,box):
	data=Numeric.take(data,range(box[0],box[1]+1))
	data=Numeric.transpose(data)
	data=Numeric.take(data,range(box[2],box[3]+1))
	return data

def readrow(fitsfile,bitpix,naxis,naxes,bscale,bzero,lh,row):

	if fitsfile[-3:] == '.gz': fitsfile=fitsfile[0:-3]
	gzipped = 0
	if not os.path.exists(fitsfile):
	  gzfitsfile = fitsfile+".gz"
	  if not os.path.exists(gzfitsfile):
	   print "File not found: ", fitsfile; sys.exit()
	  else: gzipped = 1; fitsfile=gzfitsfile

	iend = lh*80
	nrec = iend/2880

	if ( iend/2880.0 > iend/2880): nrec = nrec + 1

	numpix = naxes[0]
	if len(naxes) > 1: numpix = numpix*naxes[1]

	if bitpix == 16: intype=Numeric.Int16
	if bitpix == 32: intype=Numeric.Int32
	if bitpix == -32: intype=Numeric.Float32
	bytepix = abs(bitpix)/8
	length = naxes[0] * bytepix
	start = nrec*2880

	if gzipped: file=gzip.open(fitsfile,'r')
	else: file=open(fitsfile,'r')

	file.seek(start+row*length)
	data = file.read(length)
	file.close()

	data = Numeric.fromstring(data,intype)
	if os.uname()[0]=='OSF1' or os.uname()[4]=='i686':
	 data = data.byteswapped()

	if bscale != 1.0: data = multiply(data,bscale)
	if bzero != 0.0: data = add(data,bzero)

	data=data.astype(Numeric.Float32)

	return data

