#!/usr/bin/env python

from FITS import *
from types import *
from basics import *
from vtk import *

def NumToVTKArray(numarray,name=None):
      __typeDict = {'c':vtkUnsignedCharArray, 'b':vtkUnsignedCharArray, '1':vtkCharArray,
              's':vtkShortArray, 'i':vtkIntArray, 'l':vtkLongArray, 'f':vtkFloatArray,
              'd':vtkDoubleArray, 'F':vtkFloatArray, 'D':vtkDoubleArray }
      vtkarray = __typeDict[numarray.typecode()]()
      dims = numarray.shape
      vtkarray.SetVoidArray(numarray,multiply.reduce(dims),1)
      if len(dims) > 1: vtkarray.SetNumberOfComponents(dims[0])
      else: vtkarray.SetNumberOfComponents(1)
      vtkarray.SetNumberOfValues(multiply.reduce(dims))
      if name: vtkarray.SetName(name)
      return vtkarray

def VTKArrayToNum(i):
      __typeDict = { VTK_CHAR:Numeric.Int8, VTK_UNSIGNED_CHAR:Numeric.UnsignedInt8,
      VTK_SHORT:"s", VTK_UNSIGNED_SHORT:Numeric.Int16, VTK_INT:"i",
      VTK_FLOAT:"f", VTK_DOUBLE:"d", VTK_LONG:"l"}
      __vtkDict = { VTK_CHAR:vtkCharArray, VTK_UNSIGNED_CHAR:vtkUnsignedCharArray,
      VTK_SHORT:vtkShortArray, VTK_UNSIGNED_SHORT:vtkUnsignedShortArray, VTK_INT:vtkIntArray,
      VTK_FLOAT:vtkFloatArray, VTK_DOUBLE:vtkDoubleArray, VTK_LONG:vtkLongArray}
      it = i.GetDataType()
      ns = i.GetSize()
      nc = i.GetNumberOfComponents()
      nt = i.GetNumberOfTuples()
      na = i.GetActualMemorySize()
      nat = __typeDict[it]
      d = zeros((nc,nt),nat)
      for c in range(nc):
        n = __vtkDict[it]()
        n.SetNumberOfComponents(1)
        n.SetNumberOfValues(nt)
        n.CopyComponent(0,i,c)
        o = VTKImageToNum(VTKArrayToVTKImage(n))
        d[c] = o.astype(nat)
#        del n
      return d

def VTKCalculator(*args):
      __typeDict = {'c':VTK_UNSIGNED_CHAR, 'b':VTK_UNSIGNED_CHAR, '1':VTK_CHAR,
              's':VTK_SHORT, 'i':VTK_INT, 'l':VTK_LONG, 'f':VTK_FLOAT,
              'd':VTK_DOUBLE, 'F':VTK_FLOAT, 'D':VTK_DOUBLE }
      task = None
      task = [a for a in args if type(a) == StringType]
      if not len(task): return -1
      variables = [a for a in args if type(a) != StringType]
      images = [VTKArrayToVTKImage(v) for v in variables]
#      print [image.GetScalarTypeAsString() for image in images]
      ia = vtkImageAppendComponents()
      [ia.AddInput(i) for i in images]
      iu = ia.GetOutput()
      ia.Update()
      iu.GetPointData().GetScalars().SetName("InputData")
      m = vtkArrayCalculator()
      m.SetFunction(task[0])
      m.SetInput(iu)
      m.SetResultArrayName("InputData")
      [m.AddScalarVariable(variables[j].GetName(),"InputData",j) for j in range(len(variables))]
      o = m.GetOutput()
      m.Update()
      o.SetNumberOfScalarComponents( o.GetPointData().GetScalars().GetNumberOfComponents())
      return o

def VTKArrayToVTKImage(v):
      ii = vtkImageData()
      ii.SetScalarType(v.GetDataType())
      ii.SetNumberOfScalarComponents(1)
      ii.GetPointData().AllocateArrays(1)
      ii.GetPointData().SetScalars(v)
      shape2 = [v.GetNumberOfComponents(), v.GetNumberOfTuples()]
      ii.SetUpdateExtentToWholeExtent()
      ii.SetExtent(0,shape2[1]-1,0,shape2[0]-1,0,0)
      ii.SetNumberOfScalarComponents(1)
      ii.GetPointData().GetScalars().SetNumberOfComponents(1)
      ii.Update()
      ii.UpdateData()
      ii.UpdateInformation()
      return ii

def NumToVTKImage(numarray,name=None):
      vtkarray = NumToVTKArray(numarray,name=name)
      ii = VTKArrayToVTKImage(vtkarray)
      return ii

def oldNumToVTKImage(numarray,name=None):
      __typeDict = {'c':VTK_UNSIGNED_CHAR, 'b':VTK_UNSIGNED_CHAR, '1':VTK_CHAR,
              's':VTK_SHORT, 'i':VTK_INT, 'l':VTK_LONG, 'f':VTK_FLOAT,
              'd':VTK_DOUBLE, 'F':VTK_FLOAT, 'D':VTK_DOUBLE }
      ii = vtkImageImport()
      ii.SetDataScalarType(__typeDict[ numarray.typecode()])
      if len(numarray.shape) == 1: shape2 = (1,) + numarray.shape
      else: shape2 = numarray.shape
      ii.SetWholeExtent(0,shape2[1]-1,0,shape2[0]-1,0,0)
      ii.SetDataExtentToWholeExtent()
      ii.SetNumberOfScalarComponents(1)
      ii.SetImportVoidPointer(numarray)
      ii.Update()
      im = ii.GetOutput()
      im.UpdateData()
      im.UpdateInformation()
      return im

def VTKImageToNum(i):
      __typeDict = { VTK_CHAR:Numeric.Int8, VTK_UNSIGNED_CHAR:Numeric.UnsignedInt8,
      VTK_SHORT:"s", VTK_UNSIGNED_SHORT:Numeric.Int16, VTK_INT:"i",
      VTK_FLOAT:"f", VTK_DOUBLE:"d", VTK_LONG:"l"}
      ie = vtkImageExport()
      d = list(i.GetDimensions())
      d.reverse()
      if d[0] == 1: d = d[1:]
      if d[0] == 1: d = d[1:]
      it = i.GetScalarType()
      scalars = i.GetPointData().GetScalars()
      if scalars: it = scalars.GetDataType()
      else: it = VTK_FLOAT
      nat = __typeDict[it]
      x = zeros(d,nat)
      ie.SetExportVoidPointer(x)
      ie.ImageLowerLeftOn()
      ie.SetInput(i)
      ie.Export()
      return MLab.squeeze(x)

def VTKRecast(i,t):
      rc = vtkImageCast()
      rc.SetInput(i)
      if t == "d": rc.SetOutputScalarTypeToDouble()
      elif t == "f": rc.SetOutputScalarTypeToFloat()
      elif t == "s": rc.SetOutputScalarTypeToUnsignedShort()
      elif t == "b": rc.SetOutputScalarTypeToUnsignedChar()
      elif t == "i": rc.SetOutputScalarTypeToInt()
      elif t == "l": rc.SetOutputScalarTypeToLong()
      oc = rc.GetOutput()
      rc.Update()
      return oc

def VTKContourPoints(x,v):
    if type(x) == type(arange(10)): i = NumToVTKImage(x)
    else: i = x
    d = i.GetDimensions()
    c = vtkContourFilter()
    c.SetInput(i)
    [c.SetValue(j,v[j]) for j in range(len(v))]
    c.ComputeScalarsOn(); c.ComputeNormalsOn()
    c.ComputeGradientsOn(); c.UseScalarTreeOn()
    p = c.GetOutput()
    c.Update()
    p.GetBounds()
    pc = vtkPolyDataConnectivityFilter()
    pc.SetInput(p)
    pc.SetExtractionModeToAllRegions()
    pc.ScalarConnectivityOn()
    pc.Update()
    u = pc.GetOutput()
    print u
    print dir(u)
    pw = vtkPolyDataWriter()
    pw.SetInput(u)
    pw.SetFileName("pd.dat")
    pw.Write()
    print u.GetCellData()

def newVTKImContour(x,v,numret=0):
    if type(x) == ArrayType: i = NumToVTKImage(x)
    else: i = x
    d = i.GetDimensions()
    m = vtkImageMarchingCubes()
    [m.SetValue(j,v[j]) for j in range(len(v))]
    m.SetInput(i)
    m.ComputeScalarsOn()
    m.ComputeNormalsOn()
    m.ComputeGradientsOn()
    p = m.GetOutput()
    m.Update()
    print p


def oldVTKImContour(x,v,numret=0,alias=3,mag=1):
    if type(x) == type(arange(10)): i = NumToVTKImage(x)
    else: i = x
    d = i.GetDimensions()
    c = vtkContourFilter()
    c.SetInput(i)
    [c.SetValue(j,v[j]) for j in range(len(v))]
    c.ComputeScalarsOn(); c.ComputeNormalsOff()
    c.ComputeGradientsOff(); c.UseScalarTreeOn()
    f = c.GetOutput()
    f.Update()
    q = vtkPolyDataMapper()
    q.SetInput( f)
    q.SetScalarRange(v[0],v[-1])
    r = vtkActor()
    r.SetMapper(q)
    s = vtkRenderer()
    s.AddActor(r)
    s.GetActiveCamera().ParallelProjectionOn()
    s.GetActiveCamera().SetParallelScale(d[1]/2.0)
    zp = s.GetActiveCamera().GetPosition()[2]
    s.GetActiveCamera().SetPosition((d[0]-1)/2.,(d[1]-1)/2., zp)
    s.GetActiveCamera().SetFocalPoint((d[0]-1)/2.,(d[1]-1)/2., 0.0)
    u = vtkRenderWindow()
    u.AddRenderer(s)
    u.BordersOff()
    u.LineSmoothingOn()
    u.SetSize(d[0]/alias,d[1]/alias)
    u.OffScreenRenderingOn()
    u.DoubleBufferOff()
    t = vtkRenderLargeImage()
    t.SetInput(s)
    t.SetMagnification(alias)
    e = vtkImageLuminance()
    e.SetInput( t.GetOutput())
    y = vtkImageThreshold()
    y.ThresholdBetween(0,1.0)
    y.SetOutValue(1.0)
    y.ReplaceOutOn()
    y.SetOutputScalarTypeToFloat()
    y.SetInput( e.GetOutput())
    p = vtkImageMirrorPad()
    p.SetOutputWholeExtent(0,d[0]-1,0,d[1]-1,0,0)
    p.SetInput(y.GetOutput())
    w = p.GetOutput()
    w.Update()
    if numret: return VTKImageToNum(w)
    else: return w

def VTKSimpleTransform(x,origin,pa,offset=[0,0],numret=0,interp=0,cubic=0):
      if type(x) == type(arange(2)): i = NumToVTKImage(x)
      else: i = x
      t = vtkTransform()
      t.Translate(origin[0],origin[1],0)
      t.RotateWXYZ(pa,0,0,1)
      t.Translate(-origin[0]+offset[0],-origin[1]+offset[1],0)
      s = vtkImageReslice()
      s.SetResliceAxesOrigin(origin[0],origin[1],0)
      if cubic: s.SetInterpolationModeToCubic()
      else: s.SetInterpolationModeToLinear()
      s.SetOutputDimensionality(2)
      s.SetResliceTransform(t)
      s.SetInput(i)
      o = s.GetOutput()
      s.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKSlice(x,origin,pa,numret=0,interp=0):
      if type(x) == type(arange(2)): i = NumToVTKImage(x)
      else: i = x
      t = vtkTransform()
      t.Translate(origin[0],origin[1],0)
      t.RotateWXYZ(pa,0,0,1)
      t.Translate(-origin[0],-origin[1],0)
      s = vtkImageReslice()
      s.SetResliceAxesOrigin(origin[0],origin[1],0)
      s.SetInterpolationModeToCubic()
      s.SetOutputDimensionality(1)
      s.SetResliceTransform(t)
      s.SetInput(i)
      o = s.GetOutput()
      s.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKImageShift(x,u,v,numret=0,interp=1,wrap=0,mirror=0,constant=None,cubic=0):
      if type(x) == type(arange(2)): i = NumToVTKImage(x)
      else: i = x
      if 0 and int(u) == u and int(v) == v:
         s = vtkImageTranslateExtent()
         s.SetInput(i)
         s.SetTranslation(u,v,0)
         o = s.GetOutput()
         s.Update()
      else:
         s = vtkImageReslice()
         s.AutoCropOutputOn()
         if wrap: s.WrapOn()
         else: s.WrapOff()
         if mirror or constant != None: s.MirrorOn()
         else: s.MirrorOff()
         if interp:
           s.InterpolateOn()
           if cubic:
             s.SetInterpolationModeToCubic()
           else:
             s.SetInterpolationModeToLinear()
         else: s.InterpolateOff()
         s.OptimizationOn()
         s.SetOutputSpacing(1,1,1)
         s.SetOutputOrigin(u,v,0)
         s.SetInput(i)
         o=s.GetOutput()
         s.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKInvertTransform(dx,dy,numret=0):
      if type(dx) == type(arange(10)): tx = NumToVTKImage(dx)
      else: tx = dx
      if type(dy) == type(arange(10)): ty = NumToVTKImage(dy)
      else: ty = dy
      dm = tx.GetDimensions()
      dz = zeros(dy.shape,Float32)
      tz = NumToVTKImage(dz)
      a = vtkImageAppendComponents()
      a.AddInput(tx)
      a.AddInput(ty)
      a.AddInput(tz)
      t = a.GetOutput()
      r = vtkGridTransform()
      r.SetDisplacementGrid(t)
      r.SetInterpolationModeToLinear()
      r.SetInverseTolerance(0.001)
      r.SetInverseIterations(0)
      r.Update()
      i=r.GetInverse()
      print i
      o = i.GetDisplacementGrid()
      i.Update()
      print o
      if numret: return VTKImageToNum(o)
      else: return o

def VTKImageTransform(x,dx,dy,numret=0,reverse=0,origsize=None,cubic=0,interp=1,scalex=1,scaley=1,constant=0,wrap=0,mirror=0):
      maxdx = 2*int(max(abs(dx.flat))+1)
      maxdy = 2*int(max(abs(dy.flat))+1)
      dx = dx.astype(Float32)
      dy = dy.astype(Float32)
      if scalex > 1:
         xx = arange(x.shape[1])
         dx = (xx[NewAxis,::]+dx).astype(Float32)
         dy = VTKGrowN(dy,scalex,1,numret=1)
         dx = VTKGrowN(dx,scalex,1,numret=1)
         dx = (dx-arange(scalex*x.shape[1])[NewAxis,::]).astype(Float32)
      if scaley > 1:
         yy = arange(x.shape[0])
         dy = (yy[::,NewAxis]+dy).astype(Float32)
         dy = VTKGrowN(dy,1,scaley,numret=1)
         dx = VTKGrowN(dx,1,scaley,numret=1)
         dy = (dy-arange(scaley*x.shape[0])[::,NewAxis]).astype(Float32)
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      if type(dx) == ArrayType: tx = NumToVTKImage(dx)
      else: tx = dx
      if type(dy) == ArrayType: ty = NumToVTKImage(dy)
      else: ty = dy
      dm = ty.GetDimensions()
      dz = zeros(dy.shape,dy.typecode())
      tz = NumToVTKImage(dz)
      a = vtkImageAppendComponents()
      a.AddInput(tx)
      a.AddInput(ty)
      a.AddInput(tz)
      t = a.GetOutput()
      r = vtkGridTransform()
      r.SetDisplacementGrid(t)
      r.Update()
      s = vtkImageReslice()
      s.WrapOff()
      s.MirrorOn()
      if interp:
         s.InterpolateOn()
         if cubic: s.SetInterpolationModeToCubic()
         else: s.SetInterpolationModeToLinear()
      s.SetOutputDimensionality(2)
      if reverse:
         r.SetInterpolationModeToLinear()
         r.SetInverseTolerance(0.001)
         r.SetInverseIterations(1000)
         r.DebugOff()
         ir = r.GetInverse()
         ir.SetInterpolationModeToLinear()
         ir.SetInverseTolerance(0.001)
         ir.SetInverseIterations(1000)
         ir.GlobalWarningDisplayOff()
         s.SetResliceTransform(ir)
         s.AutoCropOutputOff()
         if origsize: s.SetOutputExtent(0,origsize[1]-1,0,origsize[0]-1,0,0)
         else: s.SetOutputExtent(0,scalex*dm[0]-1,0,scaley*dm[1]-1,0,0)
      else:
         r.SetInterpolationModeToCubic()
         r.SetInverseTolerance(0.001)
         r.SetInverseIterations(1000)
         r.GlobalWarningDisplayOff()
         s.SetOutputExtent(0,scalex*dm[0]-1,0,scaley*dm[1]-1,0,0)
         s.AutoCropOutputOff()
         s.SetResliceTransform(r)
      if mirror: ip = vtkImageMirrorPad()
      elif wrap: ip = vtkImageWrapPad()
      else: ip = vtkImageConstantPad(); ip.SetConstant(constant)
      ip.SetOutputWholeExtent(0-maxdx,dm[0]-1+maxdx,0-maxdy,dm[1]-1+maxdy,0,0)
      ip.SetInput(i)
      s.SetInput(ip.GetOutput())
      o=s.GetOutput()
      s.Update()
      if numret: ri = VTKImageToNum(o)
      else: ri = o
      return ri

def VTKDivergence(x,numret=0):
      if type(x) == type(arange(10)): i = NumToVTKImage(x)
      else: i = x
      d = vtkImageDivergence()
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKLaplace(x,numret=0):
      if type(x) == type(arange(10)): i = NumToVTKImage(x)
      else: i = x
      d = vtkImageLaplacian()
      d.SetDimensionality(2)
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKImageArith(x1,x2,op,numret=0,rep=0.0):
      if type(x1) == ArrayType and type(x2) == ArrayType:
        if x1.typecode() == "d" or x2.typecode() == "d":
          x1,x2 = x1.astype("d"), x2.astype("d")
        elif x1.typecode() == "f" or x2.typecode() == "f":
          x1,x2 = x1.astype("f"), x2.astype("f")
      if type(x1) == ArrayType: i1 = NumToVTKImage(x1)
      else: i1 = x1
      if type(x2) == ArrayType: i2 = NumToVTKImage(x2)
      else: i2 = x2
      m = vtkImageMathematics()
      numops = [ "multiply","divide","add","subtract"]
      if op == "divide":
        m.SetConstantC(rep); m.DivideByZeroToCOn()
      if type(x1) == ArrayType and type(x2) == ArrayType:
         m.AddInput(i1)
         m.AddInput(i2)
         vtkops = [ m.SetOperationToMultiply, m.SetOperationToDivide, m.SetOperationToAdd, m.SetOperationToSubtract]
      elif type(x1) == ArrayType:
         m.AddInput(i1)
         if i2 == 0:
            const = 0.0
         else:
            const = [i2,1.0/i2,i2,-i2][numops.index(op)]
         m.SetConstantK(const)
         m.SetConstantC(const)
         vtkops = [ m.SetOperationToMultiplyByK, m.SetOperationToMultiplyByK, m.SetOperationToAddConstant, m.SetOperationToAddConstant]
      if type(x1) != ArrayType and type(x2) != ArrayType:
         return eval(op)(x1,x2)
      else:
         vtkops[numops.index(op)]()
         o = m.GetOutput()
         m.Update()
         if numret: return VTKImageToNum(o)
         else: return o

def VTKGreaterEqual(x1,th):
      if type(x1) == ArrayType: i1 = NumToVTKImage(x1)
      else: i1 = x1
      m = vtkImageThreshold()
      m.ThresholdByUpper(th)
      m.SetInput(i1)
      m.ReplaceInOn()
      m.ReplaceOutOn()
      m.SetInValue(1.0)
      m.SetOutValue(0.0)
      m.SetOutputScalarTypeToFloat()
      o = m.GetOutput()
      m.Update()
      return VTKImageToNum(o)

def VTKLessEqual(x1,th):
      if type(x1) == ArrayType: i1 = NumToVTKImage(x1)
      else: i1 = x1
      m = vtkImageThreshold()
      m.ThresholdByLower(th)
      m.SetInput(i1)
      m.ReplaceInOn()
      m.ReplaceOutOn()
      m.SetInValue(1.0)
      m.SetOutValue(0.0)
      m.SetOutputScalarTypeToFloat()
      o = m.GetOutput()
      m.Update()
      return VTKImageToNum(o)

def VTKBetween(x1,th1,th2):
      if type(x1) == ArrayType: i1 = NumToVTKImage(x1)
      else: i1 = x1
      m = vtkImageThreshold()
      m.ThresholdBetween(th1,th2)
      m.SetInput(i1)
      m.ReplaceInOn()
      m.ReplaceOutOn()
      m.SetInValue(1.0)
      m.SetOutValue(0.0)
      m.SetOutputScalarTypeToFloat()
      o = m.GetOutput()
      m.Update()
      return VTKImageToNum(o)

def VTKImageLogic(x1,x2=None,op="or",numret=0):
      if type(x1) == ArrayType: i1 = NumToVTKImage(x1)
      else: i1 = x1
      if type(x2) == ArrayType: i2 = NumToVTKImage(x2)
      elif x2: i2 = x2
      m = vtkImageLogic()
      numops = [ "and","or","xor","nand","nor","not"]
      vtkops = [ m.SetOperationToAnd, m.SetOperationToOr, m.SetOperationToXor, m.SetOperationToNand, m.SetOperationToNor, m.SetOperationToNot]
      m.SetOutputTrueValue(1.0)
      vtkops[numops.index(op)]()
      m.AddInput(i1)
      if x2: m.AddInput(i2)
      else: m.AddInput(i1)
      o = m.GetOutput()
      m.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKOr(x1,x2): return VTKImageLogic(x1,x2=x2,op="or",numret=1)
def VTKAnd(x1,x2): return VTKImageLogic(x1,x2=x2,op="and",numret=1)
def VTKNot(x1): return VTKImageLogic(x1,x2=None,op="not",numret=1)

def VTKInvert(x1,rep=0.0):
      if type(x1) == ArrayType: i1 = NumToVTKImage(x1)
      else: i1 = x1
      m = vtkImageMathematics()
      m.SetOperationToInvert()
      m.AddInput(i1)
      m.SetConstantC(rep)
      m.DivideByZeroToCOn()
      o = m.GetOutput()
      m.Update()
      return VTKImageToNum(o)

def VTKAbs(x1):
      if type(x1) == ArrayType: i1 = NumToVTKImage(x1)
      else: i1 = x1
      m = vtkImageMathematics()
      m.SetOperationToAbsoluteValue()
      m.AddInput(i1)
      o = m.GetOutput()
      m.Update()
      return VTKImageToNum(o)

def VTKSqrt(x0,rep=0.0):
      bad = less(x0,0)
      x1 = VTKAbs(x0)
      if type(x1) == ArrayType: i1 = NumToVTKImage(x1)
      else: i1 = x1
      m = vtkImageMathematics()
      m.SetOperationToSquareRoot()
      m.AddInput(i1)
      o = m.GetOutput()
      m.Update()
      b = VTKImageToNum(o)
      return where(bad,rep,b)

def VTKAdd(x1,x2): return VTKImageArith(x1,x2,"add",numret=1)
def VTKSubtract(x1,x2): return VTKImageArith(x1,x2,"subtract",numret=1)
def VTKMultiply(x1,x2): return VTKImageArith(x1,x2,"multiply",numret=1)
def VTKDivide(x1,x2,rep=0): return VTKImageArith(x1,x2,"divide",numret=1,rep=rep)

def VTKGradient(x,numret=0):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      g = vtkImageGradient()
      g.SetInput(i)
      g.SetDimensionality(2)
      g.HandleBoundariesOn()
      o = g.GetOutput()
      g.Update()
      if numret:
         c1 = vtkImageExtractComponents()
         c1.SetInput(o)
         c1.SetComponents(0)
         o1 = c1.GetOutput()
         c2 = vtkImageExtractComponents()
         c2.SetInput(o)
         c2.SetComponents(1)
         o2 = c2.GetOutput()
         c1.Update()
         c2.Update()
         a1 = VTKImageToNum(o1)
         a2 = VTKImageToNum(o2)
         a = asarray([a1,a2])
         return a
      else: return o

def VTKGradMag(x,numret=0,dim=2):
      if type(x) == type(arange(10)): i = NumToVTKImage(x)
      else: i = x
      dm = i.GetDimensions()
      maxdx = 5
      maxdy = 5
      ip = vtkImageWrapPad()
      ip.SetOutputWholeExtent(0-maxdx,dm[0]-1+maxdx,0-maxdy,dm[1]-1+maxdy,0,0)
      ip.SetInput(i)
      d = vtkImageGradientMagnitude()
      d.SetDimensionality(dim)
      d.HandleBoundariesOn()
      d.SetInput(ip.GetOutput())
      r = vtkImageReslice()
      r.SetOutputExtent(0,dm[0]-1,0,dm[1]-1,0,0)
      r.SetInput(d.GetOutput())
      o = r.GetOutput()
      r.Update()
      if numret:
         a = VTKImageToNum(o)
         a[0] = a[1]
         a[-1] = a[-2]
         if len(a.shape) > 1:
            a[::,0] = a[::,1]
            a[::,-1] = a[::,-2]
         return a
      else: return o

def VTKAnisotropicDiffusion(x,th,fac,iter,faces=0,edges=1,corners=0,gmag=0,numret=1):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      d = vtkImageAnisotropicDiffusion2D()
      d.SetDiffusionThreshold(th)
      d.SetDiffusionFactor(fac)
      d.SetNumberOfIterations(iter)
      d.FacesOff()
      d.EdgesOff()
      d.CornersOff()
      if faces: d.FacesOn()
      if edges: d.EdgesOn()
      if corners: d.CornersOn()
      if gmag: d.GradientMagnitudeThresholdOn()
      else: d.GradientMagnitudeThresholdOff()
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKNonMaxSuppress(x,numret=0,usegrad=0):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      vi = vtkImageGradient()
      vi.SetDimensionality(2)
      vi.HandleBoundariesOn()
      vi.SetInput(i)
      d = vtkImageNonMaximumSuppression()
      d.HandleBoundariesOn()
      d.SetDimensionality(2)
      if usegrad:
         mi = vtkImageGradientMagnitude()
         mi.SetDimensionality(2)
         mi.HandleBoundariesOn()
         mi.SetInput(i)
         d.SetMagnitudeInput(mi.GetOutput())
      else:
         d.SetMagnitudeInput(i)
      d.SetVectorInput(vi.GetOutput())
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKContours(x,v):
    if type(x) == ArrayType: i = NumToVTKImage(x)
    else: i = x
    d = i.GetDimensions()
    c = vtkKitwareContourFilter()
#    c = vtkMarchingSquares()
    c.SetInput(i)
    [c.SetValue(j,v[j]) for j in range(len(v))]
    c.ComputeScalarsOn(); c.ComputeNormalsOff()
    c.ComputeGradientsOff(); c.UseScalarTreeOn()
    p = c.GetOutput()
    c.Update()
    return p

def VTKConnectivity(p,x=None,y=None,regions=[],num=0):
    pc = vtkPolyDataConnectivityFilter()
    pc.SetInput(p)
    if len(regions):
       pc.SetExtractionModeToSpecifiedRegions()
       pc.InitializeSpecifiedRegionList()
       [pc.AddSpecifiedRegion(region) for region in regions]
#       [pc.DeleteSpecifiedRegion(region) for region in range(num) if region not in regions]
    else:
       pc.SetExtractionModeToClosestPointRegion()
       pc.SetClosestPoint([x,y,0])
       pc.ScalarConnectivityOn()
    pc.Update()
    u = pc.GetOutput()
    u.Update()
    return u

def VTKPolyDataToImageStencil(p,x,bg=500):
    n = ones(x.shape,x.typecode())
    u = NumToVTKImage(n)
    s = vtkPolyDataToImageStencil()
    s.SetInput(p)
    t = vtkImageStencil()
    t.SetStencil(s.GetOutput())
    t.SetInput(u)
    t.SetBackgroundValue(bg)
    t.ReverseStencilOn()
    o = t.GetOutput()
    o.Update()
    o = VTKImageToNum(o)
    return o

def VTKGetContours(x,v,verb=0,nsamp=1,minsides=0):
     iso = VTKContours(x,[v])
     pc = vtkPolyDataConnectivityFilter()
     pc.ScalarConnectivityOff()
     pc.ColorRegionsOn()
     pc.SetExtractionModeToAllRegions()
     pc.SetInput(iso)
     pc.Update()
     num = pc.GetNumberOfExtractedRegions()
     if verb:
        if nsamp > 1: print "Number of regions (sampled): %d (%d)" % (num,num/nsamp)
        else: print "Number of regions: %d" % (num)
     contours = []
     for region in range(0,num,nsamp):
       if verb:
          sys.stdout.write(".")
          sys.stdout.flush()
       contour = VTKConnectivity(iso,regions=[region])
       nc = contour.GetNumberOfCells()
       if nc >= minsides:
          segments = []
          for q in range(nc):
             line = contour.GetCell(q)
             segment = VTKArrayToNum(line.GetPoints().GetData())[:2]
             segments.append(segment.astype(Float64))
          contours.append(segments)
     if verb: print
     return contours

def VTKDrawContour(x,contours,color=1,fill=0):
    if type(x) == ArrayType: i = NumToVTKImage(x)
    else: i = x
    c = vtkImageCanvasSource2D()
    e = i.GetExtent()
    c.SetExtent(e[0],e[1],e[2],e[3],e[4],e[5])
    c.SetScalarType( i.GetScalarType())
    for contour in contours:
       for segment in contour:
          c.SetDrawColor(color)
          x,y = (segment+0.5).astype(Int32)
          c.DrawSegment(x[0],y[0],x[1],y[1])
       if contour and fill:
         (xb,yb),(xe,ye) = transpose(contour)
         c.SetDrawColor(fill)
         xm = int(mean(xb)+0.5)
         ym = int(mean(yb)+0.5)
         c.FillPixel(xm,ym)
    o = c.GetOutput()
    c.Update()
    if o: n = VTKImageToNum(o)
    else: n = zeros(x.shape,x.typecode())
    return n


def VTKDrawContours(x,vs,color=1,fill=0,verb=0,nsamp=1,minsides=0):
    if type(x) == ArrayType: i = NumToVTKImage(x)
    else: i = x
    c = vtkImageCanvasSource2D()
    e = i.GetExtent()
    c.SetExtent(e[0],e[1],e[2],e[3],e[4],e[5])
    c.SetScalarType( i.GetScalarType())
    n = zeros(x.shape,x.typecode())
    for v in sort(vs)[::-1]:
      if color <= 0: c.SetDrawColor(v)
      else: c.SetDrawColor(color)
      contours = VTKGetContours(i,v,verb=verb,nsamp=nsamp,minsides=minsides)
      if contours:
         n = n + VTKDrawContour(x,contours,color=1,fill=0)
    return n

def VTKSeedConnectivity(t,x=[],y=[],val=None):
    if type(t) == ArrayType:
       t = t.astype("b")
       i = NumToVTKImage(t)
    else: i = t
    sc = vtkImageSeedConnectivity()
    sc.SetInput( i)
    if not len(x):
       y,x = indices(t.shape)
       y,x = compress(t.flat,[y.flat,x.flat],1)
    for j in range(len(x)):
        sc.AddSeed(int(x[j]),int(y[j]))
        ival = t[int(y[j]),int(x[j])]
        if val: oval = val
        else: oval = 1
        sc.SetInputConnectValue(1)
        sc.SetOutputConnectedValue(oval)
        sc.SetOutputUnconnectedValue(0)
    sc.Update()
    so = sc.GetOutput()
    so = VTKRecast(so ,"f")
    si = VTKImageToNum(so)
    return si

def VTKIslandRemoval(x,isval,repval,area,numret=0):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      d = vtkImageIslandRemoval2D()
      d.SetIslandValue(isval)
      d.SetReplaceValue(repval)
      d.SetAreaThreshold(area)
      d.SquareNeighborhoodOn()
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKOpenClose(x,openval,closeval,k=5,numret=0):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      d = vtkImageOpenClose3D()
      d.SetKernelSize(k,k,1)
      d.SetOpenValue(openval)
      d.SetCloseValue(closeval)
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKErode(x,k=5,l=0,numret=0):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      d = vtkImageContinuousErode3D()
      if not l: l = k
      d.SetKernelSize(k,l,1)
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKDilate(x,k=5,l=None,numret=0):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      if l == None: l = k
      d = vtkImageContinuousDilate3D()
      d.SetKernelSize(k,l,1)
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKSkeleton(x,iter,prune=0,numret=1):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      d = vtkImageSkeleton2D()
      d.SetNumberOfIterations(iter)
      d.SetPrune(prune)
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKGauss(x,sigma=5,numret=1,radius=3,sigmax=None,sigmay=None):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      if sigmax == None: sigmax=sigma
      if sigmay == None: sigmay=sigma
      d = vtkImageGaussianSmooth()
      d.SetStandardDeviations(sigmax,sigmay,0)
      d.SetRadiusFactors(radius,radius,1)
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKConstantPad(im,t,c,numret=0):
      i = NumToVTKImage(im)
      d = i.GetDimensions()
      e = i.GetExtent()
      ip = vtkImageConstantPad()
      ip.SetOutputWholeExtent(t[0],t[1],t[2],t[3],0,0)
      ip.SetConstant(c)
      ip.SetInput(i)
      o = ip.GetOutput()
      ip.Update()
      if numret: return VTKImageToNum(o)
      else: return o


def VTKMirrorPad(im,t,numret=0):
      i = NumToVTKImage(im)
      d = i.GetDimensions()
      e = i.GetExtent()
      ip = vtkImageMirrorPad()
      ip.SetOutputWholeExtent(t[0],t[1],t[2],t[3],0,0)
      ip.SetInput(i)
      o = ip.GetOutput()
      ip.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKConvolve(im,numret=0,k=5,clip=1,x=1,y=1,wrap=0,mirror=1,constant=None,kernel=None):
      if type(im) == ArrayType: i = NumToVTKImage(im)
      else: i = im
      d = i.GetDimensions()
      e = i.GetExtent()
      dtest = sort(d)
      if sum(dtest[:2]) == 2: onedim=1
      else: onedim = 0
      if (d[0] == 1): oned = 1
      elif (d[1] == 1): oned = 2
      else: oned = 0
      if x and not y: oned = 2
      if y and not x: oned = 1
      if onedim: oned = 1
      if constant != None:
          ip = vtkImageConstantPad()
          if d[0] > 1: x0,x1=-k,d[0]+k-1
          else: x0,x1=0,0
          if d[1] > 1: y0,y1=-k,d[1]+k-1
          else: y0,y1=0,0
          ip.SetOutputWholeExtent(x0,x1,y0,y1,0,0)
          ip.SetConstant(constant)
          ip.SetInput(i)
          i = ip.GetOutput()
          ip.Update()
      elif mirror:
          ip = vtkImageMirrorPad()
          if d[0] > 1: x0,x1=-k,d[0]+k-1
          else: x0,x1=0,0
          if d[1] > 1: y0,y1=-k,d[1]+k-1
          else: y0,y1=0,0
          ip.SetOutputWholeExtent(x0,x1,y0,y1,0,0)
          ip.SetInput(i)
          i = ip.GetOutput()
          ip.Update()
      elif wrap:
          ip = vtkImageWrapPad()
          if d[0] > 1: x0,x1=-k,d[0]+k-1
          else: x0,x1=0,0
          if d[1] > 1: y0,y1=-k,d[1]+k-1
          else: y0,y1=0,0
          ip.SetOutputWholeExtent(x0,x1,y0,y1,0,0)
          ip.SetInput(i)
          i = ip.GetOutput()
          ip.Update()
      c = vtkImageConvolve()
      if not kernel:
          if k == 3: k1 = asarray([0.25,0.5,0.25])
          elif k == 5: k1 = asarray([0.0625,0.25,0.375,0.25,0.0625])
          if onedim: ke = ones((k,k),Float64) * k1
          elif not oned: ke = k1[::,NewAxis]*k1[NewAxis,::]
          elif oned == 1: ke = zeros((k,k),Float64); ke[::,2] = k1
          elif oned == 2: ke = zeros((k,k),Float64); ke[2] = k1
          ke = ravel(ke)
          ke = ke / sum(ke)
          if onedim: ke = len(k1)*ke
      else:
          k = kernel.shape[0]
          ke = ravel(kernel)
      if k == 3: c.SetKernel3x3(ke)
      if k == 5: c.SetKernel5x5(ke)
      if k == 7: c.SetKernel7x7(ke)
      c.SetInput(i)
      o = c.GetOutput()
      c.Update()
      if clip:
         ic = vtkImageClip()
         notx = -(not x) * 0
         noty = -(not y) * 0
         ic.SetOutputWholeExtent(notx,d[0]-1+notx,noty,d[1]-1+noty,0,0)
         ic.SetInput(o)
         ic.ClipDataOn()
         o = ic.GetOutput()
         ic.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKImageClip(x,d,numret=0):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      e = i.GetDimensions()
      ic = vtkImageClip()
      ic.SetOutputWholeExtent( e[0]/2-d[1]/2, e[0]/2+d[1]/2-1, e[1]/2-d[0]/2, e[1]/2+d[0]/2-1,0,0)
      ic.SetInput(i)
      ic.ClipDataOn()
      o = ic.GetOutput()
      ic.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKHybridMedian(x,numret=0):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      d = vtkImageHybridMedian2D()
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKVariance(x,n=3,m=None,numret=0):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      if not m: m = n
      d = vtkImageVariance3D()
      d.SetKernelSize(n,m,1)
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o


def VTKMedian(x,n=3,m=None,numret=0):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      if not m: m = n
      d = vtkImageMedian3D()
      d.SetKernelSize(n,m,1)
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKShrinkN(x,n=2,m=None,numret=0,xshift=0,yshift=0,usemean=0,usemed=0,usemax=0,usemin=0,verb=0):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      d = i.GetDimensions()
      if not m: m = n
      if d[0] == 1: n = 1
      if d[1] == 1: m = 1
      rx,ry,rz = i.GetDimensions()
      tx,ty = rx,ry
      sn = n
      sm = m
      pad = 0
      if rx % sn or ry % sm:
         tx = int(1.*rx/sn+0.5)*sn
         ty = int(1.*ry/sm+0.5)*sm
         if rx == 1: tx = 1
         if ry == 1: ty = 1
         if verb: print "padding to",tx,ty,rx,ry
         pad = 1
         ip = vtkImageMirrorPad()
         ip.SetOutputWholeExtent(0,tx-1,0,ty-1,0,0)
         ip.SetInput(i)
         i = ip.GetOutput()
         ip.Update()
      im = vtkImageShrink3D()
      im.SetShrinkFactors(n,m,1)
      im.MeanOff(); im.MedianOff(); im.MaximumOff(); im.MinimumOff()
      if usemed: im.MedianOn()
      elif usemean: im.MeanOn()
      elif usemax: im.MaximumOn()
      elif usemin: im.MinimumOn()
      im.SetShift(xshift,yshift,0)
      im.SetInput(i)
      o = im.GetOutput()
      o.SetSpacing(1,1,1)
      o.Update()
      im.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKMedianSmooth(x,n,m=None,numret=0):
     if not m: m = n
     a = VTKShrinkN(x,n,m,usemean=0,usemed=1,numret=0)
     xs = -0.5*(n-1)
     ys = -0.5*(m-1)
     b = VTKGrowN(a,n,m,origsize=x.shape,interp=1,numret=1,mirror=1,xshift=xs,yshift=ys)
     return b

def VTKMeanUp(x,l,radix=2):
    i = x
    for j in range(l):
      m = pow(radix,j+1)+1
      i = VTKMeanSmooth(i,m,numret=1)
    return i

def VTKMedianUp(x,l,radix=2):
    i = x
    for j in range(l):
      m = pow(radix,j+1)+1
      i = VTKMedianSmooth(i,m,numret=1)
    return i

def VTKMeanSmooth(x,n,m=None,numret=0):
     if not m: m = n
     a = VTKShrinkN(x,n,m,usemean=1,numret=0)
     xs = -0.5*(n-1)
     ys = -0.5*(m-1)
     b = VTKGrowN(a,n,m,origsize=x.shape,interp=1,numret=1,mirror=1,xshift=xs,yshift=ys)
     return b

def oldVTKMeanSmooth(x,n,m=None,numret=0):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      if m == None: m = n
      s = VTKResample(i,1.0/n,1.0/m,interp=1)
      o = VTKResample(s,n,m,interp=1)
      if numret: return VTKImageToNum(o)
      else: return o

def VTKResample(x,n,m=None,numret=0,origsize=None,interp=0):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      r = vtkImageResample()
      if interp: r.InterpolateOn()
      else: r.InterpolateOff()
      if not m: m=n
      r.SetAxisMagnificationFactor(0,n)
      r.SetAxisMagnificationFactor(1,m)
      r.SetInput(i)
      o = r.GetOutput()
      r.Update()
      if origsize == None:
        origsize = o.GetDimensions()
      p = vtkImageMirrorPad()
      p.SetOutputWholeExtent(0,origsize[0]-1,0,origsize[1]-1,0,0)
      p.SetInput(o)
      f = p.GetOutput()
      p.Update()
      o = f
      if numret: return VTKImageToNum(o)
      else: return o


def VTKImageCorrelation(x,y):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      if type(y) == ArrayType: j = NumToVTKImage(y)
      else: j = y
      c = vtkImageCorrelation()
      c.SetDimensionality(2)
      c.SetInput1(i)
      c.SetInput2(j)
      o = c.GetOutput()
      c.Update()
      return VTKImageToNum(o)

def VTKFFT(x,y):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      c = vtkImageFFT()
      c.SetInput(i)
      o = c.GetOutput()
      c.Update()
      return VTKImageToNum(o)

def VTKGrowN(x,n=2,m=None,numret=0,origsize=None,interp=0,wrap=0,constant=None,mirror=0,xshift=0,yshift=0):
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      d = i.GetDimensions()
      if not m: m = n
      if d[0] == 1: n = 1
      if d[1] == 1: m = 1
      im = vtkImageMagnify()
      im.SetMagnificationFactors(n,m,1)
      if interp: im.InterpolateOn()
      else: im.InterpolateOff()
      im.SetInput(i)
      o = im.GetOutput()
      im.Update()
      o.SetSpacing(1,1,1)
      o.Update()
      o = VTKImageShift(o,xshift,yshift,interp=1,wrap=wrap,constant=constant,mirror=mirror)
      if origsize:
            if len(origsize) == 1: origsize=(1,)+origsize
            p = vtkImageMirrorPad()
            p.SetOutputWholeExtent(0,origsize[1]-1,0,origsize[0]-1,0,0)
            p.SetInput(o)
            f = p.GetOutput()
            p.Update()
            o = f
      if numret: return VTKImageToNum(o)
      else: return o

def oldVTKLevel(x,level=1,k=5,radix=2,pad=0,verb=0):
      x = x.astype(Float32)
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      d = i.GetDimensions()
      if level: 
         level = level
      else: 
         nx = int(log2(d[0]-1))
         ny = int(log2(d[1]-1))
         level = min([nx,ny])
      f = i

      for j in range(level):
         rx,ry,rz = f.GetDimensions()
         tx,ty = rx,ry
         s = radix
         if rx % s or ry % s:
            tx = int(1.*rx/s+1)*s
            ty = int(1.*ry/s+1)*s
            if rx == 1: tx = 1
            if ry == 1: ty = 1
            if verb: print "padding to",tx,ty,rx,ry
            ip = vtkImageMirrorPad()
            ip.SetOutputWholeExtent(0,tx-1,0,ty-1,0,0)
            ip.SetInput(f)
            f = ip.GetOutput()
            ip.Update()

         f = VTKShrinkN(f,n=radix)

         if verb: print j,f.GetDimensions()

      for l in range(j+1):
             f = VTKConvolve(VTKGrowN(f,n=radix),k=k,clip=0)

      ic = vtkImageClip()
      ic.SetOutputWholeExtent(0,d[0]-1,0,d[1]-1,0,0)
      ic.SetInput(f)
      ic.ClipDataOn()
      f = ic.GetOutput()
      ic.Update()

      y = VTKImageToNum(f)
      return y

def VTKLevelErode(x,level=1,k=5,radix=2,pad=0,verb=0):
      x = x.astype(Float32)
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      d = i.GetDimensions()
      if level: 
         level = level
      else: 
         nx = int(log2(d[0]-1))
         ny = int(log2(d[1]-1))
         level = min([nx,ny])
      f = i

      for j in range(level):
         rx,ry,rz = f.GetDimensions()
         tx,ty = rx,ry
         s = radix
         if rx % s or ry % s:
            tx = int(1.*rx/s+1)*s
            ty = int(1.*ry/s+1)*s
            if rx == 1: tx = 1
            if ry == 1: ty = 1
            if verb: print "padding to",tx,ty,rx,ry
            ip = vtkImageMirrorPad()
            ip.SetOutputWholeExtent(0,tx-1,0,ty-1,0,0)
            ip.SetInput(f)
            f = ip.GetOutput()
            ip.Update()

         f = VTKShrinkN(f,n=radix)

         if verb: print j,f.GetDimensions()

      for l in range(j+1):
             f = VTKErode(VTKGrowN(f,n=radix),k=k)

      ic = vtkImageClip()
      ic.SetOutputWholeExtent(0,d[0]-1,0,d[1]-1,0,0)
      ic.SetInput(f)
      ic.ClipDataOn()
      f = ic.GetOutput()
      ic.Update()

      y = VTKImageToNum(f)
      return y

def VTKLevelMedian(x,level=1,k=5,radix=2,pad=0,verb=0):
      x = x.astype(Float32)
      if type(x) == ArrayType: i = NumToVTKImage(x)
      else: i = x
      d = i.GetDimensions()
      if level: 
         level = level
      else: 
         nx = int(log2(d[0]-1))
         ny = int(log2(d[1]-1))
         level = min([nx,ny])
      f = i

      for j in range(level):
         rx,ry,rz = f.GetDimensions()
         tx,ty = rx,ry
         s = radix
         if rx % s or ry % s:
            tx = int(1.*rx/s+1)*s
            ty = int(1.*ry/s+1)*s
            if rx == 1: tx = 1
            if ry == 1: ty = 1
            if verb: print "padding to",tx,ty,rx,ry
            ip = vtkImageMirrorPad()
            ip.SetOutputWholeExtent(0,tx-1,0,ty-1,0,0)
            ip.SetInput(f)
            f = ip.GetOutput()
            ip.Update()

         f = VTKShrinkN(f,n=radix,usemed=1)

         if verb: print j,f.GetDimensions()

      for l in range(j+1):
             f = VTKMedian(VTKGrowN(f,n=radix),n=k,clip=0)

      ic = vtkImageClip()
      ic.SetOutputWholeExtent(0,d[0]-1,0,d[1]-1,0,0)
      ic.SetInput(f)
      ic.ClipDataOn()
      f = ic.GetOutput()
      ic.Update()

      y = VTKImageToNum(f)
      return y

def VTKWave(x,level=0,radix=2,pad=0,wt=None,verb=0):

      if type(x) == type(arange(10)): i = NumToVTKImage(x)
      else: i = x
      if wt:
        if type(wt) == type(arange(10)):
           wt = wt/mean(wt.flat)
           wt = wt.astype(x.typecode())
           iw = NumToVTKImage(wt)
        else: iw = wt
      d = i.GetDimensions()

      if level: 
         level = level
      else: 
         if d[0] >1: nx = int(log2(d[0]-1)/log2(radix))
         else: nx = 99999
         if d[1] >1: ny = int(log2(d[1]-1)/log2(radix))
         else: ny = 99999
         level = min([nx,ny])
      f = i
      if wt:
        fw = iw
#        tw = VTKWave(wt,level=level)
#        atw = asarray([sum(tw[l:]) for l in range(level+1)])

      at = []
      for j in range(level):

         rx,ry,rz = f.GetDimensions()
         tx,ty = rx,ry
         s = radix
         if rx % s or ry % s:
            tx = int(1.*rx/s+1)*s
            ty = int(1.*ry/s+1)*s
            if rx == 1: tx = 1
            if ry == 1: ty = 1
            if verb: print "padding to",tx,ty

            ip = vtkImageMirrorPad()
            ip.SetOutputWholeExtent(0,tx-1,0,ty-1,0,0)
            ip.SetInput(f)
            f = ip.GetOutput()
            ip.Update()

            if wt:
              ip = vtkImageMirrorPad()
              ip.SetOutputWholeExtent(0,tx-1,0,ty-1,0,0)
              ip.SetInput(fw)
              fw = ip.GetOutput()
              ip.Update()

         cf = VTKShrinkN(f,n=radix)
         if wt:
           cfw = VTKShrinkN(fw,n=radix)
           y = VTKImageToNum(cf)
           z = VTKImageToNum(cfw)
           y = y*z
           cf = NumToVTKImage(y)

#           op = vtkImageMathematics()
#           op.SetOperationToMultiply()
#           op.SetInput1(cf)
#           op.SetInput2(cfw)
#           op.Update()
#           cf = op.GetOutput()

         gf = VTKConvolve(VTKGrowN(cf,n=radix),k=5)

         op = vtkImageMathematics()
         op.SetOperationToSubtract()
         op.SetInput1(f)
         op.SetInput2(gf)
         op.Update()
         gf = op.GetOutput()

         for k in range(j): gf = VTKConvolve(VTKGrowN(gf,n=radix),k=5)

         ic = vtkImageClip()
         ic.SetOutputWholeExtent(0,d[0]-1,0,d[1]-1,0,0)
         ic.SetInput(gf)
         ic.ClipDataOn()
         gf = ic.GetOutput()
         ic.Update()
         y = VTKImageToNum(gf)

#         if wt:
#            gfw = VTKConvolve(VTKGrowN(cfw),k=5)
#            for k in range(j): gfw = VTKConvolve(VTKGrowN(gfw),k=5)
#            ic = vtkImageClip()
#            ic.SetOutputWholeExtent(0,d[0]-1,0,d[1]-1,0,0)
#            ic.SetInput(gfw)
#            ic.ClipDataOn()
#            gfw = ic.GetOutput()
#            ic.Update()
#            yw = VTKImageToNum(gfw)
#            y = divz(y,yw)
#            y = y * atw[level]

         at.append(y)

         f = cf
         if wt: fw = cfw

      for k in range(j+1):
         cf = VTKConvolve(VTKGrowN(cf,n=radix),k=5)
         cd = cf.GetDimensions()
         if cd[0] > d[0] or cd[1] > d[1]:
           ic = vtkImageClip()
           ic.SetOutputWholeExtent(0,d[0]-1,0,d[1]-1,0,0)
           ic.SetInput(cf)
           ic.ClipDataOn()
           cf = ic.GetOutput()
           ic.Update()
      y = VTKImageToNum(cf)
      at = concatenate([at,[y]])

      return asarray(at)


def VTKWtWave(x,w,level=0,verb=0):
      d = list(x.shape)
      d.reverse()
      if level: 
         level = level
      else: 
         if d[0] >1: nx = int(log2(d[0]-1))
         else: nx = 99999
         if d[1] >1: ny = int(log2(d[1]-1))
         else: ny = 99999
         level = min([nx,ny])
      f = x
      at = []
      for j in range(level):
         if verb: print "level=",j
         xwj = VTKLevel(x*w,j+1,verb=verb)
         wj = VTKLevel(w,j+1,verb=verb)
         l = divz(xwj,wj).astype(Float32)
         a = (f - l).astype(Float32)
         at.append(a)
         f = l
      at = asarray(at).astype(Float32)
      p = (x - sum(at)).astype(Float32)
      at = concatenate([at,[p]])
      return at


def VTKBoxToActor(x,y,color=(1,1,1),wid=1):
     eacts = []
     for j in range(4):
       edge = vtkLineSource()
       edge.SetPoint1(x[j],y[j],0)
       edge.SetPoint2(x[j+1],y[j+1],0)
       emap = vtkPolyDataMapper()
       emap.SetInput(edge.GetOutput())
       eact = vtkActor()
       eact.SetMapper(emap)
       eact.GetProperty().SetColor(color)
       eact.GetProperty().SetLineWidth(wid)
       eacts.append(eact)
     return eacts

def VTKAtrous(c,lev=None,radix=2,k=5,x=1,y=1,interp=1,mirror=1,wrap=0,verb=0):
     if x: xradix = radix
     else: xradix = 1
     if y: yradix = radix
     else: yradix = 1
     if not lev:
       nx = int(log2(c.shape[0]-1)/log2(xradix))
       ny = int(log2(c.shape[1]-1)/log2(yradix))
       lev = min([nx,ny])
     imb = copy.copy(c)
     imsizes = []
     tot = zeros(c.shape)
     if interp: xs, ys = -0.5*(xradix-1),-0.5*(yradix-1)
     else: xs, ys = 0.0, 0.0
     for p in range(lev):
       imsizes.append( imb.shape)
       imo = VTKConvolve(imb,k=k,numret=1,clip=1)
       imc = VTKShrinkN(imo,xradix,yradix,numret=1,usemean=1)
       imd = VTKGrowN(imc,xradix,yradix,numret=1,xshift=xs,yshift=ys,\
                origsize=imb.shape,interp=interp,mirror=mirror,wrap=wrap)
       imw = (imb - imd).astype(Float32)
       del imd,imo
       if verb: print p,imc.shape
       imb = imc
       for q in range(p):
          imw = VTKGrowN(imw,xradix,yradix,numret=1,xshift=xs,yshift=ys,\
                origsize=imsizes[p-q-1],interp=interp,mirror=mirror,wrap=wrap)
       tot = (tot + imw).astype(Float32)
       if p: uwt = concatenate([uwt,[imw.astype(Float32)]])
       else: uwt = [imw.astype(Float32)]
       del imw
     imw = (c - tot).astype(Float32)
     uwt = concatenate([uwt,[imw.astype(Float32)]])
     return uwt

def VTKAtrousWt(c,wc,lev=None,radix=2,k=5,x=1,y=1,interp=1,mirror=1,wrap=0,verb=0):
     if x: xradix = radix
     else: xradix = 1
     if y: yradix = radix
     else: yradix = 1
     if not lev:
       nx = int(log2(c.shape[0]-1)/log2(xradix))
       ny = int(log2(c.shape[1]-1)/log2(yradix))
       lev = min([nx,ny])
     imb = copy.copy(c)
     wimb = copy.copy(wc)
     imsizes = []
     tot = zeros(c.shape)
     if interp: xs, ys = -0.5*(xradix-1),-0.5*(yradix-1)
     else: xs, ys = 0.0, 0.0
     for p in range(lev):
       imsizes.append( imb.shape)
       imo1 = VTKConvolve(imb*wimb,k=k,numret=1,clip=1)
       imc1 = VTKShrinkN(imo1,xradix,yradix,numret=1,usemean=1)
       imd1 = VTKGrowN(imc1,xradix,yradix,numret=1,xshift=xs,yshift=ys,origsize=imb.shape,interp=interp,mirror=mirror,wrap=wrap)
       imo2 = VTKConvolve(wimb,k=k,numret=1,clip=1)
       imc2 = VTKShrinkN(imo2,xradix,yradix,numret=1,usemean=1)
       imd2 = VTKGrowN(imc2,xradix,yradix,numret=1,xshift=xs,yshift=ys,origsize=imb.shape,interp=interp,mirror=mirror,wrap=wrap)
       imc = VTKDivide(imc1,imc2)
       imd = VTKDivide(imd1,imd2)
       imw = (imb - imd).astype(Float32)
       del imd1,imd2,imo1,imo2,imd,imc1
       if verb: print p,imc.shape
       imb = imc
       wimb = imc2
       for q in range(p):
          imw = VTKGrowN(imw,xradix,yradix,numret=1,xshift=xs,yshift=ys,\
                origsize=imsizes[p-q-1],interp=interp,mirror=mirror,wrap=wrap)
       tot = (tot + imw).astype(Float32)
       if p: uwt = concatenate([uwt,[imw.astype(Float32)]])
       else: uwt = [imw.astype(Float32)]
       del imw
     imw = (c - tot).astype(Float32)
     uwt = concatenate([uwt,[imw.astype(Float32)]])
     return uwt

def VTKLevel(c,lev=None,radix=2,k=5,x=1,y=1,interp=1,mirror=1,wrap=0,constant=None,verb=0,usemean=1,usemed=0,usemax=0,usemin=0,useext=0):
     if len(c.shape) == 1: x,y=1,0
     if x: xradix = radix
     else: xradix = 1
     if y: yradix = radix
     else: yradix = 1
     imb = c.astype(Float32)
     imsizes = []
     tot = zeros(c.shape,Float32)
     if interp and (usemean or usemed or usemax or usemin or useext):
          if x: xs = -0.5*(radix-1)
          else: xs = -2 + (k-1)/2
          if y: ys = -0.5*(radix-1)
          else: ys = -2 + (k-1)/2
     else: xs, ys = 0.0, 0.0
     for p in range(lev):
       imsizes.append( imb.shape)
       imo = VTKConvolve(imb,k=k,numret=1,clip=1,x=x,y=y,mirror=mirror,wrap=wrap,constant=constant)
       if useext:
         usemax = not (p % 2)
         usemin = (p % 2)
       imc = VTKShrinkN(imo,xradix,yradix,numret=1,usemean=usemean,usemed=usemed,usemax=usemax,usemin=usemin,verb=0)
       if verb: print p+1,imc.shape
       imd = VTKGrowN(imc,xradix,yradix,numret=1,xshift=xs,yshift=ys,\
                origsize=imb.shape,interp=interp,mirror=mirror,wrap=wrap or constant != None)
       imw = VTKSubtract(imb,imd)
       del imd,imo
       imb = imc
       for q in range(p):
          imw = VTKGrowN(imw,xradix,yradix,numret=1,xshift=xs,yshift=ys,\
                origsize=imsizes[p-q-1],interp=interp,mirror=mirror,wrap=wrap or constant != None)
       tot = VTKAdd(tot,imw)
       del imw
     imw = VTKSubtract(c.astype(tot.typecode()),tot)
     del imb, imc, tot
     return imw

def VTKAtrousReduce(c,lev=None,radix=2,k=3,x=1,y=1,interp=1,mirror=1,wrap=0,verb=0):
     if x: xradix = radix
     else: xradix = 1
     if y: yradix = radix
     else: yradix = 1
     if not lev:
       nx = int(log2(c.shape[0]-1)/log2(xradix))
       ny = int(log2(c.shape[1]-1)/log2(yradix))
       lev = min([nx,ny])
     imb = copy.copy(c)
     imsizes = []
     tot = zeros(c.shape)
     if interp: xs, ys = -0.5*(xradix-1),-0.5*(yradix-1)
     else: xs, ys = 0.0, 0.0
     for p in range(lev+1):
       imsizes.append( imb.shape)
       imo = VTKConvolve(imb,k=k,numret=1,clip=1)
       imc = VTKShrinkN(imo,xradix,yradix,numret=1,usemean=1)
       imd = VTKGrowN(imc,xradix,yradix,numret=1,xshift=xs,yshift=ys,\
                origsize=imb.shape,interp=interp,mirror=mirror,wrap=wrap)
       if p < lev: imw = (imb - imd).astype(Float32)
       else: imw = imb
       del imd,imo
       if verb: print p,imc.shape
       imb = imc
       if p: uwt += [imw.astype(Float32)]
       else: uwt = [imw.astype(Float32)]
     del imb, imc
     return uwt,imsizes

def VTKAtrousRescale(imw,p,imsizes,radix=2,k=3,x=1,y=1,interp=1,mirror=1,wrap=0,verb=0):
    if x: xradix = radix
    else: xradix = 1
    if y: yradix = radix
    else: yradix = 1
    if interp: xs, ys = -0.5*(xradix-1),-0.5*(yradix-1)
    else: xs, ys = 0.0, 0.0
    for q in range(p):
       imw = VTKGrowN(imw,xradix,yradix,numret=1,xshift=xs,yshift=ys,\
             origsize=imsizes[p-q-1],interp=interp,mirror=mirror,wrap=wrap)
       if verb: print q,imw.shape
    return imw

def VTKAniModel(image,bad=2.5,th=3.5,mx=25,my=3,fac=0.2,iter=100,spec=1,mx2=11,my2=7):
   mimage = VTKMedian(image,n=mx,m=1,numret=1)
   mimage = VTKMedian(mimage,n=1,m=my,numret=1)
   if spec:
     mimage = VTKGauss(mimage,sigmax=4,sigmay=2,numret=1)
   else:
     mimage = VTKGauss(mimage,sigmax=4,sigmay=4,numret=1)
   rimage = image - mimage
   if spec: simage = 1.49*VTKMedian(abs(rimage),mx,1,numret=1)
   else: simage = 1.49*VTKMedian(abs(rimage),mx,my,numret=1)
   eimage = divz(rimage,simage)
   good = between(eimage,-bad,bad)
   vimage1 = VTKGauss(image*good,sigmax=mx/2,sigmay=my/2,numret=1)
   vimage2 = VTKGauss(good.astype(Float32),sigmax=mx/2,sigmay=my/2,numret=1)
   vimage =  divz( vimage1, vimage2)
   wt = good * getsqrt(vimage)
   sol = linear_least_squares(transpose([mimage.flat,vimage.flat])*wt.flat[::,NewAxis],image.flat*wt.flat)
   oimage = sol[0][0]*mimage+sol[0][1]*vimage
   for j in range(1):
      nimage = image - oimage
      timage = 1.49*VTKMedian(abs(nimage),mx2,my2,numret=1)
      nimage = divz(nimage,timage)
      good = between(nimage,-th,th)
      uimage = where(good,nimage,0)
      aimage = VTKAnisotropicDiffusion(uimage,th=th,fac=fac,iter=iter,edges=1,faces=0,corners=0,gmag=1)
      oimage = aimage*timage + oimage
   return oimage

